@@ -1,5 +1,6 @@
 // 
-//     Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//     Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//     Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -138,13 +139,14 @@
 /// a hint and the GUI /is/ allowed to further process and alter the information
 /// in any way.
 /// 
-/// As for the integer/float discussion: I used rect (floats) because all
+/// As for the integer/float discussion: I used SWFRect (floats) because all
 /// the bounds calculation involves floats anyway and so it's probably
 /// faster than converting between ints and floats all the way.
 
 
 #include "dsodefs.h" // for DSOEXPORT
 
+#include "FileTypes.h"
 #include "gnash.h" // Quality
 #include "DisplayObject.h"
 #include "Range2d.h"
@@ -153,12 +155,12 @@
 // Forward declarations.
 namespace gnash {
     class BitmapInfo;
-    class rect;
+    class SWFRect;
     class rgba;
     class SWFMatrix;
     class cxform;
     class fill_style;
-    class line_style;
+    class LineStyle;
 
     namespace SWF {
         class ShapeRecord;
@@ -190,11 +192,9 @@ public:
 /// Base class for render handlers.
 //
 /// You must define a subclass of Renderer, and pass an
-/// instance to set_Renderer() *before* any SWF parsing begins.
+/// instance to the core (RunResources) *before* any SWF parsing begins.
 ///
 /// For more info see page \ref Renderer_intro.
-/// 
-///
 class DSOEXPORT Renderer
 {
 public:
@@ -250,12 +250,12 @@ public:
     ///   renderer has to find the correct scaling for the video inside the
     ///   bounds.                                
     ///
-    /// @param bounds The minX/minY fields of this rect are always zero. 
+    /// @param bounds The minX/minY fields of this SWFRect are always zero. 
     ///   The width and height determine the size of the Flash video instance
     ///   on the stage (in TWIPS) prior to SWFMatrix transformations.         
     ///
     virtual void drawVideoFrame(GnashImage* frame, const SWFMatrix* mat,
-            const rect* bounds, bool smooth) = 0;
+            const SWFRect* bounds, bool smooth) = 0;
 
     /// Draw a line-strip directly, using a thin, solid line.
     //
@@ -311,6 +311,24 @@ public:
            const SWFMatrix& mat) = 0;
 
 
+    /// Draw the current rendering buffer to an image file.
+    //
+    /// Although this can be done at any time during the rendering cycle
+    /// without harmful side effects, it's advisable only to do it when 
+    /// between advance() calls, when the frame is fully renderered.
+    //
+    /// @param io       The IOChannel to write to.
+    /// @param type     The type of image output required (PNG, JPEG, GIF).
+    ///                 Note that not all FileTypes are images: rendering
+    ///                 to an FLV will not work.
+    virtual void renderToImage(boost::shared_ptr<IOChannel> /*io*/,
+        FileType /*type*/) const {
+
+        log_debug(_("Rendering to image not implemented for this "
+            "renderer"));
+    }
+        
+
     /// ==================================================================
     /// Prepare drawing area and other utilities
     /// ==================================================================
@@ -328,7 +346,7 @@ public:
     ///
     /// For more info see page \ref region_update.
     ///
-    virtual void set_invalidated_region(const rect& /*bounds*/) {}
+    virtual void set_invalidated_region(const SWFRect& /*bounds*/) {}
 
     virtual void set_invalidated_regions(const InvalidatedRanges& /*ranges*/)
     {        
@@ -378,7 +396,7 @@ public:
     /// ==================================================================
     
     /// Converts world coordinates to pixel coordinates
-    virtual geometry::Range2d<int> world_to_pixel(const rect& worldbounds) = 0;
+    virtual geometry::Range2d<int> world_to_pixel(const SWFRect& worldbounds) = 0;
     
     /// Converts pixel coordinates to world coordinates (TWIPS)
     virtual point pixel_to_world(int x, int y) = 0;
@@ -404,7 +422,7 @@ public:
 	// here, so we cast it ourselves to get rid of the warning
 	// message. Note that in both cases this rounds the float to
 	// an integer by dropping the decimal part.
-        return world_to_pixel(rect(static_cast<int>(worldbounds.getMinX()),
+        return world_to_pixel(SWFRect(static_cast<int>(worldbounds.getMinX()),
 				   static_cast<int>(worldbounds.getMinY()),
 				   static_cast<int>(worldbounds.getMaxX()),
 				   static_cast<int>(worldbounds.getMaxY())));
@@ -422,12 +440,12 @@ public:
     /// handler for better performance.
     /// 'bounds' contains TWIPS coordinates.
     ///
-    /// TODO: Take a Range2d<T> rather then a gnash::rect ?
+    /// TODO: Take a Range2d<T> rather then a gnash::SWFRect ?
     ///             Would T==int be good ? TWIPS as integer types ?
     ///
     /// See also gnash::renderer::bounds_in_clipping_area
     ///
-    virtual bool bounds_in_clipping_area(const rect& bounds) {
+    virtual bool bounds_in_clipping_area(const SWFRect& bounds) {
         return bounds_in_clipping_area(bounds.getRange());
     }
     
@@ -461,8 +479,7 @@ public:
     /// the nearest pixel is returned.
     /// The function returns false when the coordinates are outside the 
     /// main frame buffer.
-    virtual bool getPixel(rgba& /*color_return*/, int /*x*/, int /*y*/) const
-    {
+    virtual bool getPixel(rgba& /*color_return*/, int /*x*/, int /*y*/) const {
 
         log_debug("getPixel() not implemented for this renderer");
         abort();        
@@ -541,8 +558,7 @@ public:
     /// @param width stage width in pixels
     ///
     /// @param height stage height in pixels
-    virtual bool initTestBuffer(unsigned /*width*/, unsigned /*height*/)
-    {
+    virtual bool initTestBuffer(unsigned /*width*/, unsigned /*height*/) {
         return false;
     }
 
@@ -553,9 +569,7 @@ public:
     /// TODO: this should be a pure abstract function, just don't want
     ///     to scan ogl and cairo backend for an implementation *now*
     ///     but would be needed for automated testing... Quinn, can you help ?
-    ///
-    virtual unsigned int getBitsPerPixel() const
-    {
+    virtual unsigned int getBitsPerPixel() const {
         return 0;
     }
     
@@ -563,20 +577,14 @@ public:
 
 protected:
 
-
-    /// Dummy, neutral color transformation (do not change!!)
-    const cxform m_neutral_cxform;
-
     /// Kept in parallel with movie_root's setting.
     Quality _quality;
 
-}; // class Renderer
-
-
+}; 
 
 } // namespace gnash
 
-#endif // RENDER_HANDLER_H
+#endif 
 
 
 // Local Variables:
