@@ -15,38 +15,26 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef __DBUS_PLUGIN_H__
-#define __DBUS_PLUGIN_H__
+#ifndef GNASH_DBUS_PLUGIN_H
+#define GNASH_DBUS_PLUGIN_H
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
 #endif
 
-#include <memory> // for auto_ptr
 #include "as_object.h"
 
 namespace gnash
 {
 
-class Dbus {
-public:
-    Dbus();
-    ~Dbus();
-    void setSocketName(const char *sock);
-private:
-    const char *_name;
-};
-
 extern "C" {
-    void dbus_class_init(as_object &obj);  
-    /// Return an  instance
-}
 
-std::auto_ptr<as_object> init_dbus_instance();
+void dbus_class_init(as_object &obj);  
+
+}
 
 } // end of gnash namespace
 
-// __DBUS_PLUGIN_H__
 #endif
 
 // Local Variables:
