@@ -1,6 +1,7 @@
 // kde.cpp:  K Development Environment top level window, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -123,9 +124,9 @@ Kde4Gui::run()
 
 
 bool
-Kde4Gui::createWindow(const char* windowtitle, int width, int height)
+Kde4Gui::createWindow(const char* windowtitle, int width, int height,
+                     int xPosition, int yPosition)
 {
-
     _width = width;
     _height = height;
 
@@ -148,6 +149,8 @@ Kde4Gui::createWindow(const char* windowtitle, int width, int height)
     } else {
         // The QMainWindow takes ownership of the widgets.
         _window->setCentralWidget(_embedWidget);
+        if (xPosition > -1 || yPosition > -1)
+            _window->move(xPosition, yPosition);
         _window->show();
     }
 
@@ -162,16 +165,25 @@ Kde4Gui::createWindow(const char* windowtitle, int width, int height)
     _validbounds.setTo(0, 0, _width, _height);
     _glue.initBuffer(_width, _height);
     
-    log_debug("Setting renderer");
+    log_debug(_("Setting renderer"));
 
     _runResources.setRenderer(_renderer);
     
-    log_debug("Set renderer");
+    log_debug(_("Set renderer"));
    
     return true;
 }
 
 
+void
+Kde4Gui::resizeWindow(int width, int height)
+{
+    _width = width;
+    _height = height;
+
+    _drawingWidget->setMinimumSize(_width, _height);
+}
+
 void
 Kde4Gui::popupMenu(const QPoint& point)
 {
@@ -241,7 +253,9 @@ Kde4Gui::setInvalidatedRegions(const InvalidatedRanges& ranges)
 void
 Kde4Gui::setTimeout(unsigned int timeout)
 {
-    QTimer::singleShot(timeout, _application.get(), SLOT(quit()));
+    // This must go through Gui::quit() to make sure screenshots are
+    // handled if necessary.
+    QTimer::singleShot(timeout, _drawingWidget, SLOT(quit()));
 }
 
 
@@ -492,7 +506,7 @@ Kde4Gui::showPreferences()
 
 
 void
-Kde4Gui::quit()
+Kde4Gui::quitUI()
 {
     _application->quit();
 }
@@ -508,9 +522,10 @@ Kde4Gui::setupActions()
                      _drawingWidget, SLOT(properties()));
 
     quitAction = new QAction(_q("Quit Gnash"), _window.get());
-    // This is connected directly to the QApplication's quit() slot
+    // This must go through Gui::quit() to make sure we don't exit
+    // before doing whatever the Gui wants to do on exit.
     _drawingWidget->connect(quitAction, SIGNAL(triggered()),
-                     _application.get(), SLOT(quit()));
+                     _drawingWidget, SLOT(quit()));
 
     // Edit Menu actions
     preferencesAction = new QAction(_q("Preferences"), _window.get());
@@ -795,6 +810,12 @@ DrawingWidget::refresh()
     _gui.refreshView();
 }
 
+void
+DrawingWidget::quit()
+{
+    _gui.quit();
+}
+
 void
 DrawingWidget::fullscreen(bool isFull)
 {
