@@ -1,6 +1,7 @@
 // Player.cpp:  Top level SWF player, for gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,7 +29,7 @@
 #include "gui.h"
 #include "NullGui.h"
 
-#include "gnash.h" // still needed ?
+#include "MovieFactory.h"
 #include "movie_definition.h"
 #include "sound_handler.h" // for set_sound_handler and create_sound_handler_*
 #include "MovieClip.h" // for setting FlashVars
@@ -36,6 +37,8 @@
 #include "Player.h"
 #include "StreamProvider.h"
 
+#include "swf/TagLoadersTable.h"
+#include "swf/DefaultTagLoaders.h"
 #include "NamingPolicy.h"
 #include "StringPredicates.h"
 #include "URL.h"
@@ -56,6 +59,7 @@
 #include <iostream>
 #include <sstream>
 #include <iomanip>
+#include <boost/lexical_cast.hpp>
 
 using namespace gnash;
 
@@ -88,6 +92,8 @@ Player::Player()
     _delay(0),
     _width(0),
     _height(0),
+    _xPosition(-1),
+    _yPosition(-1),
     _windowID(0),
     _doLoop(true),
     _doRender(true),
@@ -187,6 +193,19 @@ Player::init_sound()
             // are playing on the stage
             _soundHandler->attach_aux_streamer(silentStream, (void*) this);
         }
+#elif defined(SOUND_AHI)
+        try {
+            _soundHandler.reset(sound::create_sound_handler_aos4(_audioDump));
+        } catch (SoundException& ex) {
+            log_error(_("Could not create sound handler: %s."
+                " Will continue w/out sound."), ex.what());
+        }
+        if (! _audioDump.empty()) {
+            // add a silent stream to the audio pool so that our output file
+            // is homogenous;  we actually want silent wave data when no sounds
+            // are playing on the stage
+            _soundHandler->attach_aux_streamer(silentStream, (void*) this);
+        }
 #elif defined(SOUND_GST)
         _soundHandler.reset(media::create_sound_handler_gst());
 #else
@@ -215,40 +234,33 @@ Player::init_media()
 void
 Player::init_gui()
 {
-    if ( _doRender )
-    {
-        _gui = getGui(); 
-    }
-    else
-    {
-        _gui.reset(new NullGui(_doLoop));
-    }
+    if (_doRender) _gui = getGui(); 
+    else _gui.reset(new NullGui(_doLoop, *_runResources));
 
     _gui->setMaxAdvances(_maxAdvances);
 
 #ifdef GNASH_FPS_DEBUG
-    if ( _fpsDebugTime )
-    {
-        log_debug(_("Activating FPS debugging every %g seconds"), _fpsDebugTime);
+    if (_fpsDebugTime) {
+        log_debug(_("Activating FPS debugging every %g seconds"),
+                _fpsDebugTime);
         _gui->setFpsTimerInterval(_fpsDebugTime);
     }
-#endif // def GNASH_FPS_DEBUG
+#endif
 }
 
 boost::intrusive_ptr<movie_definition>
 Player::load_movie()
 {
-    /// The RunInfo must be initialized by this point to provide resources
+    /// The RunResources must be initialized by this point to provide resources
     /// for parsing.
-    assert(_runInfo.get());
+    assert(_runResources.get());
 
     boost::intrusive_ptr<gnash::movie_definition> md;
 
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
     URL vurl(_url);
 
-    if ( vurl.protocol() == "file" )
-    {
+    if (vurl.protocol() == "file") {
         const std::string& path = vurl.path();
         size_t lastSlash = path.find_last_of('/');
         std::string dir = path.substr(0, lastSlash+1);
@@ -261,7 +273,7 @@ Player::load_movie()
         {
             std::auto_ptr<IOChannel> in (
                     noseek_fd_adapter::make_stream(fileno(stdin)));
-            md = gnash::create_movie(in, _url, *_runInfo, false);
+            md = MovieFactory::makeMovie(in, _url, *_runResources, false);
         }
         else
         {
@@ -272,7 +284,7 @@ Player::load_movie()
                 // We'll need to allow load of the file, no matter virtual url
                 // specified...
                 // This is kind of hackish, cleaner would be adding an argument
-                // to create_library_movie to skip the security checking phase.
+                // to createMovie to skip the security checking phase.
                 // NOTE that if we fail to allow this load, the konqueror plugin
                 // would not be able to load anything
                 //
@@ -281,7 +293,7 @@ Player::load_movie()
             }
 
             // _url should be always set at this point...
-            md = gnash::create_library_movie(url, *_runInfo, _url.c_str(),
+            md = MovieFactory::makeMovie(url, *_runResources, _url.c_str(),
                     false);
         }
     } catch (const GnashException& er) {
@@ -310,7 +322,6 @@ Player::run(int argc, char* argv[], const std::string& infile,
     init_logfile();
     init_media();
     init_sound();
-    init_gui();
    
     // gnash.cpp should check that a filename is supplied.
     assert (!infile.empty());
@@ -318,107 +329,86 @@ Player::run(int argc, char* argv[], const std::string& infile,
     _infile = infile;
 
     // Work out base url
-    if ( _baseurl.empty() )
-    {
-        if (! url.empty() ) _baseurl = url;
-        else if ( infile == "-" ) _baseurl = URL("./").str();
+    if (_baseurl.empty()) {
+        if (!url.empty()) _baseurl = url;
+        else if (infile == "-") _baseurl = URL("./").str();
         else _baseurl = infile;
     }
 
     // Set _root._url (either explicit of from infile)
-    if (! url.empty() ) {
-        _url = url;
-    }  else {
-        _url = infile;
-    }
+    if (!url.empty()) _url = url;
+    else _url = infile;
 
+    // Parse player parameters. These are not passed to the SWF, but rather
+    // control stage properties etc.
+    Params::const_iterator it = params.find("base");
+    const URL baseURL = (it == params.end()) ? _baseurl :
+                                               URL(it->second, _baseurl);
 
-    // Initialize gui (we need argc/argv for this)
-    // note that this will also initialize the renderer
-    // which is *required* during movie loading
-    if ( ! _gui->init(argc, &argv) )
-    {
-        std::cerr << "Could not initialize gui." << std::endl;
-        return EXIT_FAILURE;
-    }
+    /// The RunResources should be populated before parsing.
+    _runResources.reset(new RunResources(baseURL.str()));
+    _runResources->setSoundHandler(_soundHandler);
 
-    // Parse querystring (before FlashVars, see testsuite/misc-ming.all/FlashVarsTest*)
-    setFlashVars(URL(_url).querystring());
-
-    // These flags are here so we can construct
-    // the correct URL for base url later.
-    // If the URL class was not immutable we could do something smarter...
-    bool hasOverriddenBaseUrl=false;
-    std::string overriddenBaseUrl;
-
-    // Parse parameters
-    StringNoCaseEqual noCaseCompare;
-    for ( std::map<std::string,std::string>::const_iterator it=params.begin(),
-        itEnd=params.end(); it != itEnd; ++it)
-    {
-        if ( noCaseCompare(it->first, "flashvars") )
-        {
-            setFlashVars(it->second);
-            continue;
-        }
-
-        if ( noCaseCompare(it->first, "base") )
-        {
-            hasOverriddenBaseUrl=true;
-            overriddenBaseUrl=it->second;
-            continue;
-        }
-    }
-
-    URL baseURL = hasOverriddenBaseUrl ? URL(overriddenBaseUrl, URL(_baseurl))
-                                       : URL(_baseurl);
-
-    /// The RunInfo should be populated before parsing.
-    _runInfo.reset(new RunInfo(baseURL.str()));
-    _runInfo->setSoundHandler(_soundHandler);
+    boost::shared_ptr<SWF::TagLoadersTable> loaders(new SWF::TagLoadersTable());
+    addDefaultLoaders(*loaders);
+    _runResources->setTagLoaders(loaders);
 
     std::auto_ptr<NamingPolicy> np(new IncrementalRename(_baseurl));
     boost::shared_ptr<StreamProvider> sp(new StreamProvider(np));
 
-    _runInfo->setStreamProvider(sp);
+    _runResources->setStreamProvider(sp);
+    
+    init_gui();
 
-    // Load the actual movie.
-    _movieDef = load_movie();
-    if ( ! _movieDef )
-    {
+    // Initialize gui (we need argc/argv for this)
+    // note that this will also initialize the renderer
+    // which is *required* during movie loading
+    if (!_gui->init(argc, &argv)) {
+        std::cerr << "Could not initialize gui." << std::endl;
         return EXIT_FAILURE;
     }
 
+    // Parse querystring (before FlashVars, see
+    // testsuite/misc-ming.all/FlashVarsTest*)
+    setFlashVars(URL(_url).querystring());
+    
+    // Add FlashVars.
+    Params::const_iterator fv = params.find("flashvars");
+    if (fv != params.end()) setFlashVars(fv->second);
+
+    // Load the actual movie.
+    _movieDef = load_movie();
+    if (!_movieDef) {
+        return EXIT_FAILURE;
+    }
 
     // Get info about the width & height of the movie.
     int movie_width = static_cast<int>(_movieDef->get_width_pixels());
     int movie_height = static_cast<int>(_movieDef->get_height_pixels());
 
     if (! _width) {
-      _width = static_cast<size_t>(movie_width * _scale);
+        _width = static_cast<size_t>(movie_width * _scale);
     }
     if (! _height) {
-      _height = static_cast<size_t>(movie_height * _scale);
+        _height = static_cast<size_t>(movie_height * _scale);
     }
 
-    if ( ! _width || ! _height )
-    {
+    if (! _width || ! _height) {
         log_debug(_("Input movie has collapsed dimensions "
                     "%d/%d. Setting to 1/1 and going on."),
                      _width, _height);
-        if ( ! _width ) _width = 1;
-        if ( ! _height ) _height = 1;
+        if (!_width) _width = 1;
+        if (!_height) _height = 1;
     }
 
     // Register movie definition before creating the window
     _gui->setMovieDefinition(_movieDef.get());
 
     // Now that we know about movie size, create gui window.
-    _gui->createWindow(_url.c_str(), _width, _height);
-
-    SystemClock clock; // use system clock here...
-    movie_root root(*_movieDef, clock, *_runInfo);
+    _gui->createWindow(_url.c_str(), _width, _height, _xPosition, _yPosition);
 
+    movie_root root(*_movieDef, _gui->getClock(), *_runResources);
+    
     _callbacksHandler.reset(new CallbacksHandler(*_gui, *this)); 
     
     // Register Player to receive events from the core (Mouse, Stage,
@@ -432,7 +422,7 @@ Player::run(int argc, char* argv[], const std::string& infile,
     if ( _hostfd != -1 ) root.setHostFD(_hostfd);
 
     _gui->setStage(&root);
-
+    
     // When startStopped is true, stop here after the stage has been 
     // registered, but before the movie has started. Initial loading
     // and VM initialization have been done by this stage, but not
@@ -440,10 +430,8 @@ Player::run(int argc, char* argv[], const std::string& infile,
     // the Gui accesses movie_root to get the sound_handler, but also
     // because the gui window should be properly set up by this point.
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
-    if ( rcfile.startStopped() )
-    {
-        _gui->stop();
-    }
+
+    if (rcfile.startStopped()) _gui->stop();
 
     // Start loader thread
     // NOTE: the loader thread might (in IMPORT tag parsing)
@@ -456,9 +444,8 @@ Player::run(int argc, char* argv[], const std::string& infile,
     _movieDef->completeLoad();
 
     if (! _delay) {
-      //float movie_fps = _movieDef->get_frame_rate();
-      //_delay = static_cast<unsigned int>(1000 / movie_fps) ; // milliseconds per frame
-      _delay = 10; // 10ms per heart beat
+        // 10ms per heart beat
+        _delay = 10; 
     }
     _gui->setInterval(_delay);
 
@@ -474,6 +461,62 @@ Player::run(int argc, char* argv[], const std::string& infile,
         _gui->hideMenu();
     }
     
+    // Now handle stage alignment and scale mode. This should be done after
+    // the GUI is created, after its stage member is set, and after the
+    // interface callbacks are registered.
+    it = params.find("salign");
+    if (it != params.end()) {
+        log_debug("Setting align");
+        const short align = stringToStageAlign(it->second);
+        root.setStageAlignment(align);
+    }
+
+    it = params.find("scale");
+    if (it != params.end()) {
+		
+        StringNoCaseEqual noCaseCompare;
+        const std::string& str = it->second;
+		
+        movie_root::ScaleMode mode = movie_root::showAll;
+
+		if (noCaseCompare(str, "noScale")) mode = movie_root::noScale;
+		else if (noCaseCompare(str, "exactFit")) mode = movie_root::exactFit;
+		else if (noCaseCompare(str, "noBorder")) mode = movie_root::noBorder;
+
+        log_debug("Setting scale mode");
+	    root.setStageScaleMode(mode);
+    }
+
+    // Set up screenshots. 
+    if (!_screenshots.empty()) {
+
+        std::istringstream is(_screenshots);
+        std::string arg;
+
+        bool last = false;
+        ScreenShotter::FrameList v;
+
+        while (std::getline(is, arg, ',')) {
+            if (arg == "last") last = true;
+            else try {
+                const size_t frame = boost::lexical_cast<size_t>(arg);
+                v.push_back(frame);
+            }
+            catch (boost::bad_lexical_cast&) {}
+        }
+
+        // Use default if filename is empty.
+        if (_screenshotFile.empty()) {
+            URL url(_runResources->baseURL());
+            std::string::size_type p = url.path().rfind('/');
+            const std::string& name = (p == std::string::npos) ? url.path() :
+                url.path().substr(p + 1);
+            _screenshotFile = "screenshot-" + name + "-%f";
+        }
+
+        _gui->requestScreenShots(v, last, _screenshotFile);
+    }
+    
     _gui->run();
 
     log_debug("Main loop ended, cleaning up");
@@ -500,6 +543,8 @@ Player::CallbacksHandler::yesNo(const std::string& query)
 std::string
 Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
 {
+    StringNoCaseEqual noCaseCompare;
+	
     if (event == "Mouse.hide") {
         return _gui.showMouse(false) ? "true" : "false";
     }
@@ -521,7 +566,31 @@ Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
         _gui.updateStageMatrix();
         return "";
     }
-    
+
+    if (event == "Stage.showMenu")
+    {
+        if (noCaseCompare(arg, "true")) _gui.showMenu(true);
+        else if (noCaseCompare(arg, "false")) _gui.showMenu(false);
+        return "";
+    }
+
+    if (event == "Stage.resize")
+    {
+        if ( _gui.isPlugin() ) {
+            log_debug("Player doing nothing on Stage.resize as we're a plugin");
+            return "";
+        }
+
+        // arg contains WIDTHxHEIGHT
+        log_debug("Player got Stage.resize(%s) message", arg);
+        int width, height;
+        sscanf(arg.c_str(), "%dx%d", &width, &height);
+        _gui.resizeWindow(width, height);
+
+        return "";
+    }
+
+
     if (event == "System.capabilities.screenResolutionX")
     {
         std::ostringstream ss;
@@ -694,42 +763,46 @@ std::auto_ptr<Gui>
 Player::getGui()
 {
 #ifdef GUI_GTK
-    return createGTKGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createGTKGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_KDE3
-    return createKDEGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createKDEGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_KDE4
-    return createKDE4Gui(_windowID, _scale, _doLoop, _bitDepth);
+    return createKDE4Gui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_SDL
-    return createSDLGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createSDLGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_AQUA
-    return createAQUAGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createAQUAGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_RISCOS
-    return createRISCOSGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createRISCOSGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_FLTK
-    return createFLTKGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createFLTKGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_FB
-    return createFBGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createFBGui(_windowID, _scale, _doLoop, *_runResources);
+#endif
+
+#ifdef GUI_AOS4
+    return createAOS4Gui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_DUMP
-    return createDumpGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createDumpGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
-    return std::auto_ptr<Gui>(new NullGui(_doLoop));
+    return std::auto_ptr<Gui>(new NullGui(_doLoop, *_runResources));
 }
 
 Player::~Player()
