@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -102,6 +103,12 @@ public:
 	void setHeight(size_t h) { _height = h; }
 	size_t getHeight() { return _height; }
 
+	void setXPosition(int xPos) { _xPosition = xPos; }
+	size_t getXPosition() { return _xPosition; }
+
+	void setYPosition(int yPos) { _yPosition = yPos; }
+	size_t getYPosition() { return _yPosition; }
+
 	void setWindowId(unsigned long x) { _windowID = x; }
 
 	void setDoLoop(bool b) { _doLoop = b; }
@@ -151,6 +158,23 @@ public:
         _audioDump = filespec;
     }
 	
+    /// This should be a comma-separated list of frames.
+    //
+    /// Only numbers and the word "last" are valid.
+    //
+    /// We parse the string here rather than in gnash.cpp to avoid making
+    /// the interface to Player more complicated than it is already.
+    void setScreenShots(const std::string& screenshots) {
+        _screenshots = screenshots;
+    }
+
+    /// Set the filename for screenshot output.
+    //
+    /// A %f in the filename will be replaced with the frame number.
+    void setScreenShotFile(const std::string& file) {
+        _screenshotFile = file;
+    }
+
 private:
 
 	class CallbacksHandler : public movie_root::AbstractIfaceCallback,
@@ -220,8 +244,10 @@ private:
 
 	void setFlashVars(const std::string& varstr);
 
+    typedef std::map<std::string, std::string, StringNoCaseLessThan> Params;
+
 	// Movie parameters (for -P)
-	std::map<std::string, std::string> params;
+	Params params;
 
 	unsigned int _bitDepth;
 
@@ -233,6 +259,10 @@ private:
 	size_t _width;
 
 	size_t _height;
+	
+	int _xPosition;
+
+	int _yPosition;
 
 	unsigned long _windowID;
 
@@ -246,7 +276,6 @@ private:
 
 	std::string _baseurl;
 
-
     /// Initialization / destruction order is important here.
     //
     /// some sound_samples are destroyed in the dtor of SWFMovieDefinition,
@@ -303,9 +332,19 @@ private:
 	bool _startFullscreen;
 	bool _hideMenu;
 
-    // The filename to use for dumping audio.
+    /// The filename to use for dumping audio.
     std::string _audioDump;
 
+    /// A comma-separated list of frames to output as screenshots.
+    //
+    /// If empty, no screenshots are required.
+    std::string _screenshots;
+
+    /// The filename to save screenshots to.
+    //
+    /// If empty, a default is used.
+    std::string _screenshotFile;
+
 };
 
  
