@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -38,8 +39,9 @@
 #include "gnash.h"
 #include "gui.h"
 #include "VM.h"
+#include "RunResources.h"
 
-#include "render_handler.h"
+#include "Renderer.h"
 
 using namespace std;
 using namespace fltk;
@@ -50,9 +52,9 @@ namespace gnash
 {
 
 
-FltkGui::FltkGui(unsigned long xid, float scale, bool loop, unsigned int depth)
+FltkGui::FltkGui(unsigned long xid, float scale, bool loop, RunResources& r)
   : Window(0, 0),
-    Gui(xid, scale, loop, depth),
+    Gui(xid, scale, loop, r),
     _menu_height(_xid ? 0 : 20)
 {
 }
@@ -115,10 +117,8 @@ FltkGui::handle(int event)
         if (!_xid && event_y() < static_cast<int>(_menu_height)) {
           return Window::handle(event);
         }
-        int x = static_cast<int>(event_x() / _xscale);
-        int y = static_cast<int>((event_y() - _menu_height) / _yscale);
-        notify_mouse_moved(x, y);
-        return Window::handle(event);;
+        notify_mouse_moved(event_x(), event_y()-_menu_height);
+        return Window::handle(event);
       }
       case SHORTCUT:
       case KEY:
@@ -171,18 +171,18 @@ FltkGui::handleKey(unsigned key)
 #endif
     };
 
-    int modifier = gnash::key::MOD_NONE;
+    int modifier = gnash::key::GNASH_MOD_NONE;
 
     unsigned long state = event_state();
 
     if (state & SHIFT) { 
-        modifier = modifier | gnash::key::MOD_SHIFT;
+        modifier = modifier | gnash::key::GNASH_MOD_SHIFT;
     }
     if (state & CTRL) {
-        modifier = modifier | gnash::key::MOD_CONTROL;
+        modifier = modifier | gnash::key::GNASH_MOD_CONTROL;
     }
     if (state & ALT) {
-        modifier = modifier | gnash::key::MOD_ALT;
+        modifier = modifier | gnash::key::GNASH_MOD_ALT;
     }
 
     for (int i = 0; table[i].fltkKey; i++) {
@@ -230,7 +230,8 @@ FltkGui::create()
 }
 
 bool
-FltkGui::createWindow(const char* title, int width, int height)
+FltkGui::createWindow(const char* title, int width, int height,
+                      int xPosition, int yPosition)
 {
     resize(width, _menu_height + height);
 
@@ -256,9 +257,9 @@ FltkGui::createWindow(const char* title, int width, int height)
     createMenu();
     end();
 
-    _renderer = _glue->createRenderHandler();
-    if ( ! _renderer ) return false;
-    set_render_handler(_renderer);
+    _renderer.reset(_glue->createRenderHandler());
+    if (!_renderer.get()) return false;
+    _runResources.setRenderer(_renderer);
 
     _glue->initBuffer(width, height);
 
@@ -314,71 +315,42 @@ static void
 fltk_menu_quit(Widget*, void* ptr)
 {
     FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_quit();
+    gui->quit();
 }
 
 static void
 fltk_menu_play(Widget*, void* ptr)
 {
     FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_play();
+    gui->play();
 }
 
 static void
 fltk_menu_pause(Widget*, void* ptr)
 {
     FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_pause();
+    gui->pause();
 }
 
 static void
 fltk_menu_stop(Widget*, void* ptr)
 {
     FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_stop();
+    gui->stop();
 }
 
 static void
 fltk_menu_restart(Widget*, void* ptr)
 {
     FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_restart();
-}
-
-static void
-fltk_menu_step_forward(Widget*, void* ptr)
-{
-    FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_step_forward();
-}
-
-
-static void
-fltk_menu_step_backward(Widget*, void* ptr)
-{
-    FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_step_backward();
-}
-
-static void
-fltk_menu_jump_forward(Widget*, void* ptr)
-{
-    FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_jump_forward();
-}
-
-static void
-fltk_menu_jump_backward(Widget*, void* ptr)
-{
-    FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_jump_backward();
+    gui->restart();
 }
 
 static void
 fltk_menu_toggle_sound(Widget*, void* ptr)
 {
     FltkGui* gui = static_cast<FltkGui*>(ptr);
-    gui->menu_toggle_sound();
+    gui->toggleSound();
 }
 
 void
@@ -410,10 +382,6 @@ FltkGui::addMenuItems()
     new Item("Pause Movie",             0, fltk_menu_pause, this);
     new Item("Stop Movie",              0, fltk_menu_stop, this);
     new Item("Restart Movie",           0, fltk_menu_restart, this);
-    new Item("Step Forward Frame",      0, fltk_menu_step_forward, this);
-    new Item("Step Backward Frame",     0, fltk_menu_step_backward, this);
-    new Item("Jump Forward 10 Frames",  0, fltk_menu_jump_forward, this);
-    new Item("Jump Backward 10 Frames", 0, fltk_menu_jump_backward, this);
     new Item("Toggle Sound",            0, fltk_menu_toggle_sound, this);
     movie_ctrl->end();
 
