@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,16 +30,16 @@
 
 #include "gnash.h"
 #include "log.h"
-#include "render_handler.h"
-#include "render_handler_agg.h"
+#include "Renderer.h"
+#include "Renderer_agg.h"
+#include "Movie.h"
+#include "movie_root.h"
 #include "gtk_glue_agg_xv.h"
 #include "VideoConverter.h"
 #include "VM.h"
 
 #include <cerrno>
-
 #include <gdk/gdkx.h>
-
 #include <sys/ipc.h>
 #include <sys/shm.h>
 
@@ -102,10 +102,10 @@ GtkAggXvGlue::prepDrawingArea(GtkWidget *drawing_area)
     gtk_widget_set_double_buffered(_drawing_area, FALSE);
 }
 
-render_handler*
+Renderer*
 GtkAggXvGlue::createRenderHandler()
 {
-    _agg_renderer = create_render_handler_agg(findPixelFormat(_xv_format).c_str());
+    _agg_renderer = create_Renderer_agg(findPixelFormat(_xv_format).c_str());
     return _agg_renderer;
 }
 
@@ -116,11 +116,10 @@ GtkAggXvGlue::setupRendering()
     if (first && VM::isInitialized()) {
         first = false;
         
-        movie_definition* def = VM::get().getRoot().get_movie_definition();
-        assert(def);
+        const Movie& mi = VM::get().getRoot().getRootMovie();
     
-        _movie_width = def->get_width_pixels();
-        _movie_height = def->get_height_pixels();        
+        _movie_width = mi.widthPixels();
+        _movie_height = mi.heightPixels();        
     
         if (!create_xv_shmimage(_movie_width, _movie_height)) {
             if (!create_xv_image(_movie_width, _movie_height)) {
@@ -132,7 +131,7 @@ GtkAggXvGlue::setupRendering()
         
         if (_xv_format.type == XvRGB) {
             // init renderer to write directly to xv_image
-            static_cast<render_handler_agg_base *>(_agg_renderer)->init_buffer
+            static_cast<Renderer_agg_base *>(_agg_renderer)->init_buffer
                   ((unsigned char*) _xv_image->data, _xv_image->data_size,
                   _movie_width, _movie_height, _xv_image->pitches[0]);
     
@@ -148,8 +147,8 @@ GtkAggXvGlue::setupRendering()
 
     	    _offscreenbuf.reset(new unsigned char[bufsize]);
     	    
-    	    render_handler_agg_base * renderer =
-    	      static_cast<render_handler_agg_base *>(_agg_renderer);
+    	    Renderer_agg_base * renderer =
+    	      static_cast<Renderer_agg_base *>(_agg_renderer);
     	    renderer->init_buffer(_offscreenbuf.get(), bufsize, _movie_width,
     	                          _movie_height, _stride);
 
@@ -175,7 +174,7 @@ GtkAggXvGlue::render()
 }
 
 void
-GtkAggXvGlue::render(int minx, int miny, int maxx, int maxy)
+GtkAggXvGlue::render(int /* minx */, int /* miny */, int /* maxx */, int /* maxy */)
 {
     if (!_drawing_area || !_xv_image) {
         return;
