@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,7 +30,7 @@ class RiscosAggGlue : public RiscosGlue
 
     bool init(int argc, char **argv[]);
     void prepFramebuffer(void *framebuffer, int width, int height);
-    render_handler* createRenderHandler();
+    Renderer* createRenderHandler();
     void setRenderHandlerSize(int width, int height);
     void render(int x, int y);
     void render(int x, int y, int minx, int miny, int maxx, int maxy);
@@ -38,7 +39,7 @@ class RiscosAggGlue : public RiscosGlue
   private:
     unsigned char *_offscreenbuf;
     int _offscreenbuf_size;
-    render_handler *_agg_renderer;
+    Renderer *_agg_renderer;
     int _width, _height, _bpp;
 };
 
