@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,7 +16,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: riscossup.h,v 1.7 2008/01/21 20:55:43 rsavoye Exp $ */
 
 #ifndef __ROSUP_H__
 #define __ROSUP_H__
@@ -25,7 +25,6 @@
 #endif
 
 #include "gnash.h"
-#include "tu_config.h"
 
 #include "oslib/colourtrans.h"
 #include "oslib/wimp.h"
@@ -39,21 +38,22 @@
 namespace gnash
 {
 
-class DSOEXPORT RiscosGui : public Gui
+class RiscosGui : public Gui
 {
  public:
-    RiscosGui(unsigned long xid, float scale, bool loop, unsigned int depth);
+    RiscosGui(unsigned long xid, float scale, bool loop, RunResources& r);
     virtual ~RiscosGui();
     virtual bool init(int argc, char **argv[]);
     virtual bool createWindow(int width, int height);
-    virtual bool createWindow(const char *title, int width, int height);
+    virtual bool createWindow(const char *title, int width, int height,
+                              int xPosition = 0, int yPosition = 0);
     virtual bool run();
     virtual bool createMenu();
     virtual bool setupEvents();
     virtual void renderBuffer();
     virtual void setInterval(unsigned int interval);
     virtual void setTimeout(unsigned int timeout);
-    virtual void setInvalidatedRegion(const rect& bounds);
+    virtual void setInvalidatedRegion(const SWFRect& bounds);
 
 
  private:
