@@ -1,9 +1,10 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -15,16 +16,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
 #include "sdl_glue.h"
+
 #include <SDL.h>
-
-
-
-using namespace std;
-
+#include <boost/cstdint.hpp> // for boost::?int??_t 
 
 namespace gnash
 {
@@ -36,16 +31,20 @@ class SdlAggGlue : public SdlGlue
     virtual ~SdlAggGlue();
 
     bool init(int argc, char **argv[]);
-    render_handler* createRenderHandler(int depth);
-    bool prepDrawingArea(int width, int height, uint32_t sdl_flags);
-    uint32_t maskFlags(uint32_t sdl_flags);
+    Renderer* createRenderHandler(int depth);
+    void setInvalidatedRegions(const InvalidatedRanges& ranges);
+    bool prepDrawingArea(int width, int height, boost::uint32_t sdl_flags);
+    boost::uint32_t maskFlags(boost::uint32_t sdl_flags);
     void render();
     void render(int minx, int miny, int maxx, int maxy);
   private:
     SDL_Surface     *_sdl_surface;
     unsigned char   *_offscreenbuf;
     SDL_Surface     *_screen;
-    render_handler  *_agg_renderer;
+    Renderer  *_agg_renderer;
+    
+    geometry::Range2d<int> _validbounds;
+    std::vector< geometry::Range2d<int> > _drawbounds;
 };
 
 }
