@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,6 +21,11 @@
 #include <SDL.h>
 
 
+// Forward declarations
+namespace gnash {
+    class Renderer;
+}
+
 namespace gnash
 {
 
@@ -30,7 +36,8 @@ class SdlCairoGlue : public SdlGlue
     virtual ~SdlCairoGlue();
 
     bool init(int argc, char **argv[]);
-    render_handler* createRenderHandler( int depth);
+    Renderer* createRenderHandler( int depth);
+    void setInvalidatedRegions(const InvalidatedRanges& ranges);
     bool prepDrawingArea(int width, int height, boost::uint32_t sdl_flags);
     boost::uint32_t maskFlags(boost::uint32_t sdl_flags);
     void render();
@@ -40,6 +47,7 @@ class SdlCairoGlue : public SdlGlue
     SDL_Surface     *_sdl_surface;
     unsigned char   *_render_image;
     SDL_Surface     *_screen;
+    Renderer  *_renderer;
 };
 
 }
