@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -200,15 +200,16 @@ Buffer &
 Buffer::copy(boost::uint8_t *data, size_t nbytes)
 {    
 //    GNASH_REPORT_FUNCTION;
-    _seekptr =_data.get();
     if (_data) {
-	std::copy(data, data + nbytes, _data.get());
-	_seekptr = _data.get() + nbytes;
-    } else {
-        boost::format msg("Not enough storage was allocated to hold the "
-        "copied data! Needs %1%, only has %2% bytes");
-        msg % nbytes % _nbytes;
+	if (_nbytes >= nbytes) {
+	    std::copy(data, data + nbytes, _data.get());
+	    _seekptr = _data.get() + nbytes;
+	} else {
+	    boost::format msg("Not enough storage was allocated to hold the "
+			      "copied data! Needs %1%, only has %2% bytes");
+	    msg % nbytes % _nbytes;
 	    throw GnashException(msg.str());
+	}
     }
     return *this;
 }
@@ -230,9 +231,9 @@ Buffer::append(boost::uint8_t *data, size_t nbytes)
 	    std::copy(data, data + nbytes, _seekptr);
 	    _seekptr += nbytes;
 	} else {
-        boost::format msg("Not enough storage was allocated to hold the "
-        "appended data! Needs %1%, only has %2% bytes");
-        msg % nbytes % _nbytes;
+	    boost::format msg("Not enough storage was allocated to hold the "
+			      "appended data! Needs %1%, only has %2% bytes");
+	    msg % nbytes % spaceLeft();
 	    throw GnashException(msg.str());
 	}
     }
