@@ -1,6 +1,7 @@
 // amf.cpp:  AMF (Action Message Format) rpc marshalling, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +18,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include <string>
 #include <vector>
 #include <cmath>
@@ -234,7 +231,7 @@ Element::getDataSize() const
 	return _buffer->size();
     }
     return 0;
-};
+}
 
 /// \brief Cast the data in this Element to a double value.
 ///
@@ -292,7 +289,7 @@ Element::to_string() const
 	return "NULL";
     }
     return 0;
-};
+}
 
 /// \brief Cast the data in this Element to a boolean value.
 ///
@@ -305,7 +302,7 @@ Element::to_bool() const
 	return *(reinterpret_cast<bool *>(_buffer->reference()));
     }
     return false;
-};
+}
 
 /// \brief Cast the data in this Element to an real pointer to data.
 ///
@@ -318,7 +315,7 @@ Element::to_reference()
 	return _buffer->reference();
     }
     return 0;
-};
+}
 
 const boost::uint8_t *
 Element::to_reference() const
@@ -328,7 +325,7 @@ Element::to_reference() const
 	return _buffer->reference();
     }
     return 0;
-};
+}
 
 /// \brief Test equivalance against another Element.
 ///	This compares all the data and the data type in the
@@ -397,7 +394,7 @@ Element::operator==(bool x)
 	*_buffer += x;
     }
     return false;
-};
+}
 
 size_t
 Element::calculateSize()
@@ -538,7 +535,7 @@ Element::operator[](size_t index)
     
     boost::shared_ptr<Element> el; 
     return el;
-};
+}
 
 /// \brief Make this Element be the same as another Element.
 ///
@@ -553,7 +550,7 @@ Element::operator=(Element &el)
 //    GNASH_REPORT_FUNCTION;
     _type = el.getType();
     if (el.getNameSize()) {
-        _name = strdup(el.getName());
+	setName(el.getName());
     }
     _buffer.reset(new Buffer(el.getDataSize()));
     _buffer->copy(el.to_reference(), el.getDataSize());
