@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,21 +16,8 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
-#include <boost/cstdint.hpp>
-#include <cerrno>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <string>
-#include <vector>
-#include <iostream>
-#include <fstream>
-#include <cassert>
-
-#include "network.h"
+#include "GnashSystemNetHeaders.h"
+#include "GnashFileUtilities.h"
 #include "element.h"
 #include "amf.h"
 #include "buffer.h"
@@ -37,17 +25,16 @@
 #include "log.h"
 #include "GnashException.h"
 
-#if defined(HAVE_WINSOCK_H) && !defined(__OS2__)
-# include <winsock2.h>
-# include <windows.h>
-# include <sys/stat.h>
-# include <io.h>
-# include <ws2tcpip.h>
-#else
-#include <arpa/inet.h>
-#endif
-
 #include <boost/scoped_array.hpp>
+#include <boost/cstdint.hpp>
+#include <boost/shared_ptr.hpp>
+#include <cerrno>
+#include <string>
+#include <vector>
+#include <iostream>
+#include <fstream>
+#include <cassert>
+
 
 using namespace std;
 using namespace amf;
@@ -69,11 +56,23 @@ const short SOL_MAGIC = 0x00bf;	// is in big-endian format, this is the first
 //char *SOL_FILETYPE = "TCSO";
 const short SOL_BLOCK_MARK = 0x0004;
 
+/// \define ENSUREBYTES
+///
+/// @param from The base address to check.
+///
+/// @param tooFar The ending address that is one byte too many.
+///
+/// @param size The number of bytes to check for: from to tooFar.
+///
+/// @remarks May throw an Exception
 #define ENSUREBYTES(from, toofar, size) { \
 	if ( from+size >= toofar ) \
 		throw ParserException("Premature end of AMF stream"); \
 }
 
+/// \namespace amf
+///
+/// This namespace is for all the AMF specific classes in libamf.
 namespace amf
 {
 
@@ -86,15 +85,13 @@ SOL::SOL()
 SOL::~SOL()
 {
 //    GNASH_REPORT_FUNCTION;
-    vector<amf::Element *>::iterator it;
-    for (it = _amfobjs.begin(); it != _amfobjs.end(); it++) {
-	amf::Element *el = (*(it));
-	if (el) {
-	    delete el;
-	}
-    }
 }
 
+/// \brief Extract the header from the file.
+///
+/// @param filespec The name and path of the .sol file to parse.
+///
+/// @return true if this succeeded. false if it doesn't.
 bool
 SOL::extractHeader(const std::string & /*filespec*/)
 {
@@ -102,6 +99,12 @@ SOL::extractHeader(const std::string & /*filespec*/)
       return false;
 }
 
+/// \brief Extract the header from the file.
+///
+/// @param data a reference to a vector of bytes that contains the
+///	.sol file data.
+///
+/// @return true if this succeeded. false if it doesn't.
 bool
 SOL::extractHeader(const vector<unsigned char> & /*data*/)
 {
@@ -109,14 +112,25 @@ SOL::extractHeader(const vector<unsigned char> & /*data*/)
       return false;
 }
 
+/// \brief Add the AMF objects that are the data of the file
+//
+/// @param el A smart pointer to the Element to add to the .sol file.
+///
+/// @return nothing.
 void
-SOL::addObj(amf::Element *el)
+SOL::addObj(boost::shared_ptr<amf::Element> el)
 {
 //    GNASH_REPORT_FUNCTION;
     _amfobjs.push_back(el);
-//    _filesize += el->getName().size() + el->getLength() + 5;
+//    _filesize += el->getName().size() + el->getDataSize() + 5;
 }
 
+/// \brief Create the file header.
+///
+/// @param data a reference to a vector of bytes that contains the
+///		.sol file data.
+///
+/// @return true if this succeeded. false if it doesn't.
 bool
 SOL::formatHeader(const vector<unsigned char> & /*data*/)
 {
@@ -124,13 +138,24 @@ SOL::formatHeader(const vector<unsigned char> & /*data*/)
       return false;
 }
 
-// name is the object name
+/// \brief Create the file header.
+///
+/// @param name The name of the SharedObject for this file.
+///
+/// @return true if this succeeded. false if it doesn't.
 bool
 SOL::formatHeader(const std::string &name)
 {
     return formatHeader(name, _filesize);
 }
 
+/// \brief Create the file header.
+///
+/// @param name The name of the SharedObject for this file.
+///
+/// @param filesize The size of the file.
+///
+/// @return true if this succeeded. false if it doesn't.
 bool
 SOL::formatHeader(const std::string &name, int filesize)
 {
@@ -141,7 +166,7 @@ SOL::formatHeader(const std::string &name, int filesize)
     // so we swap it first.
     boost::uint16_t swapped = SOL_MAGIC;
     swapped = htons(swapped);
-    Network::byte_t *ptr = reinterpret_cast<Network::byte_t *>(&swapped);
+    boost::uint8_t *ptr = reinterpret_cast<boost::uint8_t *>(&swapped);
     for (i=0; i<sizeof(boost::uint16_t); i++) {
         _header.push_back(ptr[i]);
     }
@@ -152,7 +177,7 @@ SOL::formatHeader(const std::string &name, int filesize)
     filesize += name.size() + 16;
     boost::uint32_t len = filesize;
     len = htonl(len);
-    ptr = reinterpret_cast<Network::byte_t *>(&len);
+    ptr = reinterpret_cast<boost::uint8_t *>(&len);
     for (i=0; i<sizeof(boost::uint32_t); i++) {
         _header.push_back(ptr[i]);
     }
@@ -161,14 +186,14 @@ SOL::formatHeader(const std::string &name, int filesize)
     // we just built it the same way it always is.
     // first is the TCSO, we have no idea what this stands for.
 //    ptr = reinterpret_cast<uint8_t *>(const_cast<uint8_t *>("TCSO");
-    ptr = (uint8_t *)"TCSO";
+	ptr = (boost::uint8_t *)"TCSO";
     for (i=0; i<sizeof(boost::uint32_t); i++) {
         _header.push_back(ptr[i]);
     }
     // then the 0x0004 bytes, also a mystery
     swapped = SOL_BLOCK_MARK;
     swapped = htons(swapped);
-    ptr = reinterpret_cast<Network::byte_t *>(&swapped);
+    ptr = reinterpret_cast<boost::uint8_t *>(&swapped);
     for (i=0; i<sizeof(boost::uint16_t); i++) {
         _header.push_back(ptr[i]);
     }
@@ -182,12 +207,12 @@ SOL::formatHeader(const std::string &name, int filesize)
     //  First the length in two bytes
     swapped = name.size();
     swapped = htons(swapped);
-    ptr = reinterpret_cast<Network::byte_t *>(&swapped);
+    ptr = reinterpret_cast<boost::uint8_t *>(&swapped);
     for (i=0; i<sizeof(boost::uint16_t); i++) {
         _header.push_back(ptr[i]);
     }
     // then the string itself
-    ptr = (Network::byte_t *)name.c_str();
+    ptr = (boost::uint8_t *)name.c_str();
     for (i=0; i<name.size(); i++) {
         _header.push_back(ptr[i]);
     }
@@ -209,8 +234,13 @@ SOL::formatHeader(const std::string &name, int filesize)
     return true;
 }    
 
-// write the data to disk as a .sol file
-
+/// \brief Write the data to disk as a .sol file
+///
+/// @param filespec The name and path of the .sol file to parse.
+///
+/// @param name The name of the SharedObject for this file.
+///
+/// @return true if this succeeded. false if it doesn't.
 bool
 SOL::writeFile(const string &filespec, const string &name)
 {
@@ -221,8 +251,8 @@ SOL::writeFile(const string &filespec, const string &name)
         return false;
     }
     
-    vector<Network::byte_t>::iterator it;
-    vector<amf::Element *>::iterator ita; 
+    vector<boost::uint8_t>::iterator it;
+    vector<boost::shared_ptr<amf::Element> >::iterator ita; 
     AMF amf_obj;
     char *ptr;
     int size = 0;
@@ -232,20 +262,20 @@ SOL::writeFile(const string &filespec, const string &name)
     }
 
     for (ita = _amfobjs.begin(); ita != _amfobjs.end(); ita++) {
-        amf::Element *el = (*(ita));
-	size += el->getNameSize() + el->getLength() + 7;
+        boost::shared_ptr<amf::Element> el = (*(ita));
+	size += el->getNameSize() + el->getDataSize() + 7;
     }
     _filesize = size;
     
     boost::scoped_array<char> body ( new char[size + 20] );
-    memset(body.get(), 0, size);
+    std::fill_n(body.get(), size, 0);
     ptr = body.get();
     char* endPtr = ptr+size+20; // that's the amount we allocated..
 
     for (ita = _amfobjs.begin(); ita != _amfobjs.end(); ita++) {
-        amf::Element *el = (*(ita));
-        Buffer *var = amf_obj.encodeProperty(el); 
-        //  Network::byte_t *var = amf_obj.encodeProperty(el, outsize); 
+        boost::shared_ptr<Element> el = (*(ita));
+        boost::shared_ptr<amf::Buffer> var = amf_obj.encodeProperty(el); 
+        //  boost::uint8_t *var = amf_obj.encodeProperty(el, outsize); 
         if (!var) {
             continue;
         }
@@ -274,9 +304,9 @@ SOL::writeFile(const string &filespec, const string &name)
 // 	      *ptr++ = 0;	// doubles are terminated too!
 	      break;
 	  case Element::STRING_AMF0:
-	      if (el->getLength() == 0) {
+	      if (el->getDataSize() == 0) {
               	  assert(ptr+outsize+1 < endPtr);
-		  memcpy(ptr, var, outsize+1);
+		  memcpy(ptr, var->reference(), outsize+1);
 		  ptr += outsize+1;
 	      } else {		// null terminate the string
                   assert(ptr+outsize < endPtr);
@@ -290,11 +320,10 @@ SOL::writeFile(const string &filespec, const string &name)
 	      memcpy(ptr, var->reference(), outsize);
 	      ptr += outsize;
 	}
-	delete var;
     }
     
     _filesize = ptr - body.get();
-    int len = name.size() + sizeof(uint16_t) + 16;
+	int len = name.size() + sizeof(boost::uint16_t) + 16;
     boost::scoped_array<char> head ( new char[len + 4] );
     memset(head.get(), 0, len);
     ptr = head.get();
@@ -305,14 +334,15 @@ SOL::writeFile(const string &filespec, const string &name)
     
     if ( ofs.write(head.get(), _header.size()).fail() )
     {
-        log_error("Error writing %d bytes of header to output file %s", _header.size(), filespec.c_str());
+        log_error("Error writing %d bytes of header to output file %s", 
+                _header.size(), filespec);
         return false;
     }
 
-//    ofs.write(body, (ptr - body));
     if ( ofs.write(body.get(), _filesize).fail() )
     {
-        log_error("Error writing %d bytes of body to output file %s", _filesize, filespec.c_str());
+        log_error("Error writing %d bytes of body to output file %s",
+                _filesize, filespec);
         return false;
     }
     ofs.close();
@@ -320,26 +350,34 @@ SOL::writeFile(const string &filespec, const string &name)
     return true;
 }
 
-// read the .sol file from disk
+/// \brief Read a .sol file from disk
+///
+/// @param filespec The name and path of the .sol file to parse.
+///
+/// @return true if this succeeded. false if it doesn't.
 bool
 SOL::readFile(const std::string &filespec)
 {
 //    GNASH_REPORT_FUNCTION;
     struct stat st;
     boost::uint16_t size;
-    Network::byte_t *buf = 0;
-    Network::byte_t *ptr = 0;
     size_t bodysize;
 
     // Make sure it's an SOL file
-    if (stat(filespec.c_str(), &st) == 0) {
+    if (stat(filespec.c_str(), &st) != 0)  return false;
 
 	try {
+
+        boost::uint8_t *ptr = 0;
+
 	    ifstream ifs(filespec.c_str(), ios::binary);
-	    _filesize = st.st_size;
-	    buf = new Network::byte_t[_filesize + sizeof(int)];
-	    ptr = buf;
-	    Network::byte_t* tooFar = buf+_filesize;
+
+        _filesize = st.st_size;
+        boost::scoped_array<boost::uint8_t> buf(
+                new boost::uint8_t[_filesize + sizeof(int)]);
+
+	    ptr = buf.get();
+	    boost::uint8_t* tooFar = buf.get() + _filesize;
 	    
 	    bodysize = st.st_size - 6;
 	    _filespec = filespec;
@@ -362,22 +400,25 @@ SOL::readFile(const std::string &filespec)
 	    
 	    // consistency check
 	    if ((buf[0] == 0) && (buf[1] == 0xbf)) {
-		if (bodysize == length) {
-		    log_debug("%s is an SOL file", filespec.c_str());
-		} else {
-		    log_error("%s looks like an SOL file, but the length is wrong. Should be %d, got %d",
-			      filespec.c_str(), (_filesize - 6), length);
-		}
-		
-        } else {
-		log_error("%s isn't an SOL file", filespec.c_str());
+            if (bodysize == length) {
+                log_debug("%s is an SOL file", filespec);
+            }
+            else {
+		    log_error("%s looks like an SOL file, but the length is wrong. "
+                    "Should be %d, got %d",
+                    filespec, (_filesize - 6), length);
+            }
+        }
+        else {
+            log_error("%s isn't an SOL file", filespec);
 	    }
 	    
 #ifndef GNASH_TRUST_AMF
 	    ENSUREBYTES(ptr, tooFar, 2); 
 #endif
 	    
-	    // 2 bytes for the length of the object name, but it's also null terminated
+	    // 2 bytes for the length of the object name, but it's also 
+        // null terminated
 	    size = *(reinterpret_cast<boost::uint16_t *>(ptr));
 	    size = ntohs(size);
 	    ptr += 2;
@@ -395,60 +436,77 @@ SOL::readFile(const std::string &filespec)
 	    ptr += 4;
 	    
 	    AMF amf_obj;
-	    amf::Element *el;
+	    boost::shared_ptr<amf::Element> el;
 	    while ( ptr < tooFar) {
-		if (ptr) {
-		    el = amf_obj.extractProperty(ptr, tooFar);
-		    if (el != 0) {
-			ptr += amf_obj.totalsize() + 1;
-			_amfobjs.push_back(el);
-		    } else {
-			break;
-		    }
-//		log_debug("Bodysize is: %d size is: %d for %s", bodysize, size, el->getName());
-		} else {
-		    break;
-		}
+            if (ptr) {
+                el = amf_obj.extractProperty(ptr, tooFar);
+                if (el != 0) {
+                    ptr += amf_obj.totalsize() + 1;
+                    _amfobjs.push_back(el);
+                }
+                else break;
+            } else break;
 	    }
-	    delete[] buf;
 	    
 	    ifs.close();
 	    return true;
-	} catch (std::exception& e) {
+	}
+    catch (std::exception& e) {
 	    log_error("Reading SharedObject %s: %s", filespec, e.what());
 	    return false;
 	}
-    }
     
-//    log_error("Couldn't open file: %s", strerror(errno));
-    return false;
 }
 
+bool 
+SOL::updateSO(boost::shared_ptr<amf::Element> &newel)
+{
+//    GNASH_REPORT_FUNCTION;
+    vector<boost::shared_ptr<amf::Element> >::iterator ita; 
+    for (ita = _amfobjs.begin(); ita != _amfobjs.end(); ita++) {
+        boost::shared_ptr<amf::Element> oldel = (*(ita));
+	if (oldel == newel) {
+	    oldel = newel;
+	}
+   }
+   return true;
+}
+
+bool
+SOL::updateSO(int index, boost::shared_ptr<amf::Element> &el)
+{
+//    GNASH_REPORT_FUNCTION;
+    _amfobjs[index] = el;    
+    return true;
+}
+
+///  \brief Dump the internal data of this class in a human readable form.
+///
+/// @remarks This should only be used for debugging purposes.
 void
 SOL::dump()
 {
-    vector<amf::Element *>::iterator it;
+    vector<boost::shared_ptr<amf::Element> >::iterator it;
 
     cerr << "Dumping SOL file" << endl;
     cerr << "The file name is: " << _filespec << endl;
     cerr << "The size of the file is: " << _filesize << endl;
     cerr << "The name of the object is: " << _objname << endl;
     for (it = _amfobjs.begin(); it != _amfobjs.end(); it++) {
-	amf::Element *el = (*(it));
+	boost::shared_ptr<amf::Element> el = (*(it));
         cerr << el->getName() << ": ";
         if (el->getType() == Element::STRING_AMF0) {
-            if (el->getLength() != 0) {
-                cerr << el->getData();
+            if (el->getDataSize() != 0) {
+                cerr << el->to_string();
             } else {
                 cerr << "null";
             }
         }
         if (el->getType() == Element::NUMBER_AMF0) {
-            double ddd = *((double *)el->getData());
+            double ddd = el->to_number();
 	    swapBytes(&ddd, sizeof(double));
-	    cerr << ddd << " ";
-
-            cerr << "( " << hexify(el->getData(), 8, false) << ")";
+	    cerr << ddd << endl;
+//            cerr << "( " << hexify(el->to_(), 8, false) << ")";
         }
         if (el->getType() == Element::BOOLEAN_AMF0) {
             if (el->to_bool() == true) {
