@@ -1,6 +1,7 @@
 // Image.cpp: image data class for Gnash.
 // 
-//   Copyright (C) 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
 #include <boost/scoped_array.hpp>
 #include <boost/shared_ptr.hpp>
 
-#include "gnash.h" // for image file types
+#include "FileTypes.h"
 #include "GnashImage.h"
 #ifdef USE_PNG
 # include "GnashImagePng.h"
@@ -74,7 +75,7 @@ GnashImage::GnashImage(int width, int height,
     assert(pitch >= width);
 }
 
-void GnashImage::update(boost::uint8_t* data)
+void GnashImage::update(const boost::uint8_t* data)
 {
     std::memcpy(this->data(), data, _size);
 }
@@ -138,8 +139,9 @@ ImageRGBA::~ImageRGBA()
 }
 
 
-void ImageRGBA::setPixel(size_t x, size_t y, boost::uint8_t r, boost::uint8_t g, boost::uint8_t b, boost::uint8_t a)
-// Set the pixel at the given position.
+void
+ImageRGBA::setPixel(size_t x, size_t y, boost::uint8_t r, boost::uint8_t g,
+        boost::uint8_t b, boost::uint8_t a)
 {
     assert(x < _width);
     assert(y < _height);
