@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,8 +16,8 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef __GNASH_URL_H__
-#define __GNASH_URL_H__
+#ifndef GNASH_URL_H
+#define GNASH_URL_H
 
 #include "dsodefs.h"
 
@@ -71,6 +72,8 @@ public:
 	const std::string& port() const { return _port; }
 
 	/// Return the 'path' member of this URL, as a string
+    //
+    /// The returned path starts with '/'
 	const std::string& path() const { return _path; }
 
 	/// Return the 'anchor' member of this URL, as a string
