@@ -1,6 +1,7 @@
 // sharedlib.cpp:  Shared Library support, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -59,31 +60,15 @@ lt_dlhandle lt_dlopenext       (const char *filename);
 # define PLUGINSDIR "./"
 #endif
 
-using namespace std;
-
 namespace gnash {
 
-SharedLib::SharedLib()
-{
-//    GNASH_REPORT_FUNCTION;
-#ifdef LT_DLMUTEX
-//     return lt_dlmutex_register (gnash_mutex_lock, gnash_mutex_unlock,
-//                                 gnash_mutex_seterror, gnash_mutex_geterror);
-#endif
-}
-
 SharedLib::SharedLib(const std::string& filespec)
 {
-    GNASH_REPORT_FUNCTION;
+    _filespec = filespec;
 }
 
-SharedLib::SharedLib(const std::string &filespec, const std::string &envvar)
+SharedLib::SharedLib(const std::string& filespec, const std::string& envvar)
 {
-    GNASH_REPORT_FUNCTION;
-#ifdef LT_DLMUTEX
-//     return lt_dlmutex_register (gnash_mutex_lock, gnash_mutex_unlock,
-//                                 gnash_mutex_seterror, gnash_mutex_geterror);
-#endif
     _filespec = filespec;
     scoped_lock lock(_libMutex);
     
@@ -91,11 +76,9 @@ SharedLib::SharedLib(const std::string &filespec, const std::string &envvar)
     int errors = lt_dlinit ();
     if (errors) {
         log_error (_("Couldn't initialize ltdl: %s"), lt_dlerror());
-//     } else {
-//         log_debug ("Initialized ltdl");
     }
     
-    string pluginsdir;
+    std::string pluginsdir;
     char *env = std::getenv (envvar.c_str());
     if (env) {
         pluginsdir = env;
@@ -103,14 +86,10 @@ SharedLib::SharedLib(const std::string &filespec, const std::string &envvar)
         pluginsdir = PLUGINSDIR;
     }
     
-    // lt_dladdsearchdir(pluginsdir.c_str());
 }
 
 SharedLib::~SharedLib()
 {
-    //   GNASH_REPORT_FUNCTION;
-//    closeLib();
-//    lt_dlexit();
 }
 
 bool
@@ -128,26 +107,10 @@ SharedLib::openLib()
 bool
 SharedLib::openLib (const std::string& filespec)
 {
-//    GNASH_REPORT_FUNCTION;
-    
-#if 0
-    // ltdl should use the same mallocation as us
-    lt_dlmalloc = (lt_ptr (*) (size_t)) xmalloc;
-    lt_dlfree = (void (*) (lt_ptr)) free;
-    
-#endif
-    
-    // Make sure preloaded modules are initialised
-//  LTDL_SET_PRELOADED_SYMBOLS();
     
     scoped_lock lock(_libMutex);
-    
-//     // libtool's dynamic library loader is already initialized in constructor
-    
-//     cerr << "Searching in " << lt_dlgetsearchpath()
-//          << "for database drivers" << endl;
 
-//    log_debug ("Trying to open shared library \"%s\"", filespec);
+    log_debug ("Trying to open shared library \"%s\"", filespec);
     _dlhandle = lt_dlopenext (filespec.c_str());
     
     if (_dlhandle == NULL) {
@@ -168,7 +131,7 @@ SharedLib::openLib (const std::string& filespec)
 SharedLib::initentry *
 SharedLib::getInitEntry (const std::string& symbol)
 {
-    GNASH_REPORT_FUNCTION;
+    // GNASH_REPORT_FUNCTION;
     lt_ptr run = NULL;
     
     scoped_lock lock(_libMutex);
