@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,15 +19,8 @@
 #ifndef __SHM_H__
 #define __SHM_H__
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include <string>
 
-//#include "impl.h"
-#include "as_object.h" // for inheritance
-
 #include <sys/types.h>
 #if !defined(HAVE_WINSOCK_H) && !defined(__riscos__) && !defined(__OS2__)
 # include <sys/ipc.h>
@@ -169,17 +163,6 @@ public:
     void destroy(pointer __p)   { __p->~_Tp(); }
 };
 
-class shm_as_object : public as_object
-{
-public:
-    Shm obj;
-};
-
-as_value shm_getname(const fn_call& fn);
-as_value shm_getsize(const fn_call& fn);
-as_value shm_getallocated(const fn_call& fn);
-as_value shm_exists(const fn_call& fn);
-
 } // end of gnash namespace
 
 // end of __SHM_H__
