@@ -1,6 +1,7 @@
 // string_table.h -- A shared string table for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -108,25 +109,26 @@ public:
 	/// @return The assigned key
 	key insert(const std::string& to_insert);
 
-	/// \brief
-	/// Insert a group of strings with their ids preset. This allows
+	/// Insert a group of strings with their ids preset.
+    //
+    /// This allows
 	/// for switches and enums and such, but be careful you don't set two
 	/// strings with the same id, as this does not check for such occurrences.
-	/// Converts the strings to lower case if mSetToLower is true.
-	/// In any case, sets mSetToLower to false at the end.
 	///
 	/// @param pList
 	/// An array of svt objects, these should be fully constructed, including
 	/// their ids.
-	void insert_group(svt* pList, std::size_t size);
+    ///
+	/// @param size
+    /// Number of elements in the svt objects array
+    ///
+	void insert_group(const svt* pList, std::size_t size);
 
-	/// \brief
-	/// Call this just before calling insert_group if the next group should
-	/// be set to lower_case before addition.
-	void lower_next_group() { mSetToLower = true; }
-
-	/// \brief
 	/// Insert a string when you will handle the locking yourself.
+    //
+	/// @param to_insert
+    /// String to insert
+    ///
 	/// @param lock
 	/// Use lock_mutex to obtain the correct mutex to use for this -- using
 	/// a different mutex will not be thread safe.
@@ -145,7 +147,6 @@ public:
 		mTable(),
 		mLock(),
 		mHighestKey(0),
-		mSetToLower(false),
 		mCaseInsensitive(false)
 	{/**/}
 
@@ -154,7 +155,6 @@ private:
 	static const std::string mEmpty; // The empty string, universally.
 	boost::mutex mLock;
 	std::size_t mHighestKey;
-	bool mSetToLower; // If true, affects the next group addition.
 	bool mCaseInsensitive;
 };
 
