@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -45,11 +45,11 @@ class Bitmap : public DisplayObject
 {
 public:
 
-	Bitmap(boost::intrusive_ptr<BitmapData_as> bd, DisplayObject* parent,
-            int id);
+	Bitmap(movie_root& mr, as_object* object, BitmapData_as* bd,
+            DisplayObject* parent);
 	
-    Bitmap(const BitmapMovieDefinition* const def, DisplayObject* parent,
-            int id);
+    Bitmap(movie_root& mr, as_object* object, const BitmapMovieDefinition* def,
+            DisplayObject* parent);
 
     ~Bitmap();
 
@@ -63,7 +63,7 @@ public:
 
     virtual void display(Renderer& renderer);
 
-    virtual rect getBounds() const;
+    virtual SWFRect getBounds() const;
 
     virtual bool pointInShape(boost::int32_t x, boost::int32_t y) const;
 
@@ -101,7 +101,7 @@ private:
 
     const boost::intrusive_ptr<const BitmapMovieDefinition> _def;
 
-    boost::intrusive_ptr<BitmapData_as> _bitmapData;
+    BitmapData_as* _bitmapData;
 
     /// The current bitmap information is stored here.
     boost::intrusive_ptr<BitmapInfo> _bitmapInfo;
