@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,20 +17,22 @@
 
 #include "BitmapMovie.h"
 #include "BitmapMovieDefinition.h"
+#include "Bitmap.h"
 
 namespace gnash {
 
-BitmapMovie::BitmapMovie(const BitmapMovieDefinition* const def,
+BitmapMovie::BitmapMovie(as_object* object, const BitmapMovieDefinition* def,
         DisplayObject* parent)
 	:
-	Movie(def, parent),
+	Movie(object, def, parent),
     _def(def)
 {
     assert(def);
-    boost::intrusive_ptr<DisplayObject> ch = def->createDisplayObject(this, 1);
+    assert(object);
+    Bitmap* bm = new Bitmap(stage(), 0, def, this);
 
     const int depth = 1 + DisplayObject::staticDepthOffset;
-    placeDisplayObject(ch.get(), depth);
+    placeDisplayObject(bm, depth);
 }
 
 } // namespace gnash
