@@ -1,6 +1,7 @@
 // dlist.cpp:    Display lists, for Gnash.
 // 
-//     Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//     Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//     Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,6 +29,7 @@
 #include <iostream>
 #include <algorithm>
 #include <stack>
+#include <cassert>
 #include <boost/bind.hpp>
 
 namespace gnash {
@@ -64,7 +66,7 @@ public:
     DepthEquals(int depth) : _depth(depth) {}
 
     bool operator() (const DisplayItem& item) {
-        if ( ! item.get() ) return false;
+        if (!item) return false;
         return item->get_depth() == _depth;
     }
 
@@ -93,7 +95,7 @@ public:
     DepthGreaterOrEqual(int depth) : _depth(depth) {}
 
     bool operator() (const DisplayItem& item) {
-        if ( ! item.get() ) return false;
+        if (!item) return false;
         return item->get_depth() >= _depth;
     }
 private:
@@ -107,7 +109,7 @@ public:
     NameEquals(const std::string& name) : _name(name) {}
 
     bool operator() (const DisplayItem& item) {
-        if ( ! item.get() ) return false;
+        assert (item);
         return item->get_name() == _name;
     }
 
@@ -122,7 +124,7 @@ public:
     NameEqualsNoCase(const std::string& name) : _name(name) {}
 
     bool operator() (const DisplayItem& item) {
-        if ( ! item.get() ) return false;
+        assert (item);
         return _noCaseEquals(item->get_name(), _name);
     }
 
@@ -142,7 +144,7 @@ DisplayList::getNextHighestDepth() const
     for (const_iterator it = _charsByDepth.begin(),
             itEnd = _charsByDepth.end(); it != itEnd; ++it) {
 
-        DisplayObject* ch = it->get();
+        DisplayObject* ch = *it;
 
         int chdepth = ch->get_depth();
         if (chdepth >= nexthighestdepth) {
@@ -160,7 +162,7 @@ DisplayList::getDisplayObjectAtDepth(int depth)
     for (iterator it = _charsByDepth.begin(), itEnd = _charsByDepth.end();
         it != itEnd; ++it) {
 
-        DisplayObject* ch = it->get();
+        DisplayObject* ch = *it;
 
         // found
         if (ch->get_depth() == depth) return ch;
@@ -186,7 +188,7 @@ DisplayList::getDisplayObjectByName(const std::string& name)
 
     if (it == e) return 0;
     
-    return it->get();
+    return *it;
 
 }
 
@@ -202,7 +204,7 @@ DisplayList::getDisplayObjectByName_i(const std::string& name)
 
     if ( it == e ) return NULL;
     
-    return it->get();
+    return *it;
 }
 
 void
@@ -227,7 +229,7 @@ DisplayList::placeDisplayObject(DisplayObject* ch, int depth,
         (*it)->add_invalidated_bounds(old_ranges, true);    
 
         // make a copy (before replacing)
-        boost::intrusive_ptr<DisplayObject> oldCh = *it;
+        DisplayObject* oldCh = *it;
 
         // replace existing char (before calling unload!)
         *it = DisplayItem(ch);
@@ -288,7 +290,7 @@ DisplayList::replaceDisplayObject(DisplayObject* ch, int depth,
     }
     else {
         // Make a copy (before replacing)
-        boost::intrusive_ptr<DisplayObject> oldch = *it;
+        DisplayObject* oldch = *it;
 
         InvalidatedRanges old_ranges;
     
@@ -299,7 +301,7 @@ DisplayList::replaceDisplayObject(DisplayObject* ch, int depth,
 
         if (use_old_matrix) {
             // Use the SWFMatrix from the old DisplayObject.
-            ch->copyMatrix(*oldch); // copy SWFMatrix and caches
+            ch->setMatrix(oldch->getMatrix(), true); 
         }
         
         // remember bounds of old char
@@ -391,7 +393,7 @@ DisplayList::removeDisplayObject(int depth)
 
     if (it != _charsByDepth.end()) {
         // Make a copy (before erasing)
-        boost::intrusive_ptr<DisplayObject> oldCh = *it;
+        DisplayObject* oldCh = *it;
 
         // Erase (before calling unload)
         _charsByDepth.erase(it);
@@ -502,7 +504,7 @@ DisplayList::removeDisplayObjectAt(int index)
 
     if (it == _charsByDepth.end()) return 0;
    
-    DisplayObject* obj = it->get();
+    DisplayObject* obj = *it;
     _charsByDepth.erase(it);
     return obj;
 }
@@ -659,7 +661,7 @@ DisplayList::display(Renderer& renderer)
     iterator it = beginNonRemoved(_charsByDepth);
     for (iterator endIt = _charsByDepth.end(); it != endIt; ++it)
     {
-        DisplayObject* ch = it->get();
+        DisplayObject* ch = *it;
 
         DisplayObject* mask = ch->getMask();
         if (mask && ch->visible() && ! mask->unloaded())
@@ -738,7 +740,7 @@ DisplayList::omit_display()
 {
     iterator it = beginNonRemoved(_charsByDepth);
     for (iterator endIt = _charsByDepth.end(); it != endIt; ++it) {
-        DisplayObject* ch = it->get();
+        DisplayObject* ch = *it;
         ch->omit_display();
     }
 }
@@ -753,9 +755,8 @@ DisplayList::dump() const
             endIt = _charsByDepth.end(); it != endIt; ++it) {
 
         const DisplayItem& dobj = *it;
-        log_debug(_("Item %d at depth %d (char id %d, name %s, type %s)"),
-            num, dobj->get_depth(), dobj->get_id(),
-            dobj->get_name(), typeName(*dobj));
+        log_debug(_("Item %d at depth %d (char name %s, type %s)"),
+            num, dobj->get_depth(), dobj->get_name(), typeName(*dobj));
         num++;
     }
 }
@@ -913,13 +914,13 @@ DisplayList::mergeDisplayList(DisplayList & newList)
     {
         iterator itOldBackup = itOld;
         
-        boost::intrusive_ptr<DisplayObject> chOld = itOldBackup->get();
+        DisplayObject* chOld = *itOldBackup;
         int depthOld = chOld->get_depth();
 
         while (itNew != itNewEnd) {
             iterator itNewBackup = itNew;
             
-            boost::intrusive_ptr<DisplayObject> chNew = itNewBackup->get();
+            DisplayObject* chNew = *itNewBackup;
             int depthNew = chNew->get_depth();
             
             // depth in old list is occupied, and empty in new list.
@@ -946,7 +947,7 @@ DisplayList::mergeDisplayList(DisplayList & newList)
                     (chOld->get_ratio() == chNew->get_ratio());
 
                 if (!is_ratio_compatible || chOld->isDynamic() ||
-                        !chOld->isActionScriptReferenceable()) {
+                        !isReferenceable(*chOld)) {
                     // replace the DisplayObject in old list with
                     // corresponding DisplayObject in new list
                     _charsByDepth.insert(itOldBackup, *itNewBackup);
@@ -962,7 +963,7 @@ DisplayList::mergeDisplayList(DisplayList & newList)
                     // replace the transformation SWFMatrix if the old
                     // DisplayObject accepts static transformation.
                     if (chOld->get_accept_anim_moves()) {
-                        chOld->copyMatrix(*chNew); // copy SWFMatrix and caches 
+                        chOld->setMatrix(chNew->getMatrix(), true); 
                         chOld->set_cxform(chNew->get_cxform());
                     }
                     chNew->unload();
@@ -987,7 +988,7 @@ DisplayList::mergeDisplayList(DisplayList & newList)
     // unload remaining DisplayObjects directly.
     while((itOld != itOldEnd) && ((*itOld)->get_depth() < 0)) {
 
-        boost::intrusive_ptr<DisplayObject> chOld = itOld->get();
+        DisplayObject* chOld = *itOld;
         itOld = _charsByDepth.erase(itOld);
 
         if (chOld->unload()) reinsertRemovedCharacter(chOld);
@@ -1004,7 +1005,7 @@ DisplayList::mergeDisplayList(DisplayList & newList)
     // old display list, and clear the new display list
     for (itNew = newList._charsByDepth.begin(); itNew != itNewEnd; ++itNew) {
 
-        boost::intrusive_ptr<DisplayObject> chNew = itNew->get();
+        DisplayObject* chNew = *itNew;
         int depthNew = chNew->get_depth();
 
         if (chNew->unloaded()) {
@@ -1024,7 +1025,7 @@ DisplayList::mergeDisplayList(DisplayList & newList)
     for (iterator i = newList._charsByDepth.begin(),
             e = newList._charsByDepth.end(); i != e; ++i) {
 
-        DisplayObject* ch = (*i).get();
+        DisplayObject* ch = *i;
         if (!ch->unloaded()) {
 
             iterator found =
@@ -1050,7 +1051,7 @@ DisplayList::mergeDisplayList(DisplayList & newList)
 
 
 void
-DisplayList::reinsertRemovedCharacter(boost::intrusive_ptr<DisplayObject> ch)
+DisplayList::reinsertRemovedCharacter(DisplayObject* ch)
 {
     assert(ch->unloaded());
     testInvariant();
@@ -1143,9 +1144,8 @@ operator<< (std::ostream& os, const DisplayList& dl)
 
         const DisplayItem& item = *it; 
         if (it != dl._charsByDepth.begin()) os << " | ";
-        os << "ch id:" << item->get_id()
-            << " name:" << item->get_name()
-            << " depth:" << item->get_depth();
+        os << " name:" << item->get_name()
+           << " depth:" << item->get_depth();
     }
 
     return os;
