@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -107,7 +107,8 @@ void
 DynamicShape::beginLinearGradientFill(const std::vector<gradient_record>& grad, const SWFMatrix& mat)
 {
 	// Add the new fill style and set as current
-	fill_style style; style.setLinearGradient(grad, mat);
+	fill_style style;
+    style.setLinearGradient(grad, mat);
 
 	endFill();
 
@@ -178,10 +179,10 @@ DynamicShape::finalize() const
 void
 DynamicShape::lineStyle(boost::uint16_t thickness, const rgba& color,
 	bool vScale, bool hScale, bool pixelHinting, bool noClose,
-	cap_style_e startCapStyle, cap_style_e endCapStyle,
-	join_style_e joinStyle, float miterLimitFactor)
+	CapStyle startCapStyle, CapStyle endCapStyle,
+	JoinStyle joinStyle, float miterLimitFactor)
 {
-	line_style style(thickness, color, vScale, hScale, pixelHinting,
+	LineStyle style(thickness, color, vScale, hScale, pixelHinting,
 		noClose, startCapStyle, endCapStyle, joinStyle,
 		miterLimitFactor);
 
@@ -217,7 +218,7 @@ DynamicShape::lineTo(boost::int32_t x, boost::int32_t y, int swfVersion)
 	_currpath->drawLineTo(x, y);
 
 	// Update bounds 
-    rect bounds = _shape.getBounds();
+    SWFRect bounds = _shape.getBounds();
 
 	unsigned thickness = _currline ? 
         _shape.lineStyles().back().getThickness() : 0;
@@ -248,7 +249,7 @@ DynamicShape::curveTo(boost::int32_t cx, boost::int32_t cy,
 
 	_currpath->drawCurveTo(cx, cy, ax, ay);
 
-    rect bounds = _shape.getBounds();
+    SWFRect bounds = _shape.getBounds();
 
 	unsigned thickness = _currline ? 
         _shape.lineStyles().back().getThickness() : 0;
@@ -281,7 +282,7 @@ DynamicShape::add_fill_style(const fill_style& stl)
 }
 
 size_t
-DynamicShape::add_line_style(const line_style& stl)
+DynamicShape::add_line_style(const LineStyle& stl)
 {
     _shape.addLineStyle(stl);
     return _shape.lineStyles().size();
