@@ -1,6 +1,7 @@
 // as_environment.cpp:  Variable, Sprite, and Movie locators, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -44,6 +45,67 @@
 
 namespace gnash {
 
+namespace {
+
+/// Find a variable in the given as_object
+//
+/// @param varname
+/// Name of the local variable
+///
+/// @param ret
+/// If a variable is found it's assigned to this parameter.
+/// Untouched if the variable is not found.
+///
+/// @return true if the variable was found, false otherwise
+bool
+getLocal(as_object& locals, const std::string& name, as_value& ret)
+{
+    string_table& st = getStringTable(locals);
+    return locals.get_member(st.find(name), &ret);
+}
+
+/// Delete a local variable
+//
+/// @param varname
+/// Name of the local variable
+///
+/// @return true if the variable was found and deleted, false otherwise
+bool
+deleteLocal(as_object& locals, const std::string& varname)
+{
+    string_table& st = getStringTable(locals);
+    return locals.delProperty(st.find(varname)).second;
+}
+
+/// Set a variable of the given object, if it exists.
+//
+/// @param varname
+/// Name of the local variable
+///
+/// @param val
+/// Value to assign to the variable
+///
+/// @return true if the variable was found, false otherwise
+bool
+setLocal(as_object& locals, const std::string& varname, const as_value& val)
+{
+    string_table& st = getStringTable(locals);
+    Property* prop = locals.getOwnProperty(st.find(varname));
+    if (!prop) return false;
+    prop->setValue(locals, val);
+    return true;
+}
+
+as_object*
+getElement(as_object* obj, string_table::key key)
+{
+    DisplayObject* d = obj->displayObject();
+    if (d) return d->pathElement(key);
+    return obj->get_path_element(key);
+}
+
+}
+
 as_value as_environment::undefVal;
 
 as_environment::as_environment(VM& vm)
@@ -108,25 +170,19 @@ as_environment::get_variable(const std::string& varname,
             varname.find(':') == std::string::npos)
         {
             // Consider it all a path ...
-                as_object* target = find_object(varname, &scopeStack); 
+            as_object* target = find_object(varname, &scopeStack); 
             if ( target ) 
             {
                 // ... but only if it resolves to a sprite
-                MovieClip* m = target->to_movie();
-                if ( m ) return as_value(m);
+                DisplayObject* d = target->displayObject();
+                MovieClip* m = d ? d->to_movie() : 0;
+                if (m) return as_value(getObject(m));
             }
         }
         return get_variable_raw(varname, scopeStack, retTarget);
     }
 }
 
-as_value
-as_environment::get_variable(const std::string& varname) const
-{
-    static ScopeStack empty_scopeStack;
-    return get_variable(varname, empty_scopeStack);
-}
-
 static bool
 validRawVariableName(const std::string& varname)
 {
@@ -168,8 +224,7 @@ as_environment::get_variable_raw(const std::string& varname,
     // Check the scope stack.
     for (size_t i = scopeStack.size(); i > 0; --i)
     {
-        // const_cast needed due to non-const as_object::get_member 
-        as_object* obj = const_cast<as_object*>(scopeStack[i-1].get());
+        as_object* obj = scopeStack[i-1];
         if (obj && obj->get_member(key, &val))
         {
             // Found the var in with context.
@@ -183,33 +238,31 @@ as_environment::get_variable_raw(const std::string& varname,
     }
 
     // Check locals for getting them
-    if ( swfVersion < 6 ) // for SWF6 and up locals should be in the scope stack
-    {
-        if ( findLocal(varname, val, retTarget) ) 
-        {
-            return val;
-        }
-    }
-
+    // for SWF6 and up locals should be in the scope stack
+    if (swfVersion < 6 && findLocal(varname, val, retTarget)) return val;
 
     // Check current target members. TODO: shouldn't target be in scope stack ?
     if (m_target)
     {
-        if (m_target->get_member(key, &val)) {
+        as_object* obj = getObject(m_target);
+        assert(obj);
+        if (obj->get_member(key, &val)) {
 #ifdef GNASH_DEBUG_GET_VARIABLE
             log_debug("Found %s in target %p", varname, m_target->getTarget());
 #endif
-            if ( retTarget ) *retTarget = m_target;
+            if ( retTarget ) *retTarget = obj;
             return val;
         }
     }
     else if ( _original_target ) // this only for swf5+ ?
     {
-        if (_original_target->get_member(key, &val)) {
+        as_object* obj = getObject(_original_target);
+        assert(obj);
+        if (obj->get_member(key, &val)) {
 #ifdef GNASH_DEBUG_GET_VARIABLE
             log_debug("Found %s in original target %s", varname, _original_target->getTarget());
 #endif
-            if ( retTarget ) *retTarget = _original_target;
+            if ( retTarget ) *retTarget = obj;
             return val;
         }
     }
@@ -219,7 +272,7 @@ as_environment::get_variable_raw(const std::string& varname,
 #ifdef GNASH_DEBUG_GET_VARIABLE
         log_debug("Took %s as this, returning original target %s", varname, _original_target->getTarget());
 #endif
-        val.set_as_object(_original_target);
+        val.set_as_object(getObject(_original_target));
         if ( retTarget ) *retTarget = NULL; // correct ??
         return val;
     }
@@ -268,8 +321,7 @@ as_environment::delVariableRaw(const std::string& varname,
     // Check the with-stack.
     for (size_t i = scopeStack.size(); i > 0; --i)
     {
-        // const_cast needed due to non-const as_object::get_member 
-        as_object* obj = const_cast<as_object*>(scopeStack[i-1].get());
+        as_object* obj = scopeStack[i-1];
         if (obj)
         {
             std::pair<bool,bool> ret = obj->delProperty(varkey);
@@ -288,7 +340,7 @@ as_environment::delVariableRaw(const std::string& varname,
 
 
     // Try target
-    std::pair<bool,bool> ret = m_target->delProperty(varkey);
+    std::pair<bool,bool> ret = getObject(m_target)->delProperty(varkey);
     if ( ret.first )
     {
         return ret.second;
@@ -300,14 +352,6 @@ as_environment::delVariableRaw(const std::string& varname,
     return _vm.getGlobal()->delProperty(varkey).second;
 }
 
-// varname must be a plain variable name; no path parsing.
-as_value
-as_environment::get_variable_raw(const std::string& varname) const
-{
-    static ScopeStack empty_scopeStack;
-    return get_variable_raw(varname, empty_scopeStack);
-}
-
 // Given a path to variable, set its value.
 void
 as_environment::set_variable(const std::string& varname, const as_value& val,
@@ -319,12 +363,12 @@ as_environment::set_variable(const std::string& varname, const as_value& val,
     );
 
     // Path lookup rigamarole.
-    as_object* target = m_target;
+    as_object* target = getObject(m_target);
     std::string path;
     std::string var;
     //log_debug(_("set_variable(%s, %s)"), varname, val);
-    if ( parse_path(varname, path, var) )
-    {
+
+    if (parse_path(varname, path, var)) {
         target = find_object(path, &scopeStack); 
         if (target)
         {
@@ -343,13 +387,6 @@ as_environment::set_variable(const std::string& varname, const as_value& val,
     }
 }
 
-void
-as_environment::set_variable(const std::string& varname, const as_value& val)
-{
-    static ScopeStack empty_scopeStack;
-    set_variable(varname, val, empty_scopeStack);
-}
-
 // No path rigamarole.
 void
 as_environment::set_variable_raw(const std::string& varname,
@@ -365,50 +402,28 @@ as_environment::set_variable_raw(const std::string& varname,
     }
 
     VM& vm = _vm;
-    int swfVersion = vm.getSWFVersion();
     string_table& st = vm.getStringTable();
     string_table::key varkey = st.find(varname);
 
-    if ( swfVersion < 6 ) 
+    // in SWF5 and lower, scope stack should just contain 'with' elements 
+
+    // Check the with-stack.
+    for (size_t i = scopeStack.size(); i > 0; --i)
     {
-        // in SWF5 and lower, scope stack should just contain 'with' elements 
-
-        // Check the with-stack.
-        for (size_t i = scopeStack.size(); i > 0; --i)
-        {
-            // const_cast needed due to non-const as_object::get_member 
-            as_object* obj = const_cast<as_object*>(scopeStack[i-1].get());
-            if (obj && obj->set_member(varkey, val, 0, true) )
-            {
-                return;
-            }
+        as_object* obj = scopeStack[i-1];
+        if (obj && obj->set_member(varkey, val, true)) {
+            return;
         }
-
-        // Check locals for setting them
-        if ( setLocal(varname, val) ) return;
-
-    }
-    else // SWF >= 6
-    {
-
-        // Check the scope-stack (would include locals)
-        //
-        for (size_t i = scopeStack.size(); i > 0; --i)
-        {
-            // const_cast needed due to non-const as_object::get_member 
-            as_object* obj = const_cast<as_object*>(scopeStack[i-1].get());
-            if (obj && obj->set_member(varkey, val, 0, true))
-            {
-        return;
-            }
-        }
-
     }
     
+    const int swfVersion = vm.getSWFVersion();
+    if (swfVersion < 6 && setLocal(varname, val)) return;
+    
     // TODO: shouldn't m_target be in the scope chain ?
-    //assert(m_target);
-    if ( m_target ) m_target->set_member(varkey, val);
-    else if ( _original_target ) _original_target->set_member(varkey, val);
+    if (m_target) getObject(m_target)->set_member(varkey, val);
+    else if (_original_target) {
+        getObject(_original_target)->set_member(varkey, val);
+    }
     else
     {
         log_error("as_environment(%p)::set_variable_raw(%s, %s): "
@@ -418,14 +433,6 @@ as_environment::set_variable_raw(const std::string& varname,
     }
 }
 
-void
-as_environment::set_variable_raw( const std::string& varname,
-        const as_value& val)
-{
-    static ScopeStack empty_scopeStack;
-    set_variable_raw(varname, val, empty_scopeStack);
-}
-
 // Set/initialize the value of the local variable.
 void
 as_environment::set_local(const std::string& varname, const as_value& val)
@@ -443,10 +450,10 @@ as_environment::set_local(const std::string& varname, const as_value& val)
     else
     {
         // Not in frame; create a new local var.
-        assert( ! varname.empty() ); // null varnames are invalid!
-        as_object* locals = _localFrames.back().locals;
+        assert(!varname.empty()); // null varnames are invalid!
+        as_object& locals = _localFrames.back().locals();
         //locals.push_back(as_environment::frame_slot(varname, val));
-        locals->set_member(varkey, val);
+        locals.set_member(varkey, val);
     }
 }
     
@@ -458,11 +465,10 @@ as_environment::declare_local(const std::string& varname)
     if ( ! findLocal(varname, tmp) )
     {
         // Not in frame; create a new local var.
-        assert( ! _localFrames.empty() );
+        assert(!_localFrames.empty());
         assert( ! varname.empty() );    // null varnames are invalid!
-        as_object* locals = _localFrames.back().locals;
-        //locals.push_back(as_environment::frame_slot(varname, as_value()));
-        locals->set_member(_vm.getStringTable().find(varname), as_value());
+        as_object& locals = _localFrames.back().locals();
+        locals.set_member(_vm.getStringTable().find(varname), as_value());
     }
 }
 
@@ -487,16 +493,16 @@ as_environment::parse_path(const std::string& var_path_in, std::string& path,
     log_debug("path: %s, var: %s", thePath, theVar);
 #endif
 
-    if ( thePath.empty() ) return false;
+    if (thePath.empty()) return false;
 
     // this check should be performed by callers (getvariable/setvariable
     // in particular)
     size_t pathlen = thePath.length();
-    size_t i = pathlen-1;
+    size_t i = pathlen - 1;
     size_t consecutiveColons = 0;
-    while ( i && thePath[i--] == ':' )
+    while (i && thePath[i--] == ':')
     {
-        if ( ++consecutiveColons > 1 )
+        if (++consecutiveColons > 1)
         {
 #ifdef DEBUG_TARGET_FINDING 
             log_debug("path '%s' ends with too many colon chars, not "
@@ -518,8 +524,8 @@ as_environment::parse_path(const std::string& var_path, as_object** target,
 {
     std::string path;
     std::string var;
-    if ( ! parse_path(var_path, path, var) ) return false;
-        as_object* target_ptr = find_object(path); 
+    if (!parse_path(var_path, path, var)) return false;
+    as_object* target_ptr = find_object(path); 
     if ( ! target_ptr ) return false;
 
     target_ptr->get_member(_vm.getStringTable().find(var), &val);
@@ -533,28 +539,29 @@ static const char*
 next_slash_or_dot(const char* word)
 {
     for (const char* p = word; *p; p++) {
-    if (*p == '.' && p[1] == '.') {
-        p++;
-    } else if (*p == '.' || *p == '/' || *p == ':') {
-        return p;
-    }
+        if (*p == '.' && p[1] == '.') {
+            p++;
+        } else if (*p == '.' || *p == '/' || *p == ':') {
+            return p;
+        }
     }
     
     return NULL;
 }
 
-// Find the sprite/movie referenced by the given path.
+/// Find the sprite/movie referenced by the given path.
 //
-// Supports both /slash/syntax and dot.syntax
+/// Supports both /slash/syntax and dot.syntax
 //
+/// @return     The found DisplayObject or 0 if it is not a DisplayObject.
 DisplayObject*
 as_environment::find_target(const std::string& path_in) const
 {
     as_object* o = find_object(path_in);
-    if ( o ) return o->toDisplayObject(); // can be NULL (not a DisplayObject)...
-    else return NULL;
+    return get<DisplayObject>(o); 
 }
 
+
 as_object*
 as_environment::find_object(const std::string& path,
         const ScopeStack* scopeStack) const
@@ -563,20 +570,19 @@ as_environment::find_object(const std::string& path,
     log_debug(_("find_object(%s) called"), path);
 #endif
 
-    if (path.empty())
-    {
+    if (path.empty()) {
 #ifdef DEBUG_TARGET_FINDING 
         log_debug(_("Returning m_target (empty path)"));
 #endif
-        return m_target; // or should we return the *original* path ?
+        return getObject(m_target); // or should we return the *original* path ?
     }
     
     VM& vm = _vm;
     string_table& st = vm.getStringTable();
-    int swfVersion = vm.getSWFVersion(); 
+    int swfVersion = vm.getSWFVersion();
 
     as_object* env = 0;
-    env = m_target; 
+    env = getObject(m_target); 
 
     bool firstElementParsed = false;
     bool dot_allowed = true;
@@ -586,31 +592,28 @@ as_environment::find_object(const std::string& path,
     {
         // Absolute path.  Start at the (AS) root (handle _lockroot)
         MovieClip* root = 0;
-        if ( m_target ) root = m_target->getAsRoot();
+        if (m_target) root = m_target->getAsRoot();
         else {
-            if ( _original_target )
-            {
+            if (_original_target) {
                 log_debug("current target is undefined on "
                         "as_environment::find_object, we'll use original");
                 root = _original_target->getAsRoot();
             }
-            else
-            {
+            else {
                 log_debug("both current and original target are undefined "
                         "on as_environment::find_object, we'll return 0");
                 return 0;
             }
         }
 
-        if ( ! *(++p) )
-        {
+        if (!*(++p)) {
 #ifdef DEBUG_TARGET_FINDING 
             log_debug(_("Path is '/', return the root (%p)"), (void*)root);
 #endif
-            return root; // that's all folks.. 
+            return getObject(root); // that's all folks.. 
         }
 
-        env = root;
+        env = getObject(root);
         firstElementParsed = true;
         dot_allowed = false;
 
@@ -620,8 +623,7 @@ as_environment::find_object(const std::string& path,
 
     }
 #ifdef DEBUG_TARGET_FINDING 
-    else
-    {
+    else {
         log_debug(_("Relative path, start at (%s)"), m_target->getTarget());
     }
 #endif
@@ -631,35 +633,30 @@ as_environment::find_object(const std::string& path,
     std::string subpart;
     while (1)
     {
-        while ( *p == ':' ) ++p;
+        while (*p == ':') ++p;
 
         // No more components to scan
-        if ( ! *p )
-        {
+        if (!*p) {
 #ifdef DEBUG_TARGET_FINDING 
             log_debug(_("Path is %s, returning whatever we were up to"), path);
 #endif
-
             return env;
         }
 
 
         const char* next_slash = next_slash_or_dot(p);
         subpart = p;
-        if (next_slash == p)
-        {
+        if (next_slash == p) {
             IF_VERBOSE_ASCODING_ERRORS(
                 log_aserror(_("invalid path '%s' (p=next_slash=%s)"),
                 path, next_slash);
             );
             return NULL;
         }
-        else if (next_slash)
-        {
-            if ( *next_slash == '.' )
+        else if (next_slash) {
+            if (*next_slash == '.')
             {
-                if ( ! dot_allowed )
-                {
+                if (!dot_allowed) {
                     IF_VERBOSE_ASCODING_ERRORS(
                     log_aserror(_("invalid path '%s' (dot not allowed "
                             "after having seen a slash)"), path);
@@ -667,8 +664,7 @@ as_environment::find_object(const std::string& path,
                     return NULL;
                 }
             }
-            else if ( *next_slash == '/' )
-            {
+            else if (*next_slash == '/') {
                 dot_allowed = false;
             }
 
@@ -679,8 +675,7 @@ as_environment::find_object(const std::string& path,
         assert(subpart[0] != ':');
 
         // No more components to scan
-        if ( subpart.empty() )
-        {
+        if (subpart.empty()) {
 #ifdef DEBUG_TARGET_FINDING 
             log_debug(_("No more subparts, env is %p"), (void*)env);
 #endif
@@ -689,53 +684,42 @@ as_environment::find_object(const std::string& path,
 
         string_table::key subpartKey = st.find(subpart);
 
-        if ( ! firstElementParsed )
-        {
-            as_object* element = NULL;
+        if (!firstElementParsed) {
+            as_object* element(0);
 
             do {
-
                 // Try scope stack
-                if ( scopeStack )
-                {
+                if (scopeStack) {
                     for (size_t i = scopeStack->size(); i > 0; --i)
                     {
-                        // const_cast needed due to non-const 
-                        // as_object::get_member 
-                        as_object* obj = 
-                            const_cast<as_object*>((*scopeStack)[i-1].get());
+                        as_object* obj = (*scopeStack)[i-1];
                         
-                        element = obj->get_path_element(subpartKey);
-                        if ( element ) break;
+                        element = getElement(obj, subpartKey);
+                        if (element) break;
                     }
-                    if ( element ) break;
+                    if (element) break;
                 }
 
                 // Try current target  (if any)
-                assert(env == m_target);
-                if ( env )
-                {
-                    element = env->get_path_element(subpartKey);
-                    if ( element ) break;
+                assert(env == getObject(m_target));
+                if (env) {
+                    element = getElement(env, subpartKey);
+                    if (element) break;
                 }
-                // else if ( _original_target) // TODO: try orig target too ?
 
                 // Looking for _global ?
                 as_object* global = _vm.getGlobal();
-                if ( swfVersion > 5 && subpartKey == NSV::PROP_uGLOBAL )
+                if (swfVersion > 5 && subpartKey == NSV::PROP_uGLOBAL)
                 {
                     element = global;
                     break;
                 }
-
                 // Try globals
-                element = global->get_path_element(subpartKey);
-                //if ( element ) break;
+                element = getElement(global, subpartKey);
 
             } while (0);
 
-            if ( ! element ) 
-            {
+            if (!element) {
 #ifdef DEBUG_TARGET_FINDING 
                 log_debug("subpart %s of path %s not found in any "
                 "scope stack element", subpart, path);
@@ -746,8 +730,7 @@ as_environment::find_object(const std::string& path,
             env = element;
             firstElementParsed = true;
         }
-        else
-        {
+        else {
 
             assert(env);
 
@@ -756,9 +739,8 @@ as_environment::find_object(const std::string& path,
                     "%p"), subpart, (void *)env);
 #endif
 
-            as_object* element = env->get_path_element(subpartKey);
-            if ( ! element )
-            {
+            as_object* element = getElement(env, subpartKey);
+            if (!element) {
 #ifdef DEBUG_TARGET_FINDING 
                 log_debug(_("Path element %s not found in "
                             "object %p"), subpart, (void *)env);
@@ -781,46 +763,28 @@ as_environment::get_version() const
     return _vm.getSWFVersion();
 }
 
-static void
-dump(const as_environment::Registers& r, std::ostream& out) 
-{
-    for (size_t i=0; i<r.size(); ++i)
-    {
-        if (i) out << ", ";
-        out << i << ':' << '"' << r[i] << '"';
-    }
-}
-
 void
 as_environment::dump_local_registers(std::ostream& out) const
 {
     if ( _localFrames.empty() ) return;
     out << "Local registers: ";
-#ifndef DUMP_LOCAL_REGISTERS_IN_ALL_CALL_FRAMES
-    dump(_localFrames.back().registers, out);
-#else
     for (CallStack::const_iterator it=_localFrames.begin(),
-            itEnd=_localFrames.end();
-            it != itEnd; ++it)
-    {
-        if ( it != _localFrames.begin() ) out << " | ";
-        dump(it->registers, out);
+            itEnd=_localFrames.end(); it != itEnd; ++it) {
+        if (it != _localFrames.begin()) out << " | ";
+        out << *it;
     }
-#endif
     out << std::endl;
 }
 
 static void
-dump(const as_object* locals, std::ostream& out)
+dump(as_object& locals, std::ostream& out)
 {
     typedef std::map<std::string, as_value> PropMap;
     PropMap props;
-    const_cast<as_object*>(locals)->dump_members(props);
+    locals.dump_members(props);
     
-    //log_debug("FIXME: implement dumper for local variables now that they are simple objects");
     int count = 0;
-    for (PropMap::iterator i=props.begin(), e=props.end(); i!=e; ++i)
-    {
+    for (PropMap::iterator i=props.begin(), e=props.end(); i!=e; ++i) {
         if (count++) out << ", ";
         // TODO: define output operator for as_value !
         out << i->first << "==" << i->second;
@@ -833,17 +797,12 @@ as_environment::dump_local_variables(std::ostream& out) const
 {
     if ( _localFrames.empty() ) return;
     out << "Local variables: ";
-#ifndef DUMP_LOCAL_VARIABLES_IN_ALL_CALL_FRAMES
-    dump(_localFrames.back().locals, out);
-#else
-    for (CallStack::const_iterator it=_localFrames.begin(),
-            itEnd=_localFrames.end();
-            it != itEnd; ++it)
-    {
+    for (CallStack::iterator it=_localFrames.begin(),
+            itEnd=_localFrames.end(); it != itEnd; ++it) {
+
         if ( it != _localFrames.begin() ) out << " | ";
-        dump(it->locals, out);
+        dump(it->locals(), out);
     }
-#endif
     out << std::endl;
 }
 
@@ -856,7 +815,7 @@ as_environment::dump_global_registers(std::ostream& out) const
 
     ss << "Global registers: ";
     int defined=0;
-    for (unsigned int i=0; i<numGlobalRegisters; ++i)
+    for (unsigned int i = 0; i < numGlobalRegisters; ++i)
     {
         if ( m_global_register[i].is_undefined() ) continue;
 
@@ -868,73 +827,41 @@ as_environment::dump_global_registers(std::ostream& out) const
     if ( defined ) out << ss.str() << std::endl;
 }
 
-/*private*/
 bool
-as_environment::findLocal(const std::string& varname, as_value& ret, as_object** retTarget)
+as_environment::findLocal(const std::string& varname, as_value& ret,
+        as_object** retTarget) const
 {
-    if ( _localFrames.empty() ) return false;
-    if ( findLocal(_localFrames.back().locals, varname, ret) )
-    {
-        if ( retTarget ) *retTarget = _localFrames.back().locals;
+    if (_localFrames.empty()) return false;
+
+    as_object& locals = _localFrames.back().locals();
+
+    if (getLocal(locals, varname, ret)) {
+        if (retTarget) *retTarget = &locals;
         return true;
     }
+
     return false;
 }
 
-/* private */
-bool
-as_environment::findLocal(as_object* locals, const std::string& name, as_value& ret)
-{
-    // TODO: get VM passed as arg
-    return locals->get_member(_vm.getStringTable().find(name), &ret);
-}
-
-/* private */
-bool
-as_environment::delLocal(as_object* locals, const std::string& varname)
-{
-    // TODO: get VM passed as arg
-    return locals->delProperty(_vm.getStringTable().find(varname)).second;
-}
-
-/* private */
 bool
 as_environment::delLocal(const std::string& varname)
 {
-    if ( _localFrames.empty() ) return false;
-    return delLocal(_localFrames.back().locals, varname);
+    if (_localFrames.empty()) return false;
+    return deleteLocal(_localFrames.back().locals(), varname);
 }
 
-/* private */
 bool
 as_environment::setLocal(const std::string& varname, const as_value& val)
 {
-    if ( _localFrames.empty() ) return false;
-    return setLocal(_localFrames.back().locals, varname, val);
-}
+    if (_localFrames.empty()) return false;
 
-/* private */
-bool
-as_environment::setLocal(as_object* locals,
-        const std::string& varname, const as_value& val)
-{
-    Property* prop = locals->getOwnProperty(_vm.getStringTable().find(varname));
-    if ( ! prop ) return false;
-    prop->setValue(*locals, val);
-    return true;
-}
-
-
-void
-as_environment::padStack(size_t /*offset*/, size_t /*count*/)
-{
-    // do nothing here, instead return undefined from top() and pop()
-    //assert( offset <= _stack.size() );
-    //m_stack.insert(m_stack.begin()+offset, count, as_value());
+    // If this name is not qualified, the compiler fails to look beyond
+    // as_environment::setLocal.
+    return gnash::setLocal(_localFrames.back().locals(), varname, val);
 }
 
 void
-as_environment::pushCallFrame(as_function* func)
+as_environment::pushCallFrame(as_function& func)
 {
 
     // The stack size can be changed by the ScriptLimits
@@ -942,7 +869,7 @@ as_environment::pushCallFrame(as_function* func)
     // TODO: override from gnashrc.
     
     // A stack size of 0 is apparently legitimate.
-    const boost::uint16_t recursionLimit = getRoot(*func).getRecursionLimit();
+    const boost::uint16_t recursionLimit = getRoot(func).getRecursionLimit();
 
     // Don't proceed if local call frames would reach the recursion limit.
     if (_localFrames.size() + 1 >= recursionLimit) {
@@ -954,7 +881,7 @@ as_environment::pushCallFrame(as_function* func)
         throw ActionLimitException(ss.str()); 
     }
 
-    _localFrames.push_back(CallFrame(func));
+    _localFrames.push_back(CallFrame(&func));
 
 }
 
@@ -968,26 +895,18 @@ as_environment::popCallFrame()
 void
 as_environment::set_target(DisplayObject* target)
 {
-    //assert(target);
-    if ( ! _original_target )
-    {
-        //assert(target); // we assume any as_environment creator sets a target too I guess..
-        // WRONG ASSUMPTION: ^^^^^^^^^ : as_value::to_primitive doesn't care about setting a target here
-
-        //log_debug("as_environment(%p)::set_target(%p): setting original target to %s", this, target, target ? target->getTarget() : "<null>");
-        _original_target = target;
-    }
+    if (!_original_target) _original_target = target;
     m_target = target;
 }
 
 void
 as_environment::add_local(const std::string& varname, const as_value& val)
 {
-    assert( ! varname.empty() );    // null varnames are invalid!
-    assert( ! _localFrames.empty() );
-    as_object* locals = _localFrames.back().locals;
-    //locals.push_back(frame_slot(varname, val));
-    locals->set_member(_vm.getStringTable().find(varname), val);
+    assert(!varname.empty());   
+    assert(!_localFrames.empty());
+
+    as_object& locals = _localFrames.back().locals();
+    locals.set_member(_vm.getStringTable().find(varname), val);
 }
 
 void
@@ -1017,18 +936,19 @@ as_environment::dump_stack(std::ostream& out, unsigned int limit) const
 unsigned int
 as_environment::setRegister(unsigned int regnum, const as_value& v)
 {
-    if ( _localFrames.empty() || _localFrames.back().registers.empty() )
-    {
-        if ( regnum >= numGlobalRegisters ) return 0;
-        m_global_register[regnum] = v;
-        return 1;
+    // If there is a call frame and it has registers, the value must be
+    // set there.
+    if (!_localFrames.empty()) {
+        CallFrame& fr = _localFrames.back();
+        if (fr.hasRegisters()) {
+            if (_localFrames.back().setRegister(regnum, v)) return 2;
+            return 0;
+        }
     }
 
-    Registers& registers = _localFrames.back().registers;
-    if ( regnum < registers.size() )
-    {
-        registers[regnum] = v;
-        return 2;
+    if (regnum < numGlobalRegisters) {
+        m_global_register[regnum] = v;
+        return 1;
     }
 
     return 0;
@@ -1037,18 +957,20 @@ as_environment::setRegister(unsigned int regnum, const as_value& v)
 unsigned int
 as_environment::getRegister(unsigned int regnum, as_value& v)
 {
-    if ( _localFrames.empty() || _localFrames.back().registers.empty() )
-    {
-        if ( regnum >= numGlobalRegisters ) return 0;
-        v = m_global_register[regnum];
-        return 1;
+    // If there is a call frame and it has registers, the value must be
+    // sought there.
+    if (!_localFrames.empty()) {
+        const CallFrame& fr = _localFrames.back();
+        if (fr.hasRegisters()) {
+            if (fr.getRegister(regnum, v)) return 2;
+            return 0;
+        }
     }
 
-    Registers& registers = _localFrames.back().registers;
-    if ( regnum < registers.size() )
-    {
-        v = registers[regnum];
-        return 2;
+    // Otherwise it can be in the global registers.
+    if (regnum < numGlobalRegisters) {
+        v = m_global_register[regnum];
+        return 1;
     }
 
     return 0;
@@ -1057,23 +979,15 @@ as_environment::getRegister(unsigned int regnum, as_value& v)
 void
 as_environment::markReachableResources() const
 {
-    for (size_t i=0; i<4; ++i)
-    {
+    for (size_t i = 0; i < 4; ++i) {
         m_global_register[i].setReachable();
     }
 
-    if ( m_target ) m_target->setReachable();
-    if ( _original_target ) _original_target->setReachable();
+    if (m_target) m_target->setReachable();
+    if (_original_target) _original_target->setReachable();
 
-    assert ( _localFrames.empty() );
-#if 1 // I think we expect the stack to be empty !
-    for (CallStack::const_iterator i=_localFrames.begin(), e=_localFrames.end(); i!=e; ++i)
-    {
-        i->markReachableResources();
-    }
-#endif
+    // _localFrames and _stack are taken care of by VM
 
-    assert ( _stack.empty() );
 }
 #endif // GNASH_USE_GC
 
@@ -1089,10 +1003,10 @@ getRoot(const as_environment& env)
     return env.getVM().getRoot();
 }
 
-Global_as*
+Global_as&
 getGlobal(const as_environment& env)
 {
-    return env.getVM().getGlobal();
+    return *env.getVM().getGlobal();
 }
 
 int
