@@ -1,6 +1,7 @@
 // as_function.cpp:  ActionScript Functions, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,9 +28,10 @@
 #include "fn_call.h"
 #include "GnashException.h"
 #include "VM.h"
-#include "Object.h" // for getObjectInterface
 #include "namedStrings.h"
 #include "NativeFunction.h"
+#include "Object.h"
+#include "DisplayObject.h"
 
 #include <iostream>
 
@@ -61,22 +63,6 @@ namespace {
 // the Function class itself, which would be a member
 // of the _global object for each movie instance.
 
-as_function::as_function(Global_as& gl, as_object* iface)
-	:
-	as_object(gl)
-{
-	int flags = PropFlags::dontDelete |
-	            PropFlags::dontEnum |
-	            PropFlags::onlySWF6Up;
-
-    init_member(NSV::PROP_uuPROTOuu, as_value(getFunctionPrototype()), flags);
-
-	if (iface) {
-		iface->init_member(NSV::PROP_CONSTRUCTOR, this); 
-		init_member(NSV::PROP_PROTOTYPE, as_value(iface));
-	}
-}
-
 as_function::as_function(Global_as& gl)
 	:
 	as_object(gl)
@@ -87,38 +73,14 @@ as_function::as_function(Global_as& gl)
 	init_member(NSV::PROP_uuPROTOuu, as_value(getFunctionPrototype()), flags);
 }
 
-
-void
-as_function::setPrototype(as_object* proto)
+const std::string&
+as_function::stringValue() const
 {
-	init_member(NSV::PROP_PROTOTYPE, as_value(proto));
+    // TODO: find out what AS3 functions return.
+    static const std::string str("[type Function]");
+    return str;
 }
 
-void
-as_function::extends(as_function& superclass)
-{
-	as_object* newproto = new as_object(superclass.getPrototype().get());
-	newproto->init_member(NSV::PROP_uuPROTOuu, superclass.getPrototype().get());
-
-    if (getSWFVersion(superclass) > 5) {
-        const int flags = PropFlags::dontEnum;
-        newproto->init_member(NSV::PROP_uuCONSTRUCTORuu, &superclass, flags); 
-    }
-
-	init_member(NSV::PROP_PROTOTYPE, as_value(newproto));
-}
-
-boost::intrusive_ptr<as_object>
-as_function::getPrototype()
-{
-	// TODO: create if not available ?
-	// TODO WARNING: what if user overwrites the 'prototype' member ?!
-	//               this function should likely return the *new*
-	//               prototype, not the old !!
-	as_value proto;
-	get_member(NSV::PROP_PROTOTYPE, &proto);
-	return proto.to_object(*VM::get().getGlobal());
-}
 
 NativeFunction*
 as_function::getFunctionConstructor()
@@ -126,8 +88,8 @@ as_function::getFunctionConstructor()
 	static NativeFunction* func = 0;
 	if ( ! func )
 	{
-        Global_as* gl = VM::get().getGlobal();
-		func = new NativeFunction(*gl, function_ctor);
+        Global_as& gl = *VM::get().getGlobal();
+		func = new NativeFunction(gl, function_ctor);
         as_object* proto = getFunctionPrototype();
 
         func->init_member(NSV::PROP_PROTOTYPE, proto);
@@ -138,38 +100,42 @@ as_function::getFunctionConstructor()
 	return func;
 }
 
-boost::intrusive_ptr<as_object>
-as_function::constructInstance(const as_environment& env, fn_call::Args& args)
+as_object*
+constructInstance(as_function& ctor, const as_environment& env,
+        fn_call::Args& args)
 {
+    Global_as& gl = getGlobal(ctor);
 
-#ifndef GNASH_USE_GC
-	assert(get_ref_count() > 0);
-#endif // GNASH_USE_GC
-
-	int swfversion = getSWFVersion(env);
-
-	as_value proto;
-    bool has_proto = get_member(NSV::PROP_PROTOTYPE, &proto);
-		
     // Create an empty object, with a ref to the constructor's prototype.
-    // TODO: The prototype should not be converted to an object!
-    as_object* newobj = new as_object();
-    if (has_proto) newobj->set_prototype(proto.to_object(*getGlobal(env)));
-    
-    // Add a __constructor__ member to the new object, but only for SWF6 up
-    // (to be checked). NOTE that we assume the builtin constructors
-    // won't set __constructor__ to some other value...
-    int flags = PropFlags::dontEnum | 
-                PropFlags::onlySWF6Up; 
+    // The function's prototype property always becomes the new object's
+    // __proto__ member, regardless of whether it is an object and regardless
+    // of its visibility.
+    as_object* newobj = new as_object(gl);
+    Property* proto = ctor.getOwnProperty(NSV::PROP_PROTOTYPE);
+    if (proto) newobj->set_prototype(proto->getValue(ctor));
 
-    newobj->init_member(NSV::PROP_uuCONSTRUCTORuu, as_value(this), flags);
+    return ctor.construct(*newobj, env, args);
+}
+
+as_object*
+as_function::construct(as_object& newobj, const as_environment& env,
+        fn_call::Args& args)
+{
+	const int swfversion = getSWFVersion(env);
+
+    // Add a __constructor__ member to the new object visible from version 6.
+    const int flags = PropFlags::dontEnum | 
+                      PropFlags::onlySWF6Up; 
+
+    newobj.init_member(NSV::PROP_uuCONSTRUCTORuu, this, flags);
 
     if (swfversion < 7) {
-        newobj->init_member(NSV::PROP_CONSTRUCTOR, as_value(this),
-               PropFlags::dontEnum);
+        newobj.init_member(NSV::PROP_CONSTRUCTOR, this, PropFlags::dontEnum);
     }
-
-    fn_call fn(newobj, env, args, newobj->get_super(), true);
+    
+    // Don't set a super so that it will be constructed only if required
+    // by the function.
+    fn_call fn(&newobj, env, args, 0, true);
     as_value ret;
 
     try {
@@ -195,19 +161,20 @@ as_function::constructInstance(const as_environment& env, fn_call::Args& args)
     // 'this' pointer. Others return a new object. This is to handle those
     // cases.
     if (isBuiltin() && ret.is_object()) {
-        newobj = ret.to_object(*getGlobal(env)).get();
+        as_object* fakeobj = ret.to_object(getGlobal(env));
 
-        newobj->init_member(NSV::PROP_uuCONSTRUCTORuu, as_value(this),
+        fakeobj->init_member(NSV::PROP_uuCONSTRUCTORuu, as_value(this),
                 flags);
 
         // Also for SWF5+ only?
         if (swfversion < 7) {
-            newobj->init_member(NSV::PROP_CONSTRUCTOR, as_value(this),
+            fakeobj->init_member(NSV::PROP_CONSTRUCTOR, as_value(this),
                     PropFlags::dontEnum);
         }
+        return fakeobj;
     }
 
-	return newobj;
+	return &newobj;
 }
 
 
@@ -228,7 +195,7 @@ function_class_init(as_object& global, const ObjectURI& uri)
 	int swf6flags = PropFlags::dontEnum | 
                     PropFlags::dontDelete | 
                     PropFlags::onlySWF6Up;
-	global.init_member(getName(uri), func, swf6flags, getNamespace(uri));
+	global.init_member(uri, func, swf6flags);
 }
 
 namespace {
@@ -242,13 +209,12 @@ getFunctionPrototype()
 	if (proto.get() == NULL) {
 
 		// Initialize Function prototype
-		proto = new as_object();
+        proto = VM::get().getGlobal()->createObject();
         
 		// We initialize the __proto__ member separately, as getObjectInterface
 		// will end up calling getFunctionPrototype again and we want that
 		// call to return the still-not-completely-constructed prototype rather
 		// then create a new one. 
-		proto->set_prototype(getObjectInterface());
 
         VM& vm = VM::get();
 
@@ -271,13 +237,22 @@ function_ctor(const fn_call& /*fn*/)
 }
 
 
+class PushFunctionArgs
+{
+public:
+    PushFunctionArgs(fn_call& fn) : _fn(fn) {}
+    void operator()(const as_value& val) {
+        _fn.pushArg(val);
+    }
+private:
+    fn_call& _fn;
+};
+
 as_value
 function_apply(const fn_call& fn)
 {
 
-	// Get function body 
-	boost::intrusive_ptr<as_function> function_obj =
-        ensureType<as_function>(fn.this_ptr);
+	as_object* function_obj = ensure<ValidThis>(fn);
 
 	// Copy new function call from old one, we'll modify 
 	// the copy only if needed
@@ -294,12 +269,18 @@ function_apply(const fn_call& fn)
 	else
 	{
 		// Get the object to use as 'this' reference
-		as_object* obj = fn.arg(0).to_object(*getGlobal(fn)).get();
+		as_object* obj = fn.arg(0).to_object(getGlobal(fn));
 
         if (!obj) obj = new as_object; 
 
         new_fn_call.this_ptr = obj;
-        new_fn_call.super = obj->get_super();
+
+        // Note: do not override fn_call::super by creating a super
+        // object, as it may not be needed. Doing so can have a very
+        // detrimental effect on memory usage!
+        // Normal supers will be created when needed in the function
+        // call.
+        new_fn_call.super = 0;
 
 		// Check for second argument ('arguments' array)
 		if (fn.nargs > 1)
@@ -315,44 +296,15 @@ function_apply(const fn_call& fn)
 			);
 
 			boost::intrusive_ptr<as_object> arg1 = 
-                fn.arg(1).to_object(*getGlobal(fn));
-
-			if (!arg1) {
-				IF_VERBOSE_ASCODING_ERRORS(
-					log_aserror(_("Second arg of Function.apply"
-						" is %s (expected array)"
-						" - considering as call with no args"),
-						fn.arg(1));
-				);
-				goto call_it;
-			}
-
-			boost::intrusive_ptr<Array_as> arg_array = 
-					boost::dynamic_pointer_cast<Array_as>(arg1);
-
-			if ( ! arg_array )
-			{
-				IF_VERBOSE_ASCODING_ERRORS(
-					log_aserror(_("Second arg of Function.apply"
-						" is of type %s, with value %s"
-						" (expected array)"
-						" - considering as call with no args"),
-						fn.arg(1).typeOf(), fn.arg(1).to_string());
-				);
-				goto call_it;
-			}
-
-			const size_t nelems = arg_array->size();
-
-			for (size_t i = 0; i < nelems; ++i) {
-				new_fn_call.pushArg(arg_array->at(i));
-			}
+                fn.arg(1).to_object(getGlobal(fn));
 
+            if (arg1) {
+                PushFunctionArgs pa(new_fn_call);
+                foreachArray(*arg1, pa);
+            }
 		}
 	}
 
-	call_it:
-
 	// Call the function 
 	as_value rv = function_obj->call(new_fn_call);
 
@@ -363,9 +315,7 @@ as_value
 function_call(const fn_call& fn)
 {
 
-	// Get function body 
-	boost::intrusive_ptr<as_function> function_obj = 
-        ensureType<as_function>(fn.this_ptr);
+	as_object* function_obj = ensure<ValidThis>(fn);
 
 	// Copy new function call from old one, we'll modify 
 	// the copy only if needed
@@ -377,7 +327,7 @@ function_call(const fn_call& fn)
 	else {
 		// Get the object to use as 'this' reference
 		as_value this_val = fn.arg(0);
-		as_object* this_ptr = this_val.to_object(*getGlobal(fn)).get();
+		as_object* this_ptr = this_val.to_object(getGlobal(fn));
 
 		if (!this_ptr) {
 			// If the first argument is not an object, we should
@@ -397,22 +347,21 @@ function_call(const fn_call& fn)
 		}
 		else {
 			new_fn_call.this_ptr = this_ptr;
-			as_object* proto = this_ptr->get_prototype().get();
-            if (proto) {
-                new_fn_call.super = this_ptr->get_super();
-            }
-            else {
-                // TODO: check this !
-                log_debug("No prototype in 'this' pointer "
-                        "passed to Function.call");
-                new_fn_call.super = function_obj->get_super();
-			}
+            // Note: do not override fn_call::super by creating a super
+            // object, as it may not be needed. Doing so can have a very
+            // detrimental effect on memory usage!
+            // Normal supers will be created when needed in the function
+            // call.
+
+            // TODO: it seems pointless to copy the old fn_call and
+            // then change almost everything...
+            new_fn_call.super = 0;
 		}
 		new_fn_call.drop_bottom();
 	}
 
 	// Call the function 
-	return (*function_obj)(new_fn_call);
+	return function_obj->call(new_fn_call);
 
 }
 
