@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,10 +17,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "Boolean_as.h"
 #include "as_object.h" // for inheritance
 #include "log.h"
@@ -30,7 +27,6 @@
 #include "NativeFunction.h"
 #include "GnashException.h"
 #include "VM.h" 
-#include "Object.h" // for getObjectInterface
 
 namespace gnash {
 
@@ -66,9 +62,9 @@ void
 boolean_class_init(as_object& where, const ObjectURI& uri)
 {
     VM& vm = getVM(where);
-    Global_as* gl = getGlobal(where);
+    Global_as& gl = getGlobal(where);
 
-    as_object* proto = gl->createObject();
+    as_object* proto = gl.createObject();
     as_object* cl = vm.getNative(107, 2);
     cl->init_member(NSV::PROP_PROTOTYPE, proto);
     proto->init_member(NSV::PROP_CONSTRUCTOR, cl);
@@ -76,8 +72,7 @@ boolean_class_init(as_object& where, const ObjectURI& uri)
     attachBooleanInterface(*proto);
     
     // Register _global.Boolean
-    where.init_member(getName(uri), cl, as_object::DefaultFlags,
-            getNamespace(uri));
+    where.init_member(uri, cl, as_object::DefaultFlags);
 
 }
 
@@ -104,7 +99,7 @@ attachBooleanInterface(as_object& o)
 as_value
 boolean_tostring(const fn_call& fn)
 {
-    Boolean_as* obj = ensureNativeType<Boolean_as>(fn.this_ptr);
+    Boolean_as* obj = ensure<ThisIsNative<Boolean_as> >(fn);
     if (obj->value()) return as_value("true");
     return as_value("false");
 }
@@ -113,7 +108,7 @@ boolean_tostring(const fn_call& fn)
 as_value
 boolean_valueof(const fn_call& fn) 
 {
-    Boolean_as* obj = ensureNativeType<Boolean_as>(fn.this_ptr);
+    Boolean_as* obj = ensure<ThisIsNative<Boolean_as> >(fn);
     return as_value(obj->value());
 }
 
