@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,13 +23,11 @@
 namespace gnash {
 
 class as_object;
+class ObjectURI;
 
 /// Initialize the global LoadVars class
-void loadvars_class_init(as_object& global);
+void loadvars_class_init(as_object& global, const ObjectURI& uri);
 
-/// Return a LoadVars instance (in case the core lib needs it)
-//std::auto_ptr<as_object> init_loadvars_instance();
-  
 } // end of gnash namespace
 
 #endif
