@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,7 +29,6 @@
 #include "log.h"
 #include "builtin_function.h" 
 #include "NativeFunction.h" 
-#include "Object.h" // for getObjectInterface
 #include "GnashNumeric.h"
 
 #include <cmath>
