@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,12 +28,13 @@
 namespace gnash {
 
 class as_object;
+class ObjectURI;
+class Global_as;
 
 /// Initialize the global Number class
-void number_class_init(as_object& global);
+void number_class_init(as_object& global, const ObjectURI& uri);
 
-/// Return a Number instance
-boost::intrusive_ptr<as_object> init_number_instance(double val);
+void registerNumberNative(as_object& global);
 
 }
 
