@@ -1,6 +1,7 @@
 // NetStream.cpp:  ActionScript class for streaming audio/video, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,9 +19,6 @@
 //
 
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 #include "NetStream_as.h"
 #include "CharacterProxy.h"
@@ -32,7 +30,6 @@
 #include "NativeFunction.h"
 #include "GnashException.h"
 #include "NetConnection_as.h"
-#include "Object.h" // for getObjectInterface
 #include "VM.h"
 #include "namedStrings.h"
 #include "movie_root.h"
@@ -49,6 +46,9 @@
 // Define the following macro to enable decoding debugging
 //#define GNASH_DEBUG_DECODING
 
+// Define the following macro to enable decoding of playhead activity
+//#define GNASH_DEBUG_PLAYHEAD
+
 namespace gnash {
 
 namespace {
@@ -93,9 +93,14 @@ NetStream_as::NetStream_as(as_object* owner)
     _videoInfoKnown(false),
     _audioDecoder(0),
     _audioInfoKnown(false),
+
+    // Playback clock starts in 'stop' mode
     _playbackClock(
             new InterruptableVirtualClock(getVM(*owner).getClock())),
+
+    // Playhead starts at position 0 with a stopped source clock
     _playHead(_playbackClock.get()), 
+
     _soundHandler(getRunResources(*owner).soundHandler()),
     _mediaHandler(media::MediaHandler::get()),
     _audioStreamer(_soundHandler),
@@ -143,7 +148,7 @@ NetStream_as::processNotify(const std::string& funcname, as_object* info_obj)
 
     string_table::key func = getStringTable(owner()).find(funcname);
 
-    owner().callMethod(func, as_value(info_obj));
+    callMethod(&owner(), func, as_value(info_obj));
 }
 
 
@@ -167,7 +172,7 @@ NetStream_as::processStatusNotifications()
     // Must be a new object every time.
     as_object* o = getStatusObject(code);
 
-    owner().callMethod(NSV::PROP_ON_STATUS, o);
+    callMethod(&owner(), NSV::PROP_ON_STATUS, o);
 }
 
 void
@@ -272,7 +277,7 @@ NetStream_as::getStatusObject(StatusCode code)
     // Enumerable and deletable.
     const int flags = 0;
 
-    as_object* o = getGlobal(owner())->createObject();
+    as_object* o = getGlobal(owner()).createObject();
     o->init_member("code",  info.first, flags);
     o->init_member("level", info.second, flags);
 
@@ -359,6 +364,13 @@ void NetStream_as::close()
     // seems to be removed before netstream is destroyed.
     _audioStreamer.detachAuxStreamer();
 
+    // Drop all information about decoders and parser
+    _videoInfoKnown = false;
+    _videoDecoder.reset();
+    _audioInfoKnown = false;
+    _audioDecoder.reset();
+    m_parser.reset();
+
     m_imageframe.reset();
 
     stopAdvanceTimer();
@@ -368,6 +380,7 @@ void NetStream_as::close()
 void
 NetStream_as::play(const std::string& c_url)
 {
+
     // It doesn't matter if the NetStream object is already streaming; this
     // starts it again, possibly with a new URL.
 
@@ -405,6 +418,9 @@ NetStream_as::play(const std::string& c_url)
         return;
     }
 
+    // Reset any previously active playback
+    close();
+
     log_security( _("Connecting to movie: %s"), url );
 
     _inputStream = _netCon->getStream(url); 
@@ -481,6 +497,13 @@ NetStream_as::initAudioDecoder(const media::AudioInfo& info)
 bool
 NetStream_as::startPlayback()
 {
+    // Make sure no old information is around
+    assert(!_videoInfoKnown);
+    assert(!_videoDecoder.get());
+    assert(!_audioInfoKnown);
+    assert(!_audioDecoder.get());
+    // TODO: assert advance timer is not running either !
+
 
     // Register advance callback. This must be registered in order for
     // status notifications to be received (e.g. streamNotFound).
@@ -525,9 +548,17 @@ NetStream_as::startPlayback()
 
     decodingStatus(DEC_BUFFERING);
 
-    // NOTE: should be paused already 
+    // NOTE: might be running due to a previous playback in progress
     _playbackClock->pause();
 
+    // NOTE: we set playhead position to 0 here but
+    //       other code should take this as a sign
+    //       that playHead should be advanced to first available
+    //       timeframe timestamp instead.
+#ifdef GNASH_DEBUG_PLAYHEAD
+     log_debug("%p.startPlayback: playHead position reset to 0", this);
+#endif
+    _playHead.seekTo(0);
     _playHead.setState(PlayHead::PLAY_PLAYING);
 
 #ifdef GNASH_DEBUG_STATUS
@@ -762,6 +793,9 @@ NetStream_as::getDecodedVideoFrame(boost::uint32_t ts)
     _audioStreamer.cleanAudioQueue();
     
     // 'newpos' will always be on a keyframe (supposedly)
+#ifdef GNASH_DEBUG_PLAYHEAD
+     log_debug("%p.seek: playHead position set to %d", this, newpos);
+#endif
     _playHead.seekTo(newpos);
     decodingStatus(DEC_BUFFERING); 
     
@@ -878,6 +912,7 @@ NetStream_as::pushDecodedAudioFrames(boost::uint32_t ts)
 
             return; 
         }
+
     }
 
     bool consumed = false;
@@ -1282,7 +1317,7 @@ NetStream_as::update()
             if (!m_imageframe.get() && 
                     _playHead.getState() != PlayHead::PLAY_PAUSED)
             {
-                log_debug("refreshing video frame for the first time");
+                //log_debug("refreshing video frame for the first time");
                 refreshVideoFrame(true);
             }
 
@@ -1300,6 +1335,28 @@ NetStream_as::update()
         _playbackClock->resume();
     }
 
+    // If playhead position needs to be updated
+    // is set to Set playhead to first available frame, if any
+    // TODO: use another flag to signify 'initialization-needed'
+    boost::uint64_t curPosition = _playHead.getPosition();
+    if ( curPosition == 0 )
+    {
+        boost::uint64_t firstFrameTimestamp;
+        if ( m_parser->nextFrameTimestamp(firstFrameTimestamp) )
+        {
+             _playHead.seekTo(firstFrameTimestamp);
+#ifdef GNASH_DEBUG_PLAYHEAD
+            log_debug("%p.advance: playHead position set to timestamp of first frame: %d", this, firstFrameTimestamp);
+#endif
+        }
+#ifdef GNASH_DEBUG_PLAYHEAD
+        else
+        {
+            log_debug("%p.advance: playHead position is 0 and parser still doesn't have a frame to set it to", this);
+        }
+#endif
+    }
+
     // Find video frame with the most suited timestamp in the video queue,
     // and put it in the output image frame.
     refreshVideoFrame();
@@ -1562,7 +1619,7 @@ BufferedAudioStreamer::cleanAudioQueue()
 {
     boost::mutex::scoped_lock lock(_audioQueueMutex);
 
-    deleteAllChecked(_audioQueue);
+    deleteChecked(_audioQueue.begin(), _audioQueue.end());
 
     _audioQueue.clear();
 }
@@ -1580,7 +1637,7 @@ netstream_new(const fn_call& fn)
     if (fn.nargs) {
 
         NetConnection_as* nc;
-        if (isNativeType(fn.arg(0).to_object(*getGlobal(fn)).get(), nc)) {
+        if (isNativeType(fn.arg(0).to_object(getGlobal(fn)), nc)) {
             ns->setNetCon(nc);
         }
         else {
@@ -1602,7 +1659,7 @@ netstream_new(const fn_call& fn)
 as_value
 netstream_close(const fn_call& fn)
 {
-    NetStream_as* ns = ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     ns->close();
     return as_value();
 }
@@ -1610,8 +1667,7 @@ netstream_close(const fn_call& fn)
 as_value
 netstream_pause(const fn_call& fn)
 {
-    NetStream_as* ns = 
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     
     // mode: -1 ==> toogle, 0==> pause, 1==> play
     NetStream_as::PauseMode mode = NetStream_as::pauseModeToggle;
@@ -1629,8 +1685,7 @@ netstream_pause(const fn_call& fn)
 as_value
 netstream_play(const fn_call& fn)
 {
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
 
     if (!fn.nargs)
     {
@@ -1657,9 +1712,7 @@ netstream_play(const fn_call& fn)
 as_value
 netstream_seek(const fn_call& fn)
 {
-    NetStream_as* ns = 
-        ensureNativeType<NetStream_as>(fn.this_ptr);
-
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     boost::uint32_t time = 0;
     if (fn.nargs > 0)
     {
@@ -1676,8 +1729,7 @@ netstream_setbuffertime(const fn_call& fn)
 
     //GNASH_REPORT_FUNCTION;
 
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
 
     // TODO: should we do anything if given no args ?
     //       are we sure setting bufferTime to 0 is what we have to do ?
@@ -1697,8 +1749,7 @@ netstream_setbuffertime(const fn_call& fn)
 as_value
 netstream_attachAudio(const fn_call& fn)
 {
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     UNUSED(ns);
 
     LOG_ONCE(log_unimpl("NetStream.attachAudio"));
@@ -1709,8 +1760,7 @@ netstream_attachAudio(const fn_call& fn)
 as_value
 netstream_attachVideo(const fn_call& fn)
 {
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     UNUSED(ns);
 
     LOG_ONCE(log_unimpl("NetStream.attachVideo"));
@@ -1721,8 +1771,7 @@ netstream_attachVideo(const fn_call& fn)
 as_value
 netstream_publish(const fn_call& fn)
 {
-    NetStream_as* ns = 
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     UNUSED(ns);
 
     LOG_ONCE(log_unimpl("NetStream.publish"));
@@ -1733,8 +1782,7 @@ netstream_publish(const fn_call& fn)
 as_value
 netstream_receiveAudio(const fn_call& fn)
 {
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     UNUSED(ns);
 
     LOG_ONCE(log_unimpl("NetStream.receiveAudio"));
@@ -1745,8 +1793,7 @@ netstream_receiveAudio(const fn_call& fn)
 as_value
 netstream_receiveVideo(const fn_call& fn)
 {
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     UNUSED(ns);
 
     LOG_ONCE(log_unimpl("NetStream.receiveVideo"));
@@ -1757,8 +1804,7 @@ netstream_receiveVideo(const fn_call& fn)
 as_value
 netstream_send(const fn_call& fn)
 {
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     UNUSED(ns);
 
     LOG_ONCE(log_unimpl("NetStream.send"));
@@ -1772,8 +1818,7 @@ netstream_time(const fn_call& fn)
 {
     //GNASH_REPORT_FUNCTION;
 
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
 
     assert(fn.nargs == 0); // we're a getter
     return as_value(double(ns->time()/1000.0));
@@ -1783,9 +1828,7 @@ netstream_time(const fn_call& fn)
 as_value
 netstream_bytesloaded(const fn_call& fn)
 {
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
-
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     if ( ! ns->isConnected() )
     {
         return as_value();
@@ -1798,9 +1841,7 @@ netstream_bytesloaded(const fn_call& fn)
 as_value
 netstream_bytestotal(const fn_call& fn)
 {
-    NetStream_as* ns = 
-        ensureNativeType<NetStream_as>(fn.this_ptr);
-
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     if ( ! ns->isConnected() )
     {
         return as_value();
@@ -1813,9 +1854,7 @@ netstream_bytestotal(const fn_call& fn)
 as_value
 netstream_currentFPS(const fn_call& fn)
 {
-    NetStream_as* ns = 
-        ensureNativeType<NetStream_as>(fn.this_ptr);
-
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     if ( ! ns->isConnected() )
     {
         return as_value();
@@ -1830,8 +1869,7 @@ netstream_currentFPS(const fn_call& fn)
 as_value
 netstream_bufferLength(const fn_call& fn)
 {
-    NetStream_as* ns = 
-        ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
 
     // NetStream_as::bufferLength returns milliseconds, we want
     // to return *fractional* seconds.
@@ -1843,9 +1881,7 @@ netstream_bufferLength(const fn_call& fn)
 as_value
 netstream_bufferTime(const fn_call& fn)
 {
-    NetStream_as* ns =
-        ensureNativeType<NetStream_as>(fn.this_ptr);
-
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     // We return bufferTime in seconds
     double ret = ns->bufferTime() / 1000.0;
     return as_value(ret);
@@ -1855,7 +1891,7 @@ netstream_bufferTime(const fn_call& fn)
 as_value
 netstream_liveDelay(const fn_call& fn)
 {
-    NetStream_as* ns = ensureNativeType<NetStream_as>(fn.this_ptr);
+    NetStream_as* ns = ensure<ThisIsNative<NetStream_as> >(fn);
     UNUSED(ns);
 
     LOG_ONCE(log_unimpl("NetStream.liveDelay getter/setter"));
@@ -1869,19 +1905,19 @@ netstream_liveDelay(const fn_call& fn)
 void
 attachNetStreamInterface(as_object& o)
 {
-    Global_as* gl = getGlobal(o);
+    Global_as& gl = getGlobal(o);
     VM& vm = getVM(o);
     
     o.init_member("close", vm.getNative(2101, 0));
-    o.init_member("pause", gl->createFunction(netstream_pause));
-    o.init_member("play", gl->createFunction(netstream_play));
-    o.init_member("seek", gl->createFunction(netstream_seek));
+    o.init_member("pause", gl.createFunction(netstream_pause));
+    o.init_member("play", gl.createFunction(netstream_play));
+    o.init_member("seek", gl.createFunction(netstream_seek));
     o.init_member("setBufferTime", vm.getNative(2101, 4));
     o.init_member("attachAudio", vm.getNative(2101, 1));
     o.init_member("attachVideo", vm.getNative(2101, 2));
-    o.init_member("publish", gl->createFunction(netstream_publish));
-    o.init_member("receiveAudio", gl->createFunction(netstream_receiveAudio));
-    o.init_member("receiveVideo", gl->createFunction(netstream_receiveVideo));
+    o.init_member("publish", gl.createFunction(netstream_publish));
+    o.init_member("receiveAudio", gl.createFunction(netstream_receiveAudio));
+    o.init_member("receiveVideo", gl.createFunction(netstream_receiveVideo));
     o.init_member("send", vm.getNative(2101, 3));
 
     // Properties
@@ -1934,7 +1970,7 @@ executeTag(const SimpleBuffer& _buffer, as_object& thisPtr)
 	}
 
 	log_debug("Calling %s(%s)", funcName, arg);
-	thisPtr.callMethod(funcKey, arg);
+	callMethod(&thisPtr, funcKey, arg);
 }
 
 } // anonymous namespace
