@@ -1,6 +1,6 @@
 // URLVariables_as.h:  ActionScript 3 "URLVariables" class, for Gnash.
 //
-//   Copyright (C) 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,9 +20,6 @@
 #ifndef GNASH_ASOBJ3_URLVARIABLES_H
 #define GNASH_ASOBJ3_URLVARIABLES_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 
 namespace gnash {
