@@ -1,6 +1,6 @@
 // XMLNode_as.cpp:  ActionScript "XMLNode" class, for Gnash.
 //
-//   Copyright (C) 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,14 +17,9 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 #include "xml/XMLNode_as.h"
 #include "xml/XMLDocument_as.h"
-#include "Array_as.h"
-#include "Object.h"
 #include "VM.h"
 #include "log.h"
 #include "fn_call.h"
@@ -35,6 +30,8 @@
 #include "string_table.h"
 #include "PropertyList.h"
 #include "Global_as.h"
+#include "Object.h"
+#include "Array_as.h"
 
 #include <boost/bind.hpp>
 #include <string>
@@ -46,12 +43,14 @@ namespace gnash {
 
 // Function Prototypes
 namespace {
-    void enumerateAttributes(const XMLNode_as& node, PropertyList::SortedPropertyList&attributes);
-    bool prefixMatches(const PropertyList::SortedPropertyList::value_type& val,
+    void enumerateAttributes(const XMLNode_as& node,
+            as_object::SortedPropertyList& attributes);
+    bool prefixMatches(const as_object::SortedPropertyList::value_type& val,
             const std::string& prefix);
     bool namespaceMatches(
-            const PropertyList::SortedPropertyList::value_type& val,
+            const as_object::SortedPropertyList::value_type& val,
             const std::string& ns);    
+
     as_value xmlnode_new(const fn_call& fn);
     as_value xmlnode_nodeName(const fn_call& fn);
     as_value xmlnode_nodeValue(const fn_call& fn);
@@ -74,29 +73,28 @@ namespace {
     as_value xmlnode_toString(const fn_call& fn);
     as_value xmlnode_localName(const fn_call& fn);
     as_value xmlnode_prefix(const fn_call& fn);
-    as_value xmlnode_ctor(const fn_call& fn);
     void attachXMLNodeInterface(as_object& o);
     void attachXMLNodeStaticInterface(as_object& o);
-    as_object* getXMLNodeInterface();
 }
 
-XMLNode_as::XMLNode_as()
-	: as_object(getXMLNodeInterface()),
-    	  _parent(0),
-    	  _attributes(new as_object),
-    	  _type(Element)
+XMLNode_as::XMLNode_as(Global_as& gl)
+    :
+    _global(gl),
+    _object(0),
+    _parent(0),
+    _attributes(new as_object),
+    _childNodes(0),
+    _type(Element)
 {
-    //log_debug("%s: %p", __PRETTY_FUNCTION__, this);
-#ifdef DEBUG_MEMORY_ALLOCATION
-    log_debug(_("\tCreating XMLNode data at %p"), this);
-#endif
 }
 
 XMLNode_as::XMLNode_as(const XMLNode_as& tpl, bool deep)
     :
-    as_object(getXMLNodeInterface()),
-    _parent(0), // _parent is never implicitly copied
-    _attributes(0),
+    _global(tpl._global),
+    _object(0),
+    _parent(0), 
+    _attributes(new as_object),
+    _childNodes(0),
     _name(tpl._name),
     _value(tpl._value),
     _type(tpl._type)
@@ -106,7 +104,6 @@ XMLNode_as::XMLNode_as(const XMLNode_as& tpl, bool deep)
         const Children& from=tpl._children;
         for (Children::const_iterator it=from.begin(), itEnd=from.end();
                         it != itEnd; ++it) {
-
             _children.push_back(new XMLNode_as(*(*it), deep));
         }
     }
@@ -114,34 +111,86 @@ XMLNode_as::XMLNode_as(const XMLNode_as& tpl, bool deep)
 
 XMLNode_as::~XMLNode_as()
 {
-#ifdef DEBUG_MEMORY_ALLOCATION
-    log_debug(_("\tDeleting XMLNode data %s with as_value %s at %p"),
-            this->_name, this->as_value, this);
-#endif
+    clearChildren();
 }
-	
+
+as_object*
+XMLNode_as::object() 
+{
+
+    // This is almost the same as if the XMLNode constructor were called,
+    // but not quite. There is no __constructor__ property, and when we
+    // override _global.XMLNode, we can show that it is not called.
+    if (!_object) {
+        as_object* o = _global.createObject();
+        as_object* xn =
+            _global.getMember(NSV::CLASS_XMLNODE).to_object(_global);
+        if (xn) {
+            o->set_prototype(xn->getMember(NSV::PROP_PROTOTYPE));
+            o->init_member(NSV::PROP_CONSTRUCTOR, xn);
+        }
+        o->setRelay(this);
+        setObject(o);
+    }
+    return _object;
+}
+
+void
+XMLNode_as::updateChildNodes()
+{
+    if (!_childNodes) return;
+
+    // Clear array of all elements.
+    _childNodes->set_member(NSV::PROP_LENGTH, 0.0);
+
+    if (_children.empty()) return;
+
+    string_table& st = getStringTable(_global);
+
+    // Set up the array without calling push()!
+    const size_t size = _children.size();
+    Children::const_iterator it = _children.begin();
+    for (size_t i = 0; i != size; ++i, ++it) {
+        XMLNode_as* node = *it;
+        const string_table::key key = arrayKey(st, i);
+        _childNodes->set_member(key, node->object());
+
+        // All elements are set to readonly.
+        _childNodes->set_member_flags(key, PropFlags::readOnly);
+    }
+}
+
+as_object*
+XMLNode_as::childNodes()
+{
+    if (!_childNodes) {
+        _childNodes = _global.createArray();
+        updateChildNodes();
+    }
+    return _childNodes;
+}
+
 bool
 XMLNode_as::hasChildNodes()
 {
-    if (_children.size()) return true;
-    return false;
+    return !_children.empty();
 }
 
-boost::intrusive_ptr<XMLNode_as>
+XMLNode_as*
 XMLNode_as::firstChild()
 {
-    if ( _children.empty() ) return NULL;
+    if (_children.empty()) return 0;
     return _children.front();
 }
 
-boost::intrusive_ptr<XMLNode_as>
+XMLNode_as*
 XMLNode_as::cloneNode(bool deep)
 {
-    boost::intrusive_ptr<XMLNode_as> newnode = new XMLNode_as(*this, deep);
+    XMLNode_as* newnode = new XMLNode_as(*this, deep);
     return newnode;
 }
 
-boost::intrusive_ptr<XMLNode_as>
+XMLNode_as*
 XMLNode_as::lastChild()
 {
 	if (_children.empty()) {
@@ -152,23 +201,27 @@ XMLNode_as::lastChild()
 }
 
 void
-XMLNode_as::appendChild(boost::intrusive_ptr<XMLNode_as> node)
+XMLNode_as::removeChild(XMLNode_as* node)
 {
-    assert (node);
-
-    boost::intrusive_ptr<XMLNode_as> oldparent = node->getParent();
-    node->setParent(this);
-    _children.push_back(node);
-    if ( oldparent ) {
-        oldparent->_children.remove(node);
-    }
-
+    node->setParent(0);
+    _children.remove(node);
+    updateChildNodes();
 }
 
 void
-XMLNode_as::insertBefore(boost::intrusive_ptr<XMLNode_as> newnode,
-        boost::intrusive_ptr<XMLNode_as> pos)
+XMLNode_as::appendChild(XMLNode_as* node)
 {
+    assert(node);
+    node->setParent(this);
+    _children.push_back(node);
+    updateChildNodes();
+}
+
+void
+XMLNode_as::insertBefore(XMLNode_as* newnode, XMLNode_as* pos)
+{
+    assert(_object);
+
 	// find iterator for positional parameter
     Children::iterator it = std::find(_children.begin(), _children.end(), pos);
     if (it == _children.end()) {
@@ -180,23 +233,14 @@ XMLNode_as::insertBefore(boost::intrusive_ptr<XMLNode_as> newnode,
     }
 
     _children.insert(it, newnode);
-    boost::intrusive_ptr<XMLNode_as> oldparent = newnode->getParent();
-    newnode->setParent(this);
-    if (oldparent) {
-        oldparent->_children.remove(newnode);
-    }
-}
 
-// \brief removes the specified XML object from its parent. Also
-// deletes all descendants of the node.
-void
-XMLNode_as::removeNode()
-{
-    boost::intrusive_ptr<XMLNode_as> oldparent = getParent();
-    if (oldparent) {
-        oldparent->_children.remove(this);
+    XMLNode_as* parent = newnode->getParent();
+    if (parent) {
+        parent->removeChild(newnode);
     }
-    _parent = 0;
+    
+    newnode->setParent(this);
+    updateChildNodes();
 }
 
 XMLNode_as *
@@ -209,9 +253,9 @@ XMLNode_as::previousSibling()
     for (Children::iterator itx = _parent->_children.begin();
             itx != _parent->_children.end(); itx++) {
 
-        if (itx->get() == this) return previous_node;
+        if (*itx == this) return previous_node;
 		
-        previous_node = itx->get();
+        previous_node = *itx;
     }
 
     return 0;
@@ -229,8 +273,8 @@ XMLNode_as::nextSibling()
     for (Children::reverse_iterator itx = _parent->_children.rbegin();
             itx != _parent->_children.rend(); ++itx) {
 
-        if (itx->get() == this) return previous_node;
-		previous_node = itx->get();
+        if (*itx == this) return previous_node;
+		previous_node = *itx;
     }
 
     return 0;
@@ -246,7 +290,7 @@ void
 XMLNode_as::setAttribute(const std::string& name, const std::string& value)
 {
     if (_attributes) {
-        string_table& st = getStringTable(*this);
+        string_table& st = getStringTable(_global);
         _attributes->set_member(st.find(name), value);
     }
 }
@@ -255,8 +299,8 @@ bool
 XMLNode_as::getPrefixForNamespace(const std::string& ns, std::string& prefix)
 {
     XMLNode_as* node = this;
-    PropertyList::SortedPropertyList::const_iterator it; 
-    PropertyList::SortedPropertyList attrs;
+    as_object::SortedPropertyList::const_iterator it; 
+    as_object::SortedPropertyList attrs;
     
     while (node) {
         enumerateAttributes(*node, attrs);
@@ -292,8 +336,8 @@ void
 XMLNode_as::getNamespaceForPrefix(const std::string& prefix, std::string& ns)
 {
     XMLNode_as* node = this;
-    PropertyList::SortedPropertyList::const_iterator it; 
-    PropertyList::SortedPropertyList attrs;
+    as_object::SortedPropertyList::const_iterator it; 
+    as_object::SortedPropertyList attrs;
     
     while (node) {
 
@@ -331,6 +375,22 @@ XMLNode_as::extractPrefix(std::string& prefix)
     return true;
 }
 
+void
+XMLNode_as::clearChildren()
+{
+    for (Children::const_iterator it = _children.begin(), e = _children.end();
+            it != e; ++it) {
+        const XMLNode_as* node = *it;
+        if (!node->_object) {
+            delete node;
+        }
+    }
+    _children.clear();
+
+    // Reset so that it is reinitialized on next access.
+    _childNodes = 0;
+}
+
 void
 XMLNode_as::stringify(const XMLNode_as& xml, std::ostream& xmlout, bool encode) 
 {
@@ -351,19 +411,19 @@ XMLNode_as::stringify(const XMLNode_as& xml, std::ostream& xmlout, bool encode)
         xmlout << "<" << nodeName;
     
         // Process the attributes, if any
-        PropertyList::SortedPropertyList attrs;
+        as_object::SortedPropertyList attrs;
         enumerateAttributes(xml, attrs);
         if (!attrs.empty()) {
 
-            for (PropertyList::SortedPropertyList::iterator i = 
+            for (as_object::SortedPropertyList::iterator i = 
                     attrs.begin(), e = attrs.end(); i != e; ++i) { 
-                XMLDocument_as::escape(i->second);
+                escapeXML(i->second);
                 xmlout << " " << i->first << "=\"" << i->second << "\"";
             }
         }
 
     	// If the node has no content, just close the tag now
-    	if ( nodeValue.empty() && xml._children.empty() ) {
+    	if (nodeValue.empty() && xml._children.empty()) {
     		xmlout << " />";
             return;
     	}
@@ -377,14 +437,13 @@ XMLNode_as::stringify(const XMLNode_as& xml, std::ostream& xmlout, bool encode)
     // Node as_value first, then children
     if (type == Text)
     {
-        as_object* global = getVM(xml).getGlobal();
-        assert(global);
+        Global_as& gl = xml._global;
 
         // Insert entities.
         std::string escaped(nodeValue);
-        XMLDocument_as::escape(escaped);
+        escapeXML(escaped);
         const std::string& val = encode ? 
-            global->callMethod(NSV::PROP_ESCAPE, escaped).to_string() :
+            callMethod(&gl, NSV::PROP_ESCAPE, escaped).to_string() :
             escaped;
 
 	    xmlout << val;
@@ -402,26 +461,23 @@ XMLNode_as::stringify(const XMLNode_as& xml, std::ostream& xmlout, bool encode)
     }
 }
 
-#ifdef GNASH_USE_GC
 void
-XMLNode_as::markReachableResources() const
+XMLNode_as::setReachable() 
 {
 	// Mark children
     std::for_each(_children.begin(), _children.end(),
-            boost::mem_fn(&as_object::setReachable));
-
-	// Mark parent
-	if (_parent) _parent->setReachable();
+            boost::mem_fn(&XMLNode_as::setReachable));
 
 	// Mark attributes object
 	if (_attributes) _attributes->setReachable();
 
-	markAsObjectReachable();
+    if (_object) _object->setReachable();
+
+    if (_childNodes) _childNodes->setReachable();
 }
-#endif // GNASH_USE_GC
 
 void
-XMLNode_as::registerNative(as_object& where)
+registerXMLNodeNative(as_object& where)
 {
     VM& vm = getVM(where);
     vm.registerNative(xmlnode_cloneNode, 253, 1);
@@ -435,31 +491,19 @@ XMLNode_as::registerNative(as_object& where)
 }
 
 void
-XMLNode_as::init(as_object& where, const ObjectURI& uri)
+xmlnode_class_init(as_object& where, const ObjectURI& uri)
 {
-    Global_as* gl = getGlobal(where);
-    as_object* proto = getXMLNodeInterface();
-    as_object* cl = gl->createClass(&xmlnode_new, proto);
+    Global_as& gl = getGlobal(where);
+    as_object* proto = gl.createObject();
+    attachXMLNodeInterface(*proto);
+    as_object* cl = gl.createClass(&xmlnode_new, proto);
 
-    where.init_member(getName(uri), cl, as_object::DefaultFlags,
-            getNamespace(uri));
+    where.init_member(uri, cl, as_object::DefaultFlags);
 
 }
 
 namespace {
 
-as_object*
-getXMLNodeInterface()
-{
-    static boost::intrusive_ptr<as_object> o;
-    if ( o == NULL ) {
-        o = new as_object(getObjectInterface());
-        attachXMLNodeInterface(*o);
-    }
-    return o.get();
-}
-
-
 void
 attachXMLNodeInterface(as_object& o)
 {
@@ -508,68 +552,86 @@ as_value
 xmlnode_new(const fn_call& fn)
 {
     
-    XMLNode_as *xml_obj = new XMLNode_as;
-    if ( fn.nargs > 0 )
-    {
-        xml_obj->nodeTypeSet(XMLNode_as::NodeType(fn.arg(0).to_int()));
-        if (fn.nargs > 1)
+    as_object* obj = ensure<ValidThis>(fn);
+
+    if (!fn.nargs) {
+        return as_value();
+    }
+
+    std::auto_ptr<XMLNode_as> xml(new XMLNode_as(getGlobal(fn)));
+    xml->nodeTypeSet(XMLNode_as::NodeType(toInt(fn.arg(0))));
+
+    if (fn.nargs > 1) {
+        const std::string& str = fn.arg(1).to_string();
+        switch (xml->nodeType())
         {
-            xml_obj->nodeValueSet(fn.arg(1).to_string());
+            case XMLNode_as::Element:
+                xml->nodeNameSet(str);
+                break;
+            default:
+                xml->nodeValueSet(str);
+                break;
         }
     }
     
-    return as_value(xml_obj);
+    // This sets the relay!
+    xml->setObject(obj);
+    obj->setRelay(xml.release());
+
+    return as_value();
 }
 
 
 as_value
 xmlnode_appendChild(const fn_call& fn)
 {
-//    GNASH_REPORT_FUNCTION;
 
-	boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+	XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
-	if ( ! fn.nargs )
-	{
+	if (!fn.nargs) {
 		IF_VERBOSE_ASCODING_ERRORS(
-		log_aserror(_("XMLNode::appendChild() needs at least one argument"));
+            log_aserror(_("XMLNode::appendChild() needs at least one "
+                    "argument"));
 		);
 		return as_value();
 	}
 
-	boost::intrusive_ptr<XMLNode_as> xml_obj = 
-        boost::dynamic_pointer_cast<XMLNode_as>(fn.arg(0).to_object(*getGlobal(fn)));	
-	if ( ! xml_obj )
-	{
+	XMLNode_as* node;
+    if (!isNativeType(fn.arg(0).to_object(getGlobal(fn)), node)) {
 		IF_VERBOSE_ASCODING_ERRORS(
-		log_aserror(_("First argument to XMLNode::appendChild() is not "
-                "an XMLNode"));
+            log_aserror(_("First argument to XMLNode::appendChild() is not "
+                    "an XMLNode"));
 		);
 		return as_value();
 	}
 
-	ptr->appendChild(xml_obj);
-	return as_value(); // undefined
+    XMLNode_as* parent = node->getParent();
+    if (parent) {
+        parent->removeChild(node);
+    }
+	ptr->appendChild(node);
+
+	return as_value(); 
 
 }
 
 as_value
 xmlnode_cloneNode(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     bool deep = false;
     if (fn.nargs > 0) deep = fn.arg(0).to_bool();
 
-    boost::intrusive_ptr<XMLNode_as> newnode = ptr->cloneNode(deep);
-    return as_value(newnode.get());
+    as_object* newnode = ptr->cloneNode(deep)->object();
+    return as_value(newnode);
 }
 
 
 as_value
 xmlnode_insertBefore(const fn_call& fn)
 {
-	boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+	XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
 	if ( fn.nargs < 2 )
 	{
@@ -581,10 +643,9 @@ xmlnode_insertBefore(const fn_call& fn)
 		return as_value();
 	}
 
-	boost::intrusive_ptr<XMLNode_as> newnode = 
-        boost::dynamic_pointer_cast<XMLNode_as>(fn.arg(0).to_object(*getGlobal(fn)));
+	XMLNode_as* newnode;
 
-	if (!newnode) {
+    if (!isNativeType(fn.arg(0).to_object(getGlobal(fn)), newnode)) {
 		IF_VERBOSE_ASCODING_ERRORS(
 		std::stringstream ss; fn.dump_args(ss);
 		log_aserror(_("First argument to XMLNode.insertBefore(%s) is not "
@@ -593,10 +654,9 @@ xmlnode_insertBefore(const fn_call& fn)
 		return as_value();
 	}
 
-	boost::intrusive_ptr<XMLNode_as> pos = 
-        boost::dynamic_pointer_cast<XMLNode_as>(fn.arg(1).to_object(*getGlobal(fn)));
+	XMLNode_as* pos;
 
-	if (!pos) {
+    if (!isNativeType(fn.arg(1).to_object(getGlobal(fn)), pos)) {
 		IF_VERBOSE_ASCODING_ERRORS(
         std::stringstream ss; fn.dump_args(ss);
 		log_aserror(_("Second argument to XMLNode.insertBefore(%s) is not "
@@ -614,7 +674,7 @@ xmlnode_insertBefore(const fn_call& fn)
 as_value
 xmlnode_getNamespaceForPrefix(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     if (!fn.nargs) {
         return as_value();
     }
@@ -630,7 +690,7 @@ xmlnode_getNamespaceForPrefix(const fn_call& fn)
 as_value
 xmlnode_getPrefixForNamespace(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     if (!fn.nargs) {
         return as_value();
     }
@@ -657,7 +717,7 @@ xmlnode_getPrefixForNamespace(const fn_call& fn)
 as_value
 xmlnode_namespaceURI(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     // Read-only property
     
@@ -678,7 +738,7 @@ xmlnode_namespaceURI(const fn_call& fn)
 
     // Search recursively for a namespace. Return an empty string
     // if none found.
-    XMLNode_as* node = ptr.get();
+    XMLNode_as* node = ptr;
     while (node && node->getNamespaceURI().empty()) {
         node = node->getParent();
     }
@@ -694,7 +754,7 @@ xmlnode_namespaceURI(const fn_call& fn)
 as_value
 xmlnode_prefix(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     // Read-only property
     
@@ -716,7 +776,7 @@ xmlnode_prefix(const fn_call& fn)
 as_value
 xmlnode_localName(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     // Read-only property
     
@@ -741,9 +801,10 @@ xmlnode_localName(const fn_call& fn)
 as_value
 xmlnode_removeNode(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     
-    ptr->removeNode();
+    XMLNode_as* parent = ptr->getParent();
+    if (parent) parent->removeChild(ptr);
     return as_value();
 }
 
@@ -751,9 +812,8 @@ xmlnode_removeNode(const fn_call& fn)
 as_value
 xmlnode_toString(const fn_call& fn)
 {
-    //GNASH_REPORT_FUNCTION;
     
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     
     std::stringstream ss;
     ptr->toString(ss);
@@ -765,7 +825,7 @@ xmlnode_toString(const fn_call& fn)
 as_value
 xmlnode_hasChildNodes(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     return as_value(ptr->hasChildNodes());
 }
 
@@ -773,7 +833,7 @@ xmlnode_hasChildNodes(const fn_call& fn)
 as_value
 xmlnode_nodeValue(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     as_value rv;
     rv.set_null();
     
@@ -793,7 +853,7 @@ xmlnode_nodeValue(const fn_call& fn)
 as_value
 xmlnode_nodeName(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     as_value rv;
     rv.set_null();
 
@@ -801,8 +861,7 @@ xmlnode_nodeName(const fn_call& fn)
         const std::string& val = ptr->nodeName();
         if ( ! val.empty() ) rv = val;
     }
-    else
-    {
+    else {
         ptr->nodeNameSet(fn.arg(0).to_string());
     }
     return rv;
@@ -812,7 +871,7 @@ xmlnode_nodeName(const fn_call& fn)
 as_value
 xmlnode_nodeType(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     return as_value(ptr->nodeType());
 }
 
@@ -820,7 +879,7 @@ xmlnode_nodeType(const fn_call& fn)
 as_value
 xmlnode_attributes(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
 
     as_object* attrs = ptr->getAttributes();
     if (attrs) return as_value(attrs);
@@ -839,13 +898,13 @@ xmlnode_attributes(const fn_call& fn)
 as_value
 xmlnode_firstChild(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> node = ptr->firstChild();
+    XMLNode_as* node = ptr->firstChild();
     if (node) {
-       rv = node.get();
+        rv = node->object();
     }
 
     return rv;
@@ -861,12 +920,14 @@ xmlnode_firstChild(const fn_call& fn)
 as_value
 xmlnode_lastChild(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> node = ptr->lastChild();
-    if (node) rv = node.get();
+    XMLNode_as* node = ptr->lastChild();
+    if (node) {
+        rv = node->object();
+    }
 
     return rv;
 }
@@ -878,10 +939,10 @@ xmlnode_nextSibling(const fn_call& fn)
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     XMLNode_as *node = ptr->nextSibling();
     if (node) {
-	rv = node;
+        rv = node->object();
     }
     return rv;
 }
@@ -893,10 +954,10 @@ xmlnode_previousSibling(const fn_call& fn)
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     XMLNode_as *node = ptr->previousSibling();
     if (node) {
-	rv = node;
+        rv = node->object();
     }
     return rv;
 }
@@ -908,43 +969,30 @@ xmlnode_parentNode(const fn_call& fn)
     as_value rv;
     rv.set_null();
 
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
     XMLNode_as *node = ptr->getParent();
     if (node) {
-	rv = node;
+        rv = node->object();
     }
     return rv;
 }
 
-
 as_value
 xmlnode_childNodes(const fn_call& fn)
 {
-    boost::intrusive_ptr<XMLNode_as> ptr = ensureType<XMLNode_as>(fn.this_ptr);
-    boost::intrusive_ptr<Array_as> ary = new Array_as();
-
-    typedef XMLNode_as::Children Children;
-
-    Children& child = ptr->childNodes();
-    for ( Children::const_iterator it=child.begin(), itEnd=child.end();
-                    it != itEnd; ++it )
-    {
-        boost::intrusive_ptr<XMLNode_as> node = *it;
-        ary->push(as_value(node.get()));
-    }
-
-    return as_value(ary.get());
+    XMLNode_as* ptr = ensure<ThisIsNative<XMLNode_as> >(fn);
+    return ptr->childNodes();
 }
 
 
 void
 enumerateAttributes(const XMLNode_as& node,
-            PropertyList::SortedPropertyList& attrs)
+        as_object::SortedPropertyList& attrs)
 {
     attrs.clear();
-    const as_object* obj = node.getAttributes();
+    as_object* obj = node.getAttributes();
     if (obj) {
-        obj->enumerateProperties(attrs);
+        enumerateProperties(*obj, attrs);
     }
 
 }
@@ -952,7 +1000,7 @@ enumerateAttributes(const XMLNode_as& node,
 /// Return true if this attribute is a namespace specifier and the
 /// namespace matches.
 bool
-namespaceMatches(const PropertyList::SortedPropertyList::value_type& val,
+namespaceMatches(const as_object::SortedPropertyList::value_type& val,
         const std::string& ns)
 {
     StringNoCaseEqual noCaseCompare;
@@ -962,7 +1010,7 @@ namespaceMatches(const PropertyList::SortedPropertyList::value_type& val,
 
 
 bool
-prefixMatches(const PropertyList::SortedPropertyList::value_type& val,
+prefixMatches(const as_object::SortedPropertyList::value_type& val,
         const std::string& prefix)
 {
     const std::string& name = val.first;
@@ -980,14 +1028,6 @@ prefixMatches(const PropertyList::SortedPropertyList::value_type& val,
     return noCaseCompare(prefix, name.substr(6));
 }
 
-as_value
-xmlnode_ctor(const fn_call& /*fn*/)
-{
-    boost::intrusive_ptr<as_object> obj = new XMLNode_as;
-
-    return as_value(obj.get()); // will keep alive
-}
-
 } // anonymous namespace 
 } // gnash namespace
 // local Variables:
