@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -50,7 +51,6 @@ public:
 	/// @param func
 	///	The C function to call when this as_function is invoked.
 	/// 	For classes, the function pointer is the constructor.
-	///
 	builtin_function(Global_as& gl, ASFunction func)
 		:
 		as_function(gl),
@@ -58,44 +58,11 @@ public:
 	{
 	}
 
-	/// Construct a builtin function with the given interface (possibly none)
-	//
-	/// @param func
-	///	The C function to call when this as_function is invoked.
-	/// 	For classes, the function pointer is the constructor.
-	///
-	/// @param iface
-	///	The interface of this class (will be inherited by
-	///	instances of this class)
-	/// 	If the given interface is NULL no interface will be
-	/// 	provided. Use the constructor taking a single argument
-	///	    to get a default interface instead.
-    /// @param useThisAsCtor
-    ///     Used only by getFunctionConstructor().
-	builtin_function(Global_as& gl, ASFunction func, as_object* iface)
-		:
-		as_function(gl, iface),
-		_func(func)
-	{
-	}
-
 	/// Invoke this function or this Class constructor
-	virtual as_value operator()(const fn_call& fn)
+	virtual as_value call(const fn_call& fn)
 	{
-		// Real native functions don't put self on the CallStack
-		// (they never end up in an arguments.caller).
-		// This is, for example, the case for Array.sort when
-		// calling a custom comparator.
-		// Unfortunately gnash implements all builtin as natives
-		// while the proprietary player doesn't (for example
-		// MovieClip.meth does end up in arguments.caller).
-		//
-		// A possible short-term solution to this could be
-		// specifying for a builtin_function whether or not
-		// it should be considered 'native'.
-		// If not 'native', we'd push a CallFrame on stack...
         as_environment env = fn.env();
-		as_environment::FrameGuard guard(env, this);
+		as_environment::FrameGuard guard(env, *this);
 
 		assert(_func);
 		return _func(fn);
