@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,10 +16,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include <string>
 #include <sstream>
 #include <map>
@@ -148,7 +145,7 @@ Debugger::console(as_environment &env)
 	    // Quit Gnash.
 	  case 'Q':
 	  case 'q':
-	      exit(0);
+	      exit(EXIT_SUCCESS);
 	      break;
 	      // Continue executing.
 	  case 'c':
@@ -569,9 +566,9 @@ Debugger::dumpStackFrame(as_environment &env)
             cerr << "\t" << i << ": ";
 	    as_value val = env.bottom(i);
 // FIXME: we want to print the name of the function
-//  	    if (val.is_as_function()) {
+//  	    if (val.is_function()) {
 // //		cerr << val.get_symbol_handle() << endl;
-// 		string name = this->lookupSymbol(val.to_object(*getGlobal(fn)));
+// 		string name = this->lookupSymbol(val.to_object(getGlobal(fn)));
 // 		if (name.size()) {
 // 		    cerr << name << " ";
 // 		}
@@ -579,7 +576,7 @@ Debugger::dumpStackFrame(as_environment &env)
         cerr << env.bottom(i);
 
 	    if (val.is_object()) {
-		boost::intrusive_ptr<as_object> o = val.to_object(*getGlobal(fn));
+		boost::intrusive_ptr<as_object> o = val.to_object(getGlobal(fn));
 		string name = lookupSymbol(o.get());
 		if (name.size()) {
 		    cerr << " \"" << name << "\"";
@@ -700,7 +697,7 @@ Debugger::lookupSymbol(std::string &name)
 //    GNASH_REPORT_FUNCTION;
     if (_symbols.size()) {
 	VM& vm = VM::get(); // cache this ?
-	std::string namei = PROPNAME(name);
+	std::string namei = name;
 	std::map<void *, std::string>::const_iterator it;
 	for (it=_symbols.begin(); it != _symbols.end(); it++) {
 	    if (it->second == namei) {
@@ -718,7 +715,7 @@ Debugger::addSymbol(void *ptr, std::string name)
 {
 //    GNASH_REPORT_FUNCTION;
     VM& vm = VM::get(); // cache this ?
-    std::string namei = PROPNAME(name);
+    std::string namei = name;
     if (namei.size() > 1)
     {
 //	log_debug ("Adding symbol %s at address: %p", namei, ptr);
