@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
 #ifndef GNASH_DRAG_STATE_H
 #define GNASH_DRAG_STATE_H
 
-#include "rect.h" // for composition
+#include "SWFRect.h" // for composition
 #include "smart_ptr.h" // we keep DisplayObject being dragged by intrusive_ptr
 
 namespace gnash
@@ -36,12 +37,11 @@ class drag_state
 
 	bool _hasbounds;
 
-	/// Boundaries to constraint the drag into.
+	/// Boundaries to constrain the drag into.
 	/// Coordinates in TWIPS.
-	/// TODO: use Range2d<float> directly ?
-	rect _bounds;
+	SWFRect _bounds;
 
-	boost::intrusive_ptr<DisplayObject> _displayObject;
+	DisplayObject* _displayObject;
 
 	bool	_lock_centered;
 
@@ -86,9 +86,9 @@ public:
 	///
 	/// Note that if hasBounds() is false
 	/// the returned rectangle is the NULL
-	/// rectangle - see rect::is_null().
+	/// rectangle - see SWFRect::is_null().
 	///
-	const rect& getBounds() const { return _bounds; }
+	const SWFRect& getBounds() const { return _bounds; }
 
 	/// \brief
 	/// Set the boundaries to constraint
@@ -97,14 +97,14 @@ public:
 	/// Coordinates of the rectangle are
 	/// expected in TWIPS.
 	///
-	void setBounds(const rect& bounds) {
+	void setBounds(const SWFRect& bounds) {
 		_bounds = bounds;
 		_hasbounds = true;
 	}
 
 	/// May return NULL !!
 	DisplayObject* getCharacter() const {
-		return _displayObject.get();
+		return _displayObject;
 	}
 
 	/// Stores DisplayObject in an intrusive pointer
@@ -133,7 +133,7 @@ public:
 	/// Mark DisplayObject as reachable (if any)
 	void markReachableResources() const
 	{
-		if ( _displayObject ) _displayObject->setReachable();
+		if (_displayObject) _displayObject->setReachable();
 	}
 };
 
