@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,16 +40,6 @@ namespace gnash {
 namespace gnash {
 
 
-// The file types that Gnash can handle
-enum FileType {
-    GNASH_FILETYPE_JPEG,
-    GNASH_FILETYPE_PNG,
-    GNASH_FILETYPE_GIF,
-    GNASH_FILETYPE_SWF,
-    GNASH_FILETYPE_FLV,
-    GNASH_FILETYPE_UNKNOWN
-};
-
 /// The display quality.
 //
 /// Required for rendering and core.
