@@ -1,6 +1,7 @@
 // SWFMovieDefinition.cpp: load a SWF definition
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,9 +41,13 @@
 #include "ExportableResource.h"
 #include "GnashAlgorithm.h"
 #include "SWFParser.h"
+#include "Global_as.h"
+#include "namedStrings.h"
+#include "as_function.h"
 
 #include <boost/bind.hpp>
 #include <boost/version.hpp>
+#include <boost/thread.hpp>
 #include <iomanip>
 #include <memory>
 #include <string>
@@ -68,7 +73,7 @@ namespace {
     template<typename T> void markMappedResources(const T& t);
 }
 
-MovieLoader::MovieLoader(SWFMovieDefinition& md)
+SWFMovieLoader::SWFMovieLoader(SWFMovieDefinition& md)
 	:
 	_movie_def(md),
 	_thread(NULL),
@@ -76,7 +81,7 @@ MovieLoader::MovieLoader(SWFMovieDefinition& md)
 {
 }
 
-MovieLoader::~MovieLoader()
+SWFMovieLoader::~SWFMovieLoader()
 {
 	// we should assert _movie_def._loadingCanceled
 	// but we're not friend yet (anyone introduce us ?)
@@ -88,7 +93,7 @@ MovieLoader::~MovieLoader()
 }
 
 bool
-MovieLoader::started() const
+SWFMovieLoader::started() const
 {
 	boost::mutex::scoped_lock lock(_mutex);
 
@@ -96,7 +101,7 @@ MovieLoader::started() const
 }
 
 bool
-MovieLoader::isSelfThread() const
+SWFMovieLoader::isSelfThread() const
 {
 	boost::mutex::scoped_lock lock(_mutex);
 
@@ -114,19 +119,19 @@ MovieLoader::isSelfThread() const
 
 // static..
 void
-MovieLoader::execute(MovieLoader& ml, SWFMovieDefinition* md)
+SWFMovieLoader::execute(SWFMovieLoader& ml, SWFMovieDefinition* md)
 {
 	ml._barrier.wait(); // let _thread assignment happen before going on
 	md->read_all_swf();
 }
 
 bool
-MovieLoader::start()
+SWFMovieLoader::start()
 {
 #ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
     std::abort();
 #endif
-	// don't start MovieLoader thread() which rely
+	// don't start SWFMovieLoader thread() which rely
 	// on boost::thread() returning before they are executed. Therefore,
 	// we must employ locking.
 	// Those tests do seem a bit redundant, though...
@@ -173,7 +178,7 @@ SWFMovieDefinition::~SWFMovieDefinition()
             e = m_playlist.end(); i != e; ++i)
 	{
 		PlayList& pl = i->second;
-        deleteAllChecked(pl);
+        deleteChecked(pl.begin(), pl.end());
     }
 
 }
@@ -330,7 +335,7 @@ SWFMovieDefinition::readHeader(std::auto_ptr<IOChannel> in,
 	_str.reset(new SWFStream(_in.get()));
 
 	m_frame_size.read(*_str);
-	// If the rect is malformed, rect::read would already 
+	// If the SWFRect is malformed, SWFRect::read would already 
 	// print an error. We check again here just to give 
 	// the error are better context.
 	if ( m_frame_size.is_null() )
@@ -418,9 +423,10 @@ SWFMovieDefinition::ensure_frame_loaded(size_t framenum) const
 }
 
 Movie*
-SWFMovieDefinition::createMovie(DisplayObject* parent)
+SWFMovieDefinition::createMovie(Global_as& gl, DisplayObject* parent)
 {
-	return new SWFMovie(this, parent);
+    as_object* o = getObjectWithPrototype(gl, NSV::CLASS_MOVIE_CLIP);
+	return new SWFMovie(o, this, parent);
 }
 
 
@@ -773,7 +779,14 @@ SWFMovieDefinition::importResources(
                         "movie '%s'"), symbolName, source->get_url());
 			continue;
         }
-        else if (Font* f = dynamic_cast<Font*>(res.get()))
+
+#ifdef DEBUG_EXPORTS
+        log_debug("Exporting symbol %s imported from source %s",
+            symbolName, source->get_url());
+#endif
+        export_resource(symbolName, res.get());
+
+        if (Font* f = dynamic_cast<Font*>(res.get()))
 		{
 			// Add this shared font to the currently-loading movie.
 			add_font(id, f);
