@@ -1,6 +1,7 @@
 // action_buffer.cpp:  holds actions for later execution, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,11 +18,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "action_buffer.h"
 #include "log.h"
 #include "SWFStream.h"
@@ -139,7 +135,7 @@ action_buffer::process_decl_dict(size_t start_pc, size_t stop_pc) const
     // Index the strings.
     for (int ct = 0; ct < count; ct++) {
         // Point into the current action buffer.
-        m_dictionary[ct] = (const char*) &m_buffer[3 + i];
+        m_dictionary[ct] = reinterpret_cast<const char*>(&m_buffer[3 + i]);
 
         while (m_buffer[3 + i]) {
             // safety check.
@@ -164,7 +160,8 @@ action_buffer::process_decl_dict(size_t start_pc, size_t stop_pc) const
 // and prevents malformed instructions causing a read past the
 // end of the buffer.
 static std::string
-disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength)
+disasm_instruction(const unsigned char* instruction_data,
+        size_t maxBufferLength)
 {
 
     using namespace SWF;
@@ -192,7 +189,7 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
         assert (maxBufferLength >= 3);
         ss << " (";
         fmt = ash[action_id].getArgFormat();
-        assert(fmt != ARG_NONE);
+        
         size_t length = (instruction_data[1] | (instruction_data[2] << 8));
         
         // Assert that length without the three initial bytes
@@ -205,28 +202,28 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
                 break;
 
             case ARG_HEX:
-            {
                 ss << hexify(&instruction_data[3], length, false) << " ";
                 break;
-            }
 
             case ARG_STR:
             {
-                std::string str = hexify(&instruction_data[3], length, true);
+                const std::string str =
+                    hexify(&instruction_data[3], length, true);
                 ss << "\"" << str.c_str() << "\"";
                 break;
             }
 
             case ARG_U8:
             {
-                int val = instruction_data[3];
+                const int val = instruction_data[3];
                 ss << " " << val;
                 break;
             }
 
             case ARG_U16:
             {
-                int val = instruction_data[3] | (instruction_data[4] << 8);
+                const int val =
+                    instruction_data[3] | (instruction_data[4] << 8);
                 ss << " " << val;
                 break;
             }
@@ -242,16 +239,15 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
             case ARG_PUSH_DATA:
             {
                 size_t i = 0;
-                while (i < length)
-                {
+                while (i < length) {
                     int type = instruction_data[3 + i];
                     
-                    // This should be safe, as the buffer is always 0-terminated.
-                    if ( i++ ) ss << ", ";
+                    // This should be safe, as the buffer is always
+                    // 0-terminated.
+                    if (i++) ss << ", ";
 
                     switch (type)
                     {
-
                         case 0:
                         {
                             // string
@@ -512,7 +508,7 @@ convert_float_little(const void *p)
         break;
     case 0x3f80:    // big-endian host
         {
-        const boost::uint8_t *cp = (const boost::uint8_t *) p;
+        const boost::uint8_t *cp = static_cast<const boost::uint8_t *>(p);
         u.c.c0 = cp[3];
         u.c.c1 = cp[2];
         u.c.c2 = cp[1];
@@ -533,7 +529,7 @@ convert_float_little(const void *p)
 double
 convert_double_wacky(const void *p)
 {
-    const boost::uint8_t *cp = (const boost::uint8_t *)p;    // Handy uchar version
+    const boost::uint8_t *cp = static_cast<const boost::uint8_t *>(p);    // Handy uchar version
     union {
         double    d;
         boost::uint64_t    i;
@@ -565,7 +561,7 @@ convert_double_wacky(const void *p)
     // exactly representable and that has different values in the
     // four 16-bit words.
     // 0x11223344 is represented as 0x41b1 2233 4400 0000 (bigendian)
-    u.d = (double) 0x11223344;
+    u.d = static_cast<double>(0x11223344);
     switch (u.s.s0) {
     case 0x0000:    // pure little-endian host: swap words only.
         memcpy(&u.l.l1, cp, 4);
