@@ -1,6 +1,7 @@
 // sprite_definition.h:  Holds immutable data for a sprite, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,9 +27,8 @@
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "movie_definition.h" // for inheritance
 #include "log.h"
-#include "rect.h"
+#include "SWFRect.h"
 #include "StringPredicates.h" // StringNoCaseLessThan
-#include "TagLoadersTable.h"
 
 // Forward declarations
 namespace gnash {
@@ -64,18 +64,7 @@ public:
 	///	to be already positioned right before the frame count
 	///
 	sprite_definition(movie_definition& m, SWFStream& in,
-            const RunInfo& runInfo);
-
-	/// \brief
-	/// Create an empty sprite
-	//
-	/// A sprite definition consists of a series control tags.
-	///
-	/// @param m
-	///	the Top-Level movie_definition this sprite is read
-	///	from (not a sprite_definition!)
-	sprite_definition(movie_definition& m);
-
+            const RunResources& runResources);
 
 	/// Destructor, releases playlist data
 	~sprite_definition();
@@ -87,17 +76,21 @@ public:
 	///
 	/// @param the_class
 	///	The class constructor to associate with
-	///	new instances of this character. If NULL
+	///	new instances of this DisplayObject. If NULL
 	///	new instances will get the MovieClip interface.
 	///
 	void registerClass(as_function* the_class);
 
 	/// Get the Class registered to this definition.
-	as_function* getRegisteredClass()
+	as_function* getRegisteredClass() const
 	{
 		return registeredClass.get();
 	}
 
+    virtual void incrementLoadedFrames() {
+        ++m_loading_frame;
+    }
+
 	// overload from movie_definition
 	virtual float	get_width_pixels() const { return 1; }
 
@@ -130,10 +123,10 @@ public:
 		return m_movie_def.get_frame_rate();
 	}
 
-	const rect& get_frame_size() const
+	const SWFRect& get_frame_size() const
 	{
 		abort();
-		static const rect unused;
+		static const SWFRect unused;
 		return unused;
 	}
 
@@ -160,7 +153,7 @@ public:
 	}
 
 	/// Delegate call to associated root movie
-	virtual BitmapInfo* getBitmap(int id)
+	virtual BitmapInfo* getBitmap(int id) const
 	{
 		return m_movie_def.getBitmap(id);
 	}
@@ -169,12 +162,12 @@ public:
 	virtual void addBitmap(int /*id*/, boost::intrusive_ptr<BitmapInfo> /*im*/)
 	{
 		IF_VERBOSE_MALFORMED_SWF (
-		log_swferror(_("add_bitmap_character_def appears in sprite tags"));
+		log_swferror(_("add_bitmap_SWF::DefinitionTag appears in sprite tags"));
 		);
 	}
 
 	/// Delegate call to associated root movie
-	virtual sound_sample* get_sound_sample(int id)
+	virtual sound_sample* get_sound_sample(int id) const
 	{
 		return m_movie_def.get_sound_sample(id);
 	}
@@ -195,7 +188,7 @@ public:
 	}
 
 	// See dox in movie_definition.h
-	virtual int get_loading_sound_stream_id()
+	virtual int get_loading_sound_stream_id() const
 	{
 		return _loadingSoundStream;
 	}
@@ -224,37 +217,37 @@ public:
 	}
 
 	/// \brief
-	/// Get a character_def from this Sprite's root movie
+	/// Get a SWF::DefinitionTag from this Sprite's root movie
 	/// CharacterDictionary.
 	///
-	virtual character_def*	get_character_def(int id)
+	virtual SWF::DefinitionTag*	getDefinitionTag(int id) const
 	{
-	    return m_movie_def.get_character_def(id);
+	    return m_movie_def.getDefinitionTag(id);
 	}
 
 	/// Delegate call to associated root movie
-	virtual void add_character(int id, character_def* ch)
+	virtual void addDisplayObject(int id, SWF::DefinitionTag* ch)
 	{
-		m_movie_def.add_character(id, ch);
+		m_movie_def.addDisplayObject(id, ch);
 	}
 
 	// Create a (mutable) instance of our definition.  The
 	// instance is created to live (temporarily) on some level on
 	// the parent movie's display list.
 	//
-	// overloads from character_def
-	virtual character* create_character_instance(
-		character* parent, int id);
+	// overloads from SWF::DefinitionTag
+	virtual DisplayObject* createDisplayObject(Global_as& gl,
+            DisplayObject* parent) const;
 
+	// See dox in movie_definition.h
+	virtual void addControlTag(SWF::ControlTag* c)
+	{
+		m_playlist[m_loading_frame].push_back(c);
+	}
 
 private:
 
-	void read(SWFStream& in, const RunInfo& runInfo);
-
-	/// Tags loader table.
-	//
-	/// TODO: make it a static member, specific to sprite_definition
-	SWF::TagLoadersTable& _tag_loaders;
+	void read(SWFStream& in, const RunResources& runResources);
 
 	/// Top-level movie definition
 	/// (the definition read from SWF stream)
@@ -274,18 +267,12 @@ private:
 	// Number of frames completely parsed 
 	size_t m_loading_frame;
 
-
-	// See dox in movie_definition.h
-	virtual void	addControlTag(ControlTag* c)
-	{
-		m_playlist[m_loading_frame].push_back(c);
-	}
-
 	// See dox in movie_definition.h
 	virtual void add_frame_name(const std::string& name);
 
 	// See dox in movie_definition
-	bool get_labeled_frame(const std::string& label, size_t& frame_number);
+	bool get_labeled_frame(const std::string& label, size_t& frame_number)
+        const;
 
 	/// frame_number is 0-based
 	const PlayList* getPlaylist(size_t frame_number) const
@@ -308,7 +295,7 @@ private:
 	/// Ensure framenum frames of this sprite
 	/// have been loaded.
 	///
-	virtual bool ensure_frame_loaded(size_t framenum)
+	virtual bool ensure_frame_loaded(size_t framenum) const
 	{
 		// TODO: return false on timeout
 		while ( m_loading_frame < framenum )
@@ -323,17 +310,6 @@ private:
 		return true;
 	}
 
-	const rect&	get_bound() const {
-    // It is required that get_bound() is implemented in character definition
-    // classes. However, it makes no sense to call it for sprite definitions.
-    // get_bound() is currently only used by generic_character which normally
-    // is used only shape character definitions. See character_def.h to learn
-    // why it is virtual anyway.
-    abort(); // should not be called
-		static rect unused;
-		return unused;
-  }
-
 	/// \brief
 	/// The constructor to use for setting up the interface
 	/// for new instances of this sprite
