@@ -1,6 +1,7 @@
 // CSMTextSettingsTag.h: parse SWF CSMTextSettings tag
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,19 +17,14 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
 #ifndef GNASH_SWF_CSMTEXTSETTINGSTAG_H
 #define GNASH_SWF_CSMTEXTSETTINGSTAG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 // Forward declarations
 namespace gnash {
 	class movie_definition;
 	class SWFStream;
-    class RunInfo;
+    class RunResources;
 }
 
 namespace gnash {
@@ -39,7 +35,7 @@ public:
 
 	CSMTextSettingsTag(movie_definition& m, SWFStream& in);
 
-	static void loader(SWFStream& in, TagType tag, movie_definition& m, const RunInfo& r);
+	static void loader(SWFStream& in, TagType tag, movie_definition& m, const RunResources& r);
 
 private:
 
