@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,10 +19,6 @@
 #ifndef GNASH_SWF_DEFAULTLOADERS_H
 #define GNASH_SWF_DEFAULTLOADERS_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "dsodefs.h" //DSOEXPORT
 #include "SWF.h"
 #include "TagLoadersTable.h"
