@@ -1,6 +1,7 @@
 // DefineButtonSoundTag.h: sounds for Button DisplayObjects.
 //
-//   Copyright (C) 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -49,21 +50,10 @@ public:
 
 		ButtonSound()
 			:
-            soundID(0),
+			soundID(0),
 			sample(0)
 		{}
 
-#ifdef GNASH_USE_GC
-		/// Mark all reachable resources (for GC)
-		//
-		/// Reachable resources are:
-		///  - sound sample (sample)
-		///
-		void markReachableResources() const
-        {
-            if (sample) sample->setReachable();
-        }
-#endif // GNASH_USE_GC
 	};
 
     typedef std::vector<ButtonSound> Sounds;
@@ -77,21 +67,6 @@ public:
         assert (index < 4);
         return _sounds[index];
     }
-#ifdef GNASH_USE_GC
-    /// Mark all reachable resources (for GC)
-    //
-    /// Reachable resources are:
-    ///  - button sound infos (_sounds)
-    ///
-    void markReachableResources() const
-    {
-        for (Sounds::const_iterator i=_sounds.begin(), e = _sounds.end();
-                i != e; ++i)
-        {
-            i->markReachableResources();
-        }
-    }
-#endif // GNASH_USE_GC
 
 private:
 
