@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -101,12 +102,6 @@ public:
         return (_definitionTag);
     }
 
-#ifdef GNASH_USE_GC
-    void markReachableResources() const {
-        if (_definitionTag) _definitionTag->setReachable();
-    }
-#endif // GNASH_USE_GC
-
 private:
 
     /// SWF8 and above can have a number of filters
@@ -208,7 +203,8 @@ public:
     virtual ~DefineButtonTag();
 
     /// Create a mutable instance of our definition.
-    DisplayObject* createDisplayObject(DisplayObject* parent, int id) const;
+    DisplayObject* createDisplayObject(Global_as& gl, DisplayObject* parent)
+        const;
 
     /// Access the ButtonRecords directly. Used for modifying the
     /// Cxform by a DefineButtonCxform tag.
@@ -262,28 +258,6 @@ public:
         }
     }
     
-protected:
-
-#ifdef GNASH_USE_GC
-    /// Mark all reachable resources (for GC)
-    //
-    /// Reachable resources are:
-    ///  - button records (_button_records)
-    ///  - button sound definition (_sound)
-    ///
-    void markReachableResources() const
-    {
-        assert(isReachable());
-        for (ButtonRecords::const_iterator i = _buttonRecords.begin(),
-                e = _buttonRecords.end(); i!=e; ++i)
-        {
-            i->markReachableResources();
-        }
-
-        if (_soundTag) _soundTag->markReachableResources();
-    }
-#endif // GNASH_USE_GC
-
 private:
 
     /// DefineButton2Tag::loader also needs to create a DefineButtonTag.
