@@ -1,6 +1,6 @@
 // RemoveObjectTag.cpp: RemoveObject* tag for Gnash.
 //
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
 #include "RemoveObjectTag.h"
 #include "MovieClip.h"
-#include "swf.h" // for TagType definition
+#include "SWF.h" // for TagType definition
 #include "log.h"
 #include "SWFStream.h"
 
@@ -42,20 +42,20 @@ RemoveObjectTag::read(SWFStream& in, TagType tag)
 	}
 
     in.ensureBytes(2);
-	m_depth = in.read_u16() + character::staticDepthOffset;
+	m_depth = in.read_u16() + DisplayObject::staticDepthOffset;
 }
 
 void
 RemoveObjectTag::execute(MovieClip* m, DisplayList& dlist) const
 {
     m->set_invalidated();
-	dlist.remove_character(m_depth);
+	dlist.removeDisplayObject(m_depth);
 }
 
 /* public static */
 void
 RemoveObjectTag::loader(SWFStream& in, TagType tag, movie_definition& m,
-        const RunInfo& /*r*/)
+        const RunResources& /*r*/)
 {
     assert(tag == SWF::REMOVEOBJECT || tag == SWF::REMOVEOBJECT2);
 
