@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ public:
     {
         log_debug("Setting script limits: recursion %s, timeout %s",
                 _recursionLimit, _timeoutLimit);
-        getRoot(*m).setScriptLimits(_recursionLimit, _timeoutLimit);
+        getRoot(*getObject(m)).setScriptLimits(_recursionLimit, _timeoutLimit);
     }
 
     static void loader(SWFStream& in, TagType tag, movie_definition& m,
