@@ -1,6 +1,6 @@
 // StartSoundTag.cpp:  for Gnash.
 //
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ void
 StartSoundTag::execute(MovieClip* m, DisplayList& /* dlist */) const
 {
 
-    sound::sound_handler* handler = getRunResources(*m).soundHandler();
+    sound::sound_handler* handler = getRunResources(*getObject(m)).soundHandler();
 
     if (handler)
     {
