@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ void
 StreamSoundBlockTag::execute(MovieClip* m, DisplayList& /*dlist*/) const
 {
 
-	sound::sound_handler* handler = getRunResources(*m).soundHandler(); 
+	sound::sound_handler* handler = getRunResources(*getObject(m)).soundHandler(); 
 	if (handler)
 	{
 		// This makes it possible to stop only the stream when framejumping.
