@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -137,13 +138,7 @@ private:
 	/// calls in DOACTION block before frame actions queued by the same
 	/// cause (the latter would be pushed in the same level gotoFrame is
 	/// found)
-	///
-	/// @param expectInconsistencies
-	///	If true, don't print an error if stack is bigger or smaller
-	///	then we expect. The parameter is used when calling the
-	///	cleanup function due to a thrown ActionLimitException.
-	///
-	void cleanupAfterRun(bool expectInconsistencies=false);
+	void cleanupAfterRun();
 
 	/// the 'with' stack associated with this execution thread
 	std::vector<with_stack_entry> _withStack;
@@ -180,7 +175,7 @@ private:
 	const swf_function* _func;
 
 	/// The 'this' pointer, if this is a function call
-	boost::intrusive_ptr<as_object> _this_ptr;
+	as_object* _this_ptr;
 
 	/// Stack size at start of execution
 	size_t _initialStackSize;
@@ -196,16 +191,6 @@ private:
 
 	bool _returning;
 
-	/// Warn about a stack underrun and fix it 
-	//
-	/// The fix is padding the stack with undefined
-	/// values for the missing slots.
-	/// 
-	/// @param required
-	///	Number of items required.
-	///
-	void fixStackUnderrun(size_t required);
-
 	bool _abortOnUnload;
 
     /// Program counter (offset of current action tag)
