@@ -1,6 +1,7 @@
 // VM.cpp: the Virtual Machine class, for Gnash
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -73,7 +74,7 @@ VM::init(int version, movie_root& root, VirtualClock& clock)
     gl->registerClasses();
 
 #ifdef ENABLE_AVM2
-	_singleton->_machine = new Machine(*_singleton);
+	_singleton->_machine = new abc::Machine(*_singleton);
     _singleton->_machine->init();
 #endif
 
@@ -242,7 +243,6 @@ VM::getTime() const
 void
 VM::markReachableResources() const
 {
-#ifdef GNASH_USE_GC
 
 	_rootMovie.markReachableResources();
 
@@ -259,6 +259,23 @@ VM::markReachableResources() const
 	}
 
     if (_shLib.get()) _shLib->markReachableResources();
+
+#ifdef ALLOW_GC_RUN_DURING_ACTIONS_EXECUTION
+    /// Mark all (including unreachable) stack elements
+    for (SafeStack<as_value>::StackSize i=0, n=_stack.totalSize(); i<n; ++i)
+    {
+        _stack.at(i).setReachable();
+    }
+
+    /// Mark call stack 
+    for (CallStack::size_type i=0, n=_callStack.size(); i<n; ++i)
+    {
+        _callStack[i].markReachableResources();
+    }
+
+#else
+    assert (_callStack.empty());
+    assert (_stack.totalSize() == 0);
 #endif
 
 }
@@ -270,9 +287,8 @@ VmGcRoot::markReachableResources() const
 }
 
 void
-VM::registerNative(as_c_function_ptr fun, unsigned int x, unsigned int y)
+VM::registerNative(Global_as::ASFunction fun, unsigned int x, unsigned int y)
 {
-    //log_debug("Registering function %p as ASnative(%d, %d) ", (void*)fun, x, y);
     assert(fun);
     assert(!_asNativeTable[x][y]);
     _asNativeTable[x][y] = fun;
@@ -285,7 +301,7 @@ VM::getNative(unsigned int x, unsigned int y) const
     if (row == _asNativeTable.end()) return 0;
     FuncMap::const_iterator col = row->second.find(y);
     if (col == row->second.end()) return 0;
-    as_c_function_ptr fun = col->second;
+    Global_as::ASFunction fun = col->second;
 
     NativeFunction* f = new NativeFunction(*_global, fun);
     f->init_member(NSV::PROP_CONSTRUCTOR,
@@ -293,6 +309,104 @@ VM::getNative(unsigned int x, unsigned int y) const
     return f;
 }
 
+///////////////////////////////////////////////////////////////////////
+//
+// Value ops
+//
+///////////////////////////////////////////////////////////////////////
+
+void
+newAdd(as_value& op1, const as_value& op2, VM& vm)
+{
+    // We can't change the original value.
+    as_value r(op2);
+
+    // The order of the operations is important: op2 is converted to
+    // primitive before op1.
+
+    /// It doesn't matter if either of these fail.
+	try { convertToPrimitive(r, vm); }
+	catch (ActionTypeError& e) {}
+	
+    try { convertToPrimitive(op1, vm); }
+	catch (ActionTypeError& e) {}
+
+#if GNASH_DEBUG
+	log_debug(_("(%s + %s) [primitive conversion done]"), op1, r);
+#endif
+
+	if (op1.is_string() || r.is_string()) {
+
+		// use string semantic
+		const int version = vm.getSWFVersion();
+		convertToString(op1, vm);
+		op1.set_string(op1.to_string(version) + r.to_string(version));
+        return;
+	}
+
+    // Otherwise use numeric semantic
+    const double num1 = op1.to_number();
+    const double num2 = r.to_number();
+    op1.set_double(num2 + num1); 
+
+}
+
+void
+subtract(as_value& op1, const as_value& op2, VM& /*vm*/)
+{
+	const double num2 = op2.to_number();
+	const double num1 = op1.to_number();
+	op1.set_double(num1 - num2);
+}
+
+as_value
+newLessThan(const as_value& op1, const as_value& op2, VM& /*vm*/)
+{
+
+    as_value operand1(op1);
+    as_value operand2(op2);
+
+    try { operand1 = op1.to_primitive(as_value::NUMBER); }
+    catch (ActionTypeError& e)
+    {
+        log_debug("%s.to_primitive() threw an error during ActionNewLessThen",
+            op1);
+    }
+    
+    if (operand1.is_object() && !operand1.is_sprite()) {
+        return false;
+    }
+
+    try { operand2 = op2.to_primitive(as_value::NUMBER); }
+    catch (ActionTypeError& e)
+    {
+        log_debug("%s.to_primitive() threw an error during ActionNewLessThen",
+            op2);
+    }
+    
+    if (operand2.is_object() && !operand2.is_sprite()) {
+        return false;
+    }
+
+    if (operand1.is_string() && operand2.is_string())
+    {
+        const std::string& s1 = operand1.to_string();
+        const std::string& s2 = operand2.to_string();
+        if (s1.empty()) return false;
+        if (s2.empty()) return true;
+        return as_value(s1 < s2);
+    }
+
+    const double num1 = operand1.to_number();
+    const double num2 = operand2.to_number();
+
+    if (isNaN(num1) || isNaN(num2)) {
+        return as_value();
+    }
+    return as_value(num1 < num2);
+}
+
+
 } // end of namespace gnash
 
 
