@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,25 +19,21 @@
 #ifndef GNASH_FN_CALL_H
 #define GNASH_FN_CALL_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
-#include "as_environment.h" // for inlines (arg)
-#include "as_object.h" // for dtor visibility by boost::intrusive_ptr
+#include "as_environment.h" 
+#include "as_object.h"
 #include "smart_ptr.h"
+#include "as_value.h"
 #include "VM.h"
 
-#include <cassert> // for inlines (arg)
-#include <ostream> // for inlines (dump_args)
-#include <sstream> // for inlines (dump_args)
+#include <cassert> 
+#include <ostream>
+#include <sstream>
+#include <algorithm>
 
 // Forward declarations
 namespace gnash {
     class as_environment;
     class as_function;
-    class as_object;
-    class as_value;
     class movie_definition;
 }
 
@@ -65,6 +62,12 @@ public:
 
     FunctionArgs() {}
 
+    /// The copy constructor copies all the arguments.
+    FunctionArgs(const FunctionArgs& other)
+        :
+        _v(other._v)
+    {}
+
     FunctionArgs& operator+=(const T& t) {
         _v.push_back(t);
         return *this;
@@ -75,6 +78,15 @@ public:
         return *this;
     }
 
+    /// Mark any reachable resources
+    //
+    /// This is only for cases where the lifetime of a FunctionArgs object
+    /// extends beyond a function call.
+    void setReachable() const {
+        std::for_each(_v.begin(), _v.end(),
+                std::mem_fun_ref(&as_value::setReachable));
+    }
+
     void swap(std::vector<T>& to) {
         std::swap(_v, to);
     }
@@ -94,60 +106,18 @@ private:
 class fn_call
 {
 public:
+
     typedef FunctionArgs<as_value> Args;
 
-	/// The as_object (or a pointer derived thereof) on which this call
-	/// is taking place.
-	as_object* this_ptr;
-
-	/// The "super" object in this function call context
-	as_object* super;
-
-	/// Number of arguments to this ActionScript function call.
-    Args::size_type nargs;
-
-    /// Definition containing caller code. 0 if spontaneous (system event).
-    const movie_definition* callerDef;
-
-    fn_call(const fn_call& fn)
-        :
-        this_ptr(fn.this_ptr),
-        super(fn.super),
-		nargs(fn.nargs),
-        callerDef(fn.callerDef),
-        _new(false),
-        _env(fn._env),
-        _args(fn._args)
-	{
-	}
-
-	fn_call(const fn_call& fn, as_object* this_in, as_object* sup = 0)
-		:
-        this_ptr(this_in),
-        super(sup),
-        nargs(fn.nargs),
-        callerDef(fn.callerDef),
-        _new(false),
-		_env(fn._env),
-        _args(fn._args)
-	{
-	}
-
-	fn_call(as_object* this_in, const as_environment& env_in,
-			int nargs_in, size_t first_in, as_object* sup = 0)
-		:
-		this_ptr(this_in),
-		super(sup),
-		nargs(nargs_in),
-        callerDef(0),
-        _new(false),
-		_env(env_in)
-	{
-		assert(first_in + 1 == env_in.stack_size());
-		readArgs(env_in, first_in, nargs);
-	}
-
-	fn_call(as_object* this_in, const as_environment& env_in,
+    /// Construct a fn_call
+    //
+    /// @param isNew        Pass true if this is a constructing fn_call,
+    ///                     i.e. if it is called as a result of 'new'.
+    /// @param super        Pass an overridden super value to the function
+    ///                     call. If this is 0, the super reference will be
+    ///                     calculated from the this pointer (if that is not
+    ///                     null) whenever a function requires it.
+    fn_call(as_object* this_in, const as_environment& env_in,
             Args& args, as_object* sup = 0, bool isNew = false)
 		:
 		this_ptr(this_in),
@@ -160,7 +130,7 @@ public:
         args.swap(_args);
 	}
 
-	fn_call(as_object* this_in, const as_environment& env_in)
+    fn_call(as_object* this_in, const as_environment& env_in)
 		:
 		this_ptr(this_in),
 		super(0),
@@ -171,6 +141,34 @@ public:
 	{
 	}
 
+    /// Copy constructor
+    fn_call(const fn_call& fn)
+        :
+        this_ptr(fn.this_ptr),
+        super(fn.super),
+		nargs(fn.nargs),
+        callerDef(fn.callerDef),
+        _new(false),
+        _env(fn._env),
+        _args(fn._args)
+	{
+	}
+
+	/// The as_object (or a pointer derived thereof) on which this call
+	/// is taking place.
+	as_object* this_ptr;
+
+	/// The "super" object in this function call context
+    //
+    /// If this is 0, the super may be constructed from the this pointer.
+	as_object* super;
+
+	/// Number of arguments to this ActionScript function call.
+    Args::size_type nargs;
+
+    /// Definition containing caller code. 0 if spontaneous (system event).
+    const movie_definition* callerDef;
+
     /// Return the VM this fn_call is running from
     VM& getVM() const
     {
@@ -180,12 +178,6 @@ public:
 	/// Return true if this call is an object instantiation
 	bool isInstantiation() const
 	{
-		// Currently the as_function::constructInstance
-		// will set 'this_ptr' to NULL when calling a builtin
-		// function, so we use this info to find out.
-		// For the future, we might use an explicit flag instead
-		// as I belive there are some cases in which 'this' is
-		// undefined even in a normal function call.
 		return _new;
 	}
 
@@ -253,16 +245,96 @@ private:
 	/// The actual arguments
     Args::container_type _args;
 
-	void readArgs(const as_environment& env, int first_in, size_t nargs)
-	{
-		_args.clear();
-		for (size_t i = 0; i < nargs; ++i) {
-			_args.push_back(env.bottom(first_in - i));
-        }
-	}
-
 };
 
+
+/// Check that the 'this' pointer is a particular type
+//
+/// NB: this function and is deprecated.
+template<typename T>
+struct ThisIs
+{
+    typedef T value_type;
+    value_type* operator()(as_object* o) const {
+        return dynamic_cast<value_type*>(o);
+    }
+};
+
+
+/// Check that the 'this' pointer has a particular native type ('Relay').
+//
+/// This is the most likely of the cases to reflect AS behaviour.
+template<typename T>
+struct ThisIsNative
+{
+    typedef T value_type;
+    value_type* operator()(as_object* o) const {
+        return dynamic_cast<value_type*>(o->relay());
+    }
+};
+
+
+/// Check that the 'this' pointer is a DisplayObject
+//
+/// By default this just checks for any DisplayObject type.
+template<typename T = DisplayObject>
+struct IsDisplayObject
+{
+    typedef T value_type;
+    value_type* operator()(as_object* o) const {
+        if (!o) return 0;
+        return dynamic_cast<T*>(o->displayObject());
+    }
+};
+
+/// Check that the 'this' pointer is not null.
+struct ValidThis
+{
+    typedef as_object value_type;
+    value_type* operator()(as_object* o) const {
+        return o;
+    }
+};
+
+/// Templated function to check the validity of a function call.
+//
+/// It throws an exception if the condition is not fulfilled, it throws
+/// an ActionTypeError, resulting in the function call being aborted and
+/// an undefined as_value returned.
+//
+/// Note that not carrying out a function because the this pointer is
+/// undefined is not ActionScript behaviour in most cases. To avoid
+/// spreading its usage outside AS function implementations, this function
+/// now takes a fn_call as an argument.
+//
+/// @tparam T       A struct defining a value_type and an operator() that
+///                 checks the as_object's validity. A pointer to the
+///                 value_type is returned on success, an exception thrown
+///                 on failure.
+/// @param fn       The function whose 'this' pointer should be checked.
+/// @return         If the cast succeeds, the pointer cast to the
+///                 requested type.
+template<typename T>
+typename T::value_type*
+ensure(const fn_call& fn)
+{
+    as_object* obj = fn.this_ptr;
+    if (!obj) throw ActionTypeError();
+
+    typename T::value_type* ret = T()(obj);
+
+    if (!ret) {
+        std::string target = typeName(ret);
+        std::string source = typeName(obj);
+
+        std::string msg = "Function requiring " + target + " as 'this' "
+            "called from " + source + " instance.";
+
+        throw ActionTypeError(msg);
+    }
+    return ret;
+}
+
 /// Check whether the currently executing code is AS3 (ABC)
 //
 /// This is a non-member, non-friend function for better encapsulation.
@@ -297,10 +369,10 @@ getVM(const fn_call& fn)
     return fn.getVM();
 }
 
-inline Global_as*
+inline Global_as&
 getGlobal(const fn_call& fn)
 {
-    return fn.getVM().getGlobal();
+    return *fn.getVM().getGlobal();
 }
 
 } // namespace gnash
