@@ -1,6 +1,6 @@
 // AudioDecoderNellymoser.cpp: Nellymoser decoding
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -47,14 +47,18 @@
  */
 
 #include "AudioDecoderNellymoser.h"
-#include "utility.h"
+#include "SoundInfo.h"
+#include "log.h"
+
 #include <ctime>
 #include <cmath>
-#include "VM.h"
+#include <boost/random.hpp>
 
 namespace gnash {
 namespace media {
 
+const double sqrt1_2 = 0.70710678118654752440;
+
 float nelly_neg_unpack_table[64] = {
 -0.0061359000, -0.0306748003, -0.0551952012, -0.0796824023, -0.1041216031,
 -0.1284981072, -0.1527972072, -0.1770042032, -0.2011045963, -0.2250839025,
@@ -623,7 +627,8 @@ static void get_sample_bits(float *buf, int *bits)
 	}
 }
 
-static unsigned char get_bits(unsigned char block[NELLY_BLOCK_LEN], int *off, int n)
+static unsigned
+char get_bits(const unsigned char block[NELLY_BLOCK_LEN], int *off, int n)
 {
 	char ret;
 	int boff = *off/8, bitpos = *off%8, mask = (1<<n)-1;
@@ -644,15 +649,19 @@ static int
 gimme_random()
 {
 	using namespace boost;
-	VM::RNG& rnd = VM::get().randomNumberGenerator();
+	
+    typedef boost::mt11213b RNG;
+
+    static RNG rnd;
 
 	uniform_int<> dist(0, std::numeric_limits<int>::max());
-	variate_generator<VM::RNG&, uniform_int<> > uni(rnd, dist);
+	variate_generator<RNG, uniform_int<> > uni(rnd, dist);
 
 	return uni();
 }
 
-static void nelly_decode_block(nelly_handle* nh, unsigned char block[NELLY_BLOCK_LEN], float audio[256])
+static void
+nelly_decode_block(nelly_handle* nh, const unsigned char block[NELLY_BLOCK_LEN], float audio[256])
 {
 	int i,j;
 	float buf[NELLY_BUF_LEN], pows[NELLY_BUF_LEN];
@@ -667,7 +676,7 @@ static void nelly_decode_block(nelly_handle* nh, unsigned char block[NELLY_BLOCK
 	for (i = 0; i < 23; i++) {
 		if (i > 0)
 			val += nelly_delta_table[get_bits(block, &bit_offset, 5)];
-		pval = pow(2, val/2048);
+		pval = std::pow(2, val/2048);
 		for (j = 0; j < nelly_copy_count[i]; j++) {
 			*bptr = val;
 			*pptr = pval;
@@ -688,7 +697,7 @@ static void nelly_decode_block(nelly_handle* nh, unsigned char block[NELLY_BLOCK
 
 		for (j = 0; j < NELLY_FILL_LEN; j++) {
 			if (bits[j] <= 0) {
-				buf[j] = M_SQRT1_2*pows[j];
+				buf[j] = sqrt1_2*pows[j];
 
         
 				if (gimme_random() % 2)
@@ -750,23 +759,32 @@ AudioDecoderNellymoser::AudioDecoderNellymoser()
 }
 
 	
-AudioDecoderNellymoser::AudioDecoderNellymoser(AudioInfo& info)
+AudioDecoderNellymoser::AudioDecoderNellymoser(const AudioInfo& info)
 	:
 	_sampleRate(0),
 	_stereo(false)
 {
     setup(info);
 	_nh = nelly_get_handle();
+
+    assert(info.type == FLASH); // or we'd have thrown an exception
+    audioCodecType codec = (audioCodecType)info.codec;
+  	log_debug(_("AudioDecoderNellymoser: initialized FLASH codec %s (%d)"),
+		(int)codec, codec);
 }
 
 
-AudioDecoderNellymoser::AudioDecoderNellymoser(SoundInfo& info)
+AudioDecoderNellymoser::AudioDecoderNellymoser(const SoundInfo& info)
 	:
 	_sampleRate(0),
 	_stereo(false)
 {
     setup(info);
 	_nh = nelly_get_handle();
+
+    audioCodecType codec = info.getFormat();
+  	log_debug(_("AudioDecoderNellymoser: initialized FLASH codec %s (%d)"),
+		(int)codec, codec);
 }
 
 
@@ -775,38 +793,60 @@ AudioDecoderNellymoser::~AudioDecoderNellymoser()
 	nelly_free_handle(_nh);
 }
 
-void AudioDecoderNellymoser::setup(SoundInfo& info)
+void
+AudioDecoderNellymoser::setup(const SoundInfo& info)
 {
+	audioCodecType codec = info.getFormat();
+    switch (codec)
+    {
+        case AUDIO_CODEC_NELLYMOSER:
+        case AUDIO_CODEC_NELLYMOSER_8HZ_MONO:
+            _sampleRate = info.getSampleRate();
+            _stereo = info.isStereo();
+            break;
 
-	if (info.getFormat() == AUDIO_CODEC_NELLYMOSER ||
-	    info.getFormat() == AUDIO_CODEC_NELLYMOSER_8HZ_MONO) {
-		_sampleRate = info.getSampleRate();
-		_stereo = info.isStereo();
-		return;
-	} 
-	throw MediaException("AudioDecoderNellymoser: attempt to use with "
-		                "non-nellymoser codec");
-}
-
-void AudioDecoderNellymoser::setup(AudioInfo& info)
-{
-	if (info.type == FLASH && (info.codec == AUDIO_CODEC_NELLYMOSER ||
-	        info.codec == AUDIO_CODEC_NELLYMOSER_8HZ_MONO)) {
-		_sampleRate = info.sampleRate;
-		_stereo = info.stereo;
-		return;
+        default:
+            boost::format err = boost::format(
+                _("AudioDecoderNellymoser: attempt to use with flash codec %d (%s)"))
+                % (int)codec % codec;
+            throw MediaException(err.str());
 	}
-
-	throw MediaException("AudioDecoderNellymoser: attempt to use with "
-		                "non-nellymoser codec");
 }
 
-float* AudioDecoderNellymoser::decode(boost::uint8_t* in_buf, boost::uint32_t inputSize, boost::uint32_t* outputSize)
+void AudioDecoderNellymoser::setup(const AudioInfo& info)
 {
-        size_t out_buf_size = (inputSize / NELLY_BLOCK_LEN) * 256;
+	if (info.type != FLASH)
+    {
+        boost::format err = boost::format(
+            _("AudioDecoderNellymoser: unable to intepret custom audio codec id %s"))
+            % info.codec;
+        throw MediaException(err.str());
+    }
+
+	audioCodecType codec = static_cast<audioCodecType>(info.codec);
+    switch (codec)
+    {
+        case AUDIO_CODEC_NELLYMOSER:
+        case AUDIO_CODEC_NELLYMOSER_8HZ_MONO:
+            _sampleRate = info.sampleRate;
+            _stereo = info.stereo;
+            break;
+
+        default:
+            boost::format err = boost::format(
+                _("AudioDecoderNellymoser: attempt to use with flash codec %d (%s)"))
+                % (int)codec % codec;
+            throw MediaException(err.str());
+	}
+}
+
+float*
+AudioDecoderNellymoser::decode(const boost::uint8_t* in_buf, boost::uint32_t inputSize, boost::uint32_t* outputSize)
+{
+    size_t out_buf_size = (inputSize / NELLY_BLOCK_LEN) * 256;
 	float* out_buf = new float[out_buf_size];
 	
-	boost::uint8_t* input_ptr = in_buf;
+	const boost::uint8_t* input_ptr = in_buf;
 	float* output_ptr = out_buf;
 
 	while (inputSize > 0) {
@@ -821,7 +861,10 @@ float* AudioDecoderNellymoser::decode(boost::uint8_t* in_buf, boost::uint32_t in
 	return out_buf;	
 }
 
-boost::uint8_t* AudioDecoderNellymoser::decode(boost::uint8_t* input, boost::uint32_t inputSize, boost::uint32_t& outputSize, boost::uint32_t& decodedBytes, bool /*parse*/)
+boost::uint8_t*
+AudioDecoderNellymoser::decode(const boost::uint8_t* input,
+        boost::uint32_t inputSize, boost::uint32_t& outputSize,
+        boost::uint32_t& decodedBytes, bool /*parse*/)
 {
 
 	float float_buf[256];
