@@ -1,6 +1,6 @@
 // MediaParser.h: Base class for media parsers
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,14 +16,9 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
 #ifndef GNASH_MEDIAPARSER_H
 #define GNASH_MEDIAPARSER_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "IOChannel.h" // for inlines
 #include "dsodefs.h" // DSOEXPORT
 #include "SimpleBuffer.h"
@@ -492,6 +487,13 @@ public:
 		_bufferTime=t;
 	}
 
+	/// Get timestamp of the next frame available, if any
+	//
+	/// @param ts will be set to timestamp of next available frame
+	/// @return false if no frame is available yet
+	///
+	DSOEXPORT bool nextFrameTimestamp(boost::uint64_t& ts) const;
+
 	/// Get timestamp of the video frame which would be returned on nextVideoFrame
 	//
 	/// @return false if there no video frame left
