@@ -1,6 +1,6 @@
 // MediaHandlerFfmpeg.cpp: FFMPEG media handler, for Gnash
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "FLVParser.h"
 #include "VideoConverterFfmpeg.h"
 #include "VideoInputFfmpeg.h"
+#include "AudioInputFfmpeg.h"
 
 #include "IOChannel.h" // for visibility of destructor
 #include "MediaParser.h" // for visibility of destructor
@@ -117,6 +118,12 @@ MediaHandlerFfmpeg::createAudioDecoder(const AudioInfo& info)
 	return ret;
 }
 
+AudioInput*
+MediaHandlerFfmpeg::getAudioInput(size_t /*index*/)
+{
+    return new AudioInputFfmpeg();
+}
+
 VideoInput*
 MediaHandlerFfmpeg::getVideoInput(size_t /*index*/)
 {
