@@ -1,5 +1,5 @@
 //
-//     Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+//     Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,14 +15,9 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA    02110-1301    USA
 
-
 #ifndef GNASH_VIDEOCONVERTERFFMPEG_H
 #define GNASH_VIDEOCONVERTERFFMPEG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "ffmpegHeaders.h"
 #include "VideoConverter.h"
 
