@@ -1,6 +1,6 @@
 // GstUtil.cpp: Generalized Gstreamer utilities for pipeline configuration.
 //
-//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,11 +29,11 @@
 
 #include "swfdec_codec_gst.h"
 
-#ifdef HAVE_GSTREAMER_0_10_GST_PBUTILS_INSTALL_PLUGINS_H
+#ifdef HAVE_GST_PBUTILS_INSTALL_PLUGINS_H
 #include <gst/pbutils/pbutils.h>
 #include <gst/pbutils/missing-plugins.h>
 #include <gst/pbutils/install-plugins.h>
-#endif // HAVE_GSTREAMER_0_10_GST_PBUTILS_INSTALL_PLUGINS_H
+#endif // HAVE_GST_PBUTILS_INSTALL_PLUGINS_H
 
 
 namespace gnash {
@@ -107,7 +107,7 @@ GstUtil::check_missing_plugins(GstCaps* caps)
         return true;
     }
 
-#ifdef HAVE_GSTREAMER_0_10_GST_PBUTILS_INSTALL_PLUGINS_H
+#ifdef HAVE_GST_PBUTILS_INSTALL_PLUGINS_H
     gst_pb_utils_init();
 
 
@@ -140,7 +140,7 @@ GstUtil::check_missing_plugins(GstCaps* caps)
 #else
     log_error(_("Missing plugin, but automatic plugin installation not "
                 "available."));
-#endif  // end of HAVE_GSTREAMER_0_10_GST_PBUTILS_INSTALL_PLUGINS_H
+#endif  // end of HAVE_GST_PBUTILS_INSTALL_PLUGINS_H
 
     return false;
 }
