@@ -1,6 +1,7 @@
 // VideoInputGst.cpp: Video input processing using Gstreamer.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -947,10 +948,13 @@ VideoInputGst::checkForSupportedFramerate(GnashWebcamPrivate *webcam,
         int fps) 
 {
     
+    if (!webcam) {
+	log_error("%s: webam isn't set!", __FUNCTION__);
+    }
     for (int i = 0; i < webcam->_currentFormat->numFramerates; ++i) {
-        int val = std::ceil(
+        int val = std::ceil(static_cast<double>(
                 webcam->_currentFormat->framerates[i].numerator /
-               webcam->_currentFormat->framerates[i].denominator);
+               webcam->_currentFormat->framerates[i].denominator));
         if (val == fps) {
             return true;
         }
