@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -69,7 +69,10 @@ CQue::wait()
 {
 //    GNASH_REPORT_FUNCTION;
     boost::mutex::scoped_lock lk(_cond_mutex);
+#ifndef _WIN32
     _cond.wait(lk);
+    log_unimpl("CQue::wait(win32)");
+#endif
 //    log_debug("wait mutex released for \"%s\"", _name);
 }
 
@@ -78,7 +81,10 @@ void
 CQue::notify()
 {
 //    GNASH_REPORT_FUNCTION;
+#ifndef _WIN32
     _cond.notify_one();
+    log_unimpl("CQue::notify(win32)");
+#endif
 //    log_debug("wait mutex triggered for \"%s\"", _name);
 }
 
