@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,6 +41,7 @@
 # include <io.h>
 # include <ws2tcpip.h>
 #else
+# include <sys/ioctl.h>
 # include <sys/time.h>
 # include <unistd.h>
 # include <sys/select.h>
@@ -50,7 +52,11 @@
 # include <netdb.h>
 # include <sys/param.h>
 # include <sys/select.h>
+#include <csignal>
+
+// This is for non-standard signal functions such as sigemptyset.
 #include <signal.h>
+
 #ifdef HAVE_POLL_H
 # include <poll.h>
 #else 
@@ -67,6 +73,10 @@
 #define MAXHOSTNAMELEN 256
 #endif
 
+#ifndef FIONREAD
+#define FIONREAD 0
+#endif
+
 using namespace std;
 
 /// \namespace gnash
@@ -101,7 +111,7 @@ Network::Network()
     wVersionRequested = MAKEWORD(1, 1);		// Windows Sockets 1.1
     if (WSAStartup( wVersionRequested, &wsaData ) != 0) {
         log_error(_("Could not find a usable WinSock DLL"));
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 #endif
 
@@ -348,13 +358,13 @@ Network::newConnection(bool block, int fd)
         }
 
         if (ret == -1) {
-            log_debug(_("The accept() socket for fd #%d never was available for writing"), fd);
+            log_debug(_("The accept() socket for fd #%d never was available"), fd);
             return -1;
         }
 
         if (ret == 0) {
             if (_debug) {
-                log_debug(_("The accept() socket for fd #%d timed out waiting to write"), fd);
+                log_debug(_("The accept() socket for fd #%d timed out waitingfor data"), fd);
 		return 0;
             }
         }
@@ -787,6 +797,18 @@ Network::readNet(int fd, amf::Buffer &buffer)
     return ret;
 }
 
+int
+Network::readNet(int fd, amf::Buffer *buffer)
+{
+//    GNASH_REPORT_FUNCTION;
+    int ret = readNet(fd, buffer->reference(), buffer->size(), _timeout);
+    if (ret > 0) {
+	buffer->setSeekPointer(buffer->reference() + ret);
+    }
+
+    return ret;
+}
+
 int
 Network::readNet(amf::Buffer &buffer)
 {
@@ -978,14 +1000,14 @@ Network::writeNet(amf::Buffer *buffer)
 {
 //     GNASH_REPORT_FUNCTION;
     return writeNet(buffer->reference(), buffer->allocated());
-};
+}
 
 int
 Network::writeNet(int fd, amf::Buffer *buffer)
 {
 //     GNASH_REPORT_FUNCTION;
     return writeNet(fd, buffer->reference(), buffer->allocated());
-};
+}
 
 // Write to the connection
 int
@@ -993,7 +1015,7 @@ Network::writeNet(amf::Buffer &buffer)
 {
 //     GNASH_REPORT_FUNCTION;
     return writeNet(buffer.reference(), buffer.allocated());
-};
+}
 
 // Write to the connection
 int
@@ -1001,7 +1023,7 @@ Network::writeNet(int fd, amf::Buffer &buffer)
 {
 //     GNASH_REPORT_FUNCTION;
     return writeNet(fd, buffer.reference(), buffer.allocated());
-};
+}
 
 int
 Network::writeNet(const std::string& data)
@@ -1201,7 +1223,7 @@ Network::getPollFDPtr()
 //    GNASH_REPORT_FUNCTION;
     boost::mutex::scoped_lock lock(_poll_mutex);
     return &_pollfds[0];
-};
+}
 
 void
 Network::erasePollFD(int fd)
@@ -1251,7 +1273,7 @@ Network::getEntry(int fd)
 boost::shared_ptr<std::vector<struct pollfd> >
 Network::waitForNetData(int limit, struct pollfd *fds)
 {
-  //    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
 
     boost::shared_ptr<vector<struct pollfd> > hits(new vector<struct pollfd>);
 
@@ -1366,19 +1388,22 @@ Network::waitForNetData(vector<int> &data)
 {
     // GNASH_REPORT_FUNCTION;
 
-    int max = 0;
-    
     fd_set fdset;
     FD_ZERO(&fdset);
-    
-    for (size_t i = 0; i<data.size(); i++) {
-	FD_SET(data[i], &fdset);
-	if (data[i] > max) {
-	    max = data[i];
+	
+    if (data.size()) {
+	int max = 0;
+	
+	for (size_t i = 0; i<data.size(); i++) {
+	    FD_SET(data[i], &fdset);
+	    if (data[i] > max) {
+		max = data[i];
+	    }
 	}
+	return waitForNetData(max+1, fdset);
     }
 
-    return waitForNetData(max+1, fdset);
+    return fdset;
 }
 
 fd_set
@@ -1393,15 +1418,15 @@ Network::waitForNetData(int limit, fd_set files)
     // Reset the timeout value, since select modifies it on return
     int timeout = _timeout;
     if (timeout <= 0) {
-	timeout = 5;
+	timeout = 30;
     }
-#ifdef HAVE_PSELECT
+#ifdef HAVE_PSELECT_XX
     struct timespec tval;
     sigset_t pending, sigmask;
     sigprocmask(SIG_BLOCK, &sigmask, NULL);
 
     tval.tv_sec = 0;
-    tval.tv_nsec = timeout * 1000;
+    tval.tv_nsec = timeout * 1000000000;
     int ret = pselect(limit+1, &fdset, NULL, NULL, &tval, &sigmask);
     sigpending(&pending);
     if (sigismember(&pending, SIGINT)) {
@@ -1417,9 +1442,8 @@ Network::waitForNetData(int limit, fd_set files)
 #else
     struct timeval tval;
     tval.tv_sec = 0;
-    tval.tv_usec = timeout;
+    tval.tv_usec = timeout * 1000; // was 1000000
     int ret = select(limit+1, &fdset, NULL, NULL, &tval);
-    FD_ZERO(&fdset);
 #endif
     // If interupted by a system call, try again
     if (ret == -1 && errno == EINTR) {
@@ -1428,14 +1452,23 @@ Network::waitForNetData(int limit, fd_set files)
     
     if (ret == -1) {
 	log_error (_("Waiting for data for fdset, was never available for reading"));
+	FD_ZERO(&fdset);
+	FD_SET(0, &fdset);
+	return fdset;
     }
     
     if (ret == 0) {
 	// log_debug (_("Waiting for data for fdset, timed out waiting for data"));
 	FD_ZERO(&fdset);
+	FD_SET(0, &fdset);
+	return fdset;
     }
 
-    if (ret) {
+    if (ret < 0) {
+	log_error("select() got an error: %s.", strerror(errno));
+	FD_ZERO(&fdset);
+	FD_SET(0, &fdset);
+    } else {
 	log_network("select() saw activity on %d file descriptors.", ret);
     }
 
@@ -1561,13 +1594,43 @@ Network::initSSL(std::string &hostname, std::string &passwd,
 }
 #endif
 
+// Use an ioctl() to see how many bytes are in the network buffers.
+size_t
+Network::sniffBytesReady(int fd)
+{
+    // GNASH_REPORT_FUNCTION;
+
+    int bytes = 0;
+    fd_set fdset;
+
+    FD_SET(fd, &fdset);
+
+    struct timeval tval;
+    tval.tv_sec = 0;
+    tval.tv_usec = 10;
+    if (select(fd+1, &fdset, NULL, NULL, &tval)) {
+ 	if (FD_ISSET(fd, &fdset)) {
+#ifndef _WIN32
+	    ioctl(fd, FIONREAD, &bytes);
+#else
+	    ioctlSocket(fd, FIONREAD, &bytes);
+#endif
+	}
+    }
+
+    log_network("#%d bytes waiting in kernel network buffer.", bytes);
+    
+    return bytes;
+}
+
 // Trap Control-C so we can cleanly exit
 static void
 cntrlc_handler (int sig)
 {
+    GNASH_REPORT_FUNCTION;
     sig_number = sig;
     log_debug(_("Got an %d interrupt while blocked on pselect()"), sig);
-    exit(-1);
+    exit(EXIT_FAILURE);
 }
 
 } // end of gnash namespace
