@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -137,6 +138,7 @@ public:
 	int netfd;
 	void *entry;
 	void *handler;
+	amf::Buffer *buffer;
 	std::string filespec;
 	protocols_supported_e protocol;
     } thread_params_t;
@@ -205,6 +207,7 @@ public:
     boost::shared_ptr<amf::Buffer> readNet();
     int readNet(amf::Buffer &buffer);
     int readNet(int fd, amf::Buffer &buffer);
+    int readNet(int fd, amf::Buffer *buffer);
     int readNet(amf::Buffer &buffer, int timeout);
     int readNet(int fd, amf::Buffer &buffer, int timeout);
     int readNet(byte_t *data, int nbytes);
@@ -320,6 +323,9 @@ public:
 		 std::string &keyfile, std::string &calist,
 		 std::string &rootpath, bool auth);
 #endif
+
+    // Use an ioctl() to see how many bytes are in the network buffers.
+    size_t sniffBytesReady(int fd);
     
  protected:
     in_addr_t   _ipaddr;
