@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,7 +20,6 @@
 #include "gnashconfig.h"
 #endif
 
-#include "gnash.h" // for point !!
 #include "Geometry.h" // for edge
 #include <iostream>
 #include <sstream>
