@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -201,11 +201,11 @@ main(int argc, char** argv)
   check(mo, "TextField.prototype.hasOwnProperty('backgroundColor')");
   check(mo, "TextField.prototype.hasOwnProperty('text')");
   check(mo, "TextField.prototype.hasOwnProperty('textColor')");
-  xcheck(mo, "!TextField.prototype.hasOwnProperty('_parent')");
-  xcheck(mo, "!TextField.prototype.hasOwnProperty('_xmouse')");
-  xcheck(mo, "!TextField.prototype.hasOwnProperty('_ymouse')");
-  xcheck(mo, "!TextField.prototype.hasOwnProperty('_xscale')");
-  xcheck(mo, "!TextField.prototype.hasOwnProperty('_yscale')");
+  check(mo, "!TextField.prototype.hasOwnProperty('_parent')");
+  check(mo, "!TextField.prototype.hasOwnProperty('_xmouse')");
+  check(mo, "!TextField.prototype.hasOwnProperty('_ymouse')");
+  check(mo, "!TextField.prototype.hasOwnProperty('_xscale')");
+  check(mo, "!TextField.prototype.hasOwnProperty('_yscale')");
   
   check_equals(mo, "typeof(dtext1)", "'object'");
   check_equals(mo, "typeof(dtext1.text)", "'string'");
