@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 #include "MovieTester.h"
 #include "MovieClip.h"
-#include "character.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 #include "VM.h"
@@ -49,7 +49,7 @@ main(int /*argc*/, char** /*argv*/)
 	as_value tmp;
 
 	check_equals(root->get_frame_count(), 5);
-	check_equals(root->get_play_state(), MovieClip::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 0);
 
 	check(! tester.findDisplayItemByDepth(*root, 70) );
@@ -81,23 +81,23 @@ main(int /*argc*/, char** /*argv*/)
 
 	string_table& st = VM::get().getStringTable();
 
-	root->get_member(st.find("mousedown"), &tmp);
+	getObject(root)->get_member(st.find("mousedown"), &tmp);
 	check(tmp.is_undefined());
-	root->get_member(st.find("mouseup"), &tmp);
+	getObject(root)->get_member(st.find("mouseup"), &tmp);
 	check(tmp.is_undefined());
 
 	// Note that we are *not* on an active entity !
 	tester.pressMouseButton();
 
-	root->get_member(st.find("mousedown"), &tmp);
+	getObject(root)->get_member(st.find("mousedown"), &tmp);
 	check_equals(tmp.to_number(), 1);
-	check ( ! root->get_member(st.find("mouseup"), &tmp) );
+	check ( ! getObject(root)->get_member(st.find("mouseup"), &tmp) );
 
 	tester.depressMouseButton();
 
-	root->get_member(st.find("mousedown"), &tmp);
+	getObject(root)->get_member(st.find("mousedown"), &tmp);
 	check_equals(tmp.to_number(), 1);
-	root->get_member(st.find("mouseup"), &tmp);
+	getObject(root)->get_member(st.find("mouseup"), &tmp);
 	check_equals(tmp.to_number(), 1);
 
 	tester.advance();
@@ -154,7 +154,7 @@ main(int /*argc*/, char** /*argv*/)
 	// Note that we are *not* on an active entity !
 	tester.pressMouseButton();
 
-	root->get_member(st.find("mousedown"), &tmp);
+	getObject(root)->get_member(st.find("mousedown"), &tmp);
 	check_equals(tmp.to_number(), 5);
 
 }
