@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 #include "MovieTester.h"
 #include "MovieClip.h"
-#include "character.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 #include "GnashException.h"
@@ -80,7 +80,7 @@ main(int /*argc*/, char** /*argv*/)
 	{
 		check_equals(root->get_current_frame(), i%framecount);
 
-		// Out of any character
+		// Out of any DisplayObject
 		check_pixel(317, 430, 2, white, 2);
 
 		// Fully on the red square: 317,330
@@ -104,7 +104,7 @@ main(int /*argc*/, char** /*argv*/)
 
 		// Let's break if we stopped, as we'll print totals() thus
 		// enlarging invalidated bounds too ...
-		if (root->get_play_state() == MovieClip::STOP) break;
+		if (root->getPlayState() == MovieClip::PLAYSTATE_STOP) break;
 
 		// Compute 1-based currentFrame
 		size_t currentFrame = root->get_current_frame()+1;
@@ -113,7 +113,7 @@ main(int /*argc*/, char** /*argv*/)
 		{
 			blackOverRed = !blackOverRed;
 			
-			// Check the intersection of the two characters to
+			// Check the intersection of the two DisplayObjects to
 			// be invalidated
 			check( tester.getInvalidatedRanges().contains(343, 330) );
 		}
