@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #define INPUT_FILENAME "loop_test2.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 
@@ -56,30 +56,30 @@ main(int /*argc*/, char** /*argv*/)
 	Range2d<int> blackRange(330,270,450,390);
 
 	// Coordinates we'll use during testing
-	int x_left = 270; // on the left of any character
+	int x_left = 270; // on the left of any DisplayObject
 	int x_red = 310; // on the red square
 	int x_int = 340; // on the intersection between the red and black squares
 	int x_black = 370; // on black square
-	int x_right = 460; // on the right of any character
+	int x_right = 460; // on the right of any DisplayObject
 	int y = 330;
 
 	Ranges invalidated;
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	// FRAME 1 (start)
 
 	check_equals(root->get_frame_count(), 4);
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 0);
 	check_equals(root->getDisplayList().size(), 1);  // dejagnu clip
 	invalidated = tester.getInvalidatedRanges();
 	check( invalidated.contains(76, 4) ); // the "-xtrace enabled-" label...
 
-	tester.advance(); // FRAME 2, place characters (black on top)
+	tester.advance(); // FRAME 2, place DisplayObjects (black on top)
 	invalidated = tester.getInvalidatedRanges();
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 1);
 	check_equals(root->getDisplayList().size(), 3); // dejagnu + red square + black square
 
@@ -94,15 +94,15 @@ main(int /*argc*/, char** /*argv*/)
 	check_pixel(x_black, y, 2, black, 2);  
 	check_pixel(x_right, y, 2, white, 2);  
 
-	tester.advance(); // FRAME 3, depth-swap the two characters
+	tester.advance(); // FRAME 3, depth-swap the two DisplayObjects
 	invalidated = tester.getInvalidatedRanges();
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 2);
 	check_equals(root->getDisplayList().size(), 3); // dejagnu + red square + black square
 
 	// check invalidated bounds to contain the intersection
-	// between the two characters.
+	// between the two DisplayObjects.
 	//
 	check( invalidated.contains(Intersection(redRange,blackRange)) );
 	
@@ -116,7 +116,7 @@ main(int /*argc*/, char** /*argv*/)
 	tester.advance(); // FRAME 4, jump to frame 2 and stop
 	invalidated = tester.getInvalidatedRanges();
 
-	check_equals(root->get_play_state(), sprite_instance::STOP);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
 	check_equals(root->get_current_frame(), 1);
 	check_equals(root->getDisplayList().size(), 3); // dejagnu + red square + black square
 
