@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
  */ 
 
 #ifndef GNASH_MING_UTILS_H
@@ -48,6 +47,18 @@ void SWFMovie_writeExports(SWFMovie movie);
 # define newSWFAction compileSWFActionCode
 #endif
 
+/*
+ * 'callFrame' was drop as a keyword since Ming-0.4.0.beta5
+ * and replaced by 'call'. Before that version 'call' would
+ * never be recognized as a "callframe" action
+ */
+#if MING_VERSION_CODE >= 00040006
+# define CALLFRAME "call"
+#else
+# define CALLFRAME "callFrame"
+#endif
+
+
 /* Missing define to allow using older Ming releases */
 
 #ifndef SWFACTION_INIT
